@@ -1,6 +1,6 @@
 /* editor.h
  *
- * Copyright 2002-2006 Vesa Halttunen
+ * Copyright 2002-2011 Vesa Halttunen
  *
  * This file is part of Tutka.
  *
@@ -46,7 +46,8 @@ enum {
 enum {
   EXTERNAL_SYNC_OFF,
   EXTERNAL_SYNC_MIDI,
-  EXTERNAL_SYNC_JACK
+  EXTERNAL_SYNC_JACK,
+  EXTERNAL_SYNC_JACK_START_ONLY
 };
 
 struct editor {
@@ -143,6 +144,8 @@ void editor_song_block_current_insert_line(struct editor *, unsigned int);
 void editor_song_block_current_insert_line_track_all(struct editor *);
 void editor_song_block_current_delete_line(struct editor *, unsigned int);
 void editor_song_block_current_delete_line_track_all(struct editor *);
+void editor_song_block_current_insert_track(struct editor *, unsigned int);
+void editor_song_block_current_delete_track(struct editor *, unsigned int);
 void editor_song_track_volume_set(struct editor *, unsigned int, unsigned int);
 unsigned int editor_song_track_volume_get(struct editor *, unsigned int);
 void editor_song_track_mute_set(struct editor *, unsigned int, unsigned int);
@@ -237,7 +240,7 @@ unsigned int editor_scheduler_get(struct editor *);
 /* Returns the tracker font name */
 char *editor_tracker_font_get(struct editor *);
 /* Enables or disables a MIDI interface by name */
-unsigned int editor_midi_interface_status_set(struct editor *, unsigned int, unsigned int, unsigned int, unsigned int);
+unsigned int editor_midi_interface_status_set(struct editor *, char *, unsigned int, unsigned int);
 /* Returns the name of the MIDI interface to use */
 char *editor_midi_interface_get(struct editor *, unsigned int, unsigned int);
 /* Marks the enabled interfaces as enabled in the preferences */
