@@ -1,5 +1,9 @@
-/*
- * The Real SoundTracker - GTK+ Tracker widget (header)
+/* trackerwidget.h
+ *
+ * Copyright 2002-2011 Vesa Halttunen
+ *
+ * This file is part of Tutka. The code in this file is based on
+ * The Real SoundTracker - GTK+ Tracker widget
  *
  * Copyright (C) 1998-2001 Michael Krause
  *
@@ -17,7 +21,6 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
-
 #ifndef _TRACKER_H
 #define _TRACKER_H
 
@@ -63,7 +66,6 @@ struct _Tracker {
 
   GdkGC *bg_gc, *bg_cursor_gc, *notes_gc, *misc_gc;
   GdkColor colors[TRACKERCOL_LAST];
-  int enable_backing_store;
   GdkPixmap *pixmap;
   guint idle_handler;
 
@@ -92,12 +94,12 @@ struct _Tracker {
 struct _TrackerClass {
   GtkWidgetClass parent_class;
 
-  void (*patpos) (gpointer, int, int, int, gpointer);
-  void (*xpanning) (gpointer, int, int, int, gpointer);
-  void (*blockmark_set) (gpointer, int, gpointer);
+  void (*patpos)(gpointer, int, int, int, gpointer);
+  void (*xpanning)(gpointer, int, int, int, gpointer);
+  void (*blockmark_set)(gpointer, int, gpointer);
 };
 
-extern const char *const notenames[128];
+extern const char * const notenames[128];
 
 GType tracker_get_type(void);
 GtkWidget *tracker_new(void);
@@ -105,7 +107,6 @@ GtkWidget *tracker_new(void);
 void tracker_set_cmdpage(Tracker * t, int);
 void tracker_set_num_channels(Tracker * t, int);
 
-void tracker_set_backing_store(Tracker * t, int on);
 gboolean tracker_set_font(Tracker * t, const gchar * fontname);
 
 void tracker_reset(Tracker * t);
