@@ -1,6 +1,6 @@
 /* block.c
  *
- * Copyright 2002-2006 Vesa Halttunen
+ * Copyright 2002-2011 Vesa Halttunen
  *
  * This file is part of Tutka.
  *
@@ -42,18 +42,12 @@ struct block *block_alloc(int tracks, int length, int commandpages)
 /* Frees a block */
 void block_free(struct block *block)
 {
-  nullcheck_void(block, block_free);
-
-  if (block->name)
+  if (block != NULL) {
     free(block->name);
-
-  if (block->notes)
     free(block->notes);
-
-  if (block->commands)
     free(block->commands);
-
-  free(block);
+    free(block);
+  }
 }
 
 /* Sets a note in a block */
@@ -136,16 +130,14 @@ void block_set_tracks(struct block *block, unsigned int tracks)
       notes[(l * tracks + t) * 2] = block->notes[(l * nt + t) * 2];
       notes[(l * tracks + t) * 2 + 1] = block->notes[(l * nt + t) * 2 + 1];
       for (ep = 0; ep < block->commandpages; ep++) {
-	commands[ep * 2 * tracks * nl + (l * tracks + t) * 2] = block->commands[ep * 2 * nt * nl + (l * nt + t) * 2];
-	commands[ep * 2 * tracks * nl + (l * tracks + t) * 2 + 1] = block->commands[ep * 2 * nt * nl + (l * nt + t) * 2 + 1];
+        commands[ep * 2 * tracks * nl + (l * tracks + t) * 2] = block->commands[ep * 2 * nt * nl + (l * nt + t) * 2];
+        commands[ep * 2 * tracks * nl + (l * tracks + t) * 2 + 1] = block->commands[ep * 2 * nt * nl + (l * nt + t) * 2 + 1];
       }
     }
 
   /* Free old arrays */
-  if (block->notes)
-    free(block->notes);
-  if (block->commands)
-    free(block->commands);
+  free(block->notes);
+  free(block->commands);
 
   /* Use new arrays */
   block->notes = notes;
@@ -180,16 +172,14 @@ void block_set_length(struct block *block, unsigned int length)
       notes[(l * nt + t) * 2] = block->notes[(l * nt + t) * 2];
       notes[(l * nt + t) * 2 + 1] = block->notes[(l * nt + t) * 2 + 1];
       for (ep = 0; ep < block->commandpages; ep++) {
-	commands[ep * 2 * nt * length + (l * nt + t) * 2] = block->commands[ep * 2 * nt * nl + (l * nt + t) * 2];
-	commands[ep * 2 * nt * length + (l * nt + t) * 2 + 1] = block->commands[ep * 2 * nt * nl + (l * nt + t) * 2 + 1];
+        commands[ep * 2 * nt * length + (l * nt + t) * 2] = block->commands[ep * 2 * nt * nl + (l * nt + t) * 2];
+        commands[ep * 2 * nt * length + (l * nt + t) * 2 + 1] = block->commands[ep * 2 * nt * nl + (l * nt + t) * 2 + 1];
       }
     }
 
   /* Free old arrays */
-  if (block->notes)
-    free(block->notes);
-  if (block->commands)
-    free(block->commands);
+  free(block->notes);
+  free(block->commands);
 
   /* Use new arrays */
   block->notes = notes;
@@ -218,8 +208,7 @@ void block_set_commandpages(struct block *block, unsigned int commandpages)
   memcpy(commands, block->commands, existingcommandpages * 2 * block->tracks * block->length);
 
   /* Free old array */
-  if (block->commands)
-    free(block->commands);
+  free(block->commands);
 
   /* Use new array */
   block->commands = commands;
@@ -231,10 +220,9 @@ void block_set_name(struct block *block, char *name)
 {
   nullcheck_void(block, block_set_name);
 
-  if (block->name)
-    free(block->name);
+  free(block->name);
 
-  block->name = (char *)strdup(name);  
+  block->name = (char *)strdup(name);
 }
 
 /* Copies a part of a block to a new block */
@@ -274,7 +262,7 @@ struct block *block_copy(struct block *block, int starttrack, int startline, int
     startline = endline;
     endline = l;
   }
-    
+
   /* Allocate new block */
   newblock = block_alloc(endtrack - starttrack + 1, endline - startline + 1, block->commandpages);
   onl = block->length;
@@ -288,8 +276,8 @@ struct block *block_copy(struct block *block, int starttrack, int startline, int
       newblock->notes[((l - startline) * nnt + (t - starttrack)) * 2] = block->notes[(l * ont + t) * 2];
       newblock->notes[((l - startline) * nnt + (t - starttrack)) * 2 + 1] = block->notes[(l * ont + t) * 2 + 1];
       for (ep = 0; ep < block->commandpages; ep++) {
-	newblock->commands[ep * 2 * nnt * nnl + ((l - startline) * nnt + (t - starttrack)) * 2] = block->commands[ep * 2 * ont * onl + (l * ont + t) * 2];
-	newblock->commands[ep * 2 * nnt * nnl + ((l - startline) * nnt + (t - starttrack)) * 2 + 1] = block->commands[ep * 2 * ont * onl + (l * ont + t) * 2 + 1];
+        newblock->commands[ep * 2 * nnt * nnl + ((l - startline) * nnt + (t - starttrack)) * 2] = block->commands[ep * 2 * ont * onl + (l * ont + t) * 2];
+        newblock->commands[ep * 2 * nnt * nnl + ((l - startline) * nnt + (t - starttrack)) * 2 + 1] = block->commands[ep * 2 * ont * onl + (l * ont + t) * 2 + 1];
       }
     }
   }
@@ -328,8 +316,8 @@ void block_paste(struct block *block, struct block *from, int track, int line)
       block->notes[((line + l) * nt + (track + t)) * 2] = from->notes[(l * fnt + t) * 2];
       block->notes[((line + l) * nt + (track + t)) * 2 + 1] = from->notes[(l * fnt + t) * 2 + 1];
       for (ep = 0; ep < cnc; ep++) {
-	block->commands[ep * 2 * nt * nl + ((line + l) * nt + (track + t)) * 2] = from->commands[ep * 2 * fnt * fnl + (l * fnt + t) * 2];
-	block->commands[ep * 2 * nt * nl + ((line + l) * nt + (track + t)) * 2 + 1] = from->commands[ep * 2 * fnt * fnl + (l * fnt + t) * 2 + 1];
+        block->commands[ep * 2 * nt * nl + ((line + l) * nt + (track + t)) * 2] = from->commands[ep * 2 * fnt * fnl + (l * fnt + t) * 2];
+        block->commands[ep * 2 * nt * nl + ((line + l) * nt + (track + t)) * 2 + 1] = from->commands[ep * 2 * fnt * fnl + (l * fnt + t) * 2 + 1];
       }
     }
   }
@@ -371,7 +359,7 @@ void block_clear(struct block *block, int starttrack, int startline, int endtrac
     startline = endline;
     endline = l;
   }
-    
+
   nl = block->length;
   nt = block->tracks;
 
@@ -380,8 +368,8 @@ void block_clear(struct block *block, int starttrack, int startline, int endtrac
       block->notes[(l * nt + t) * 2] = 0;
       block->notes[(l * nt + t) * 2 + 1] = 0;
       for (ep = 0; ep < block->commandpages; ep++) {
-	block->commands[ep * 2 * nt * nl + (l * nt + t) * 2] = 0;
-	block->commands[ep * 2 * nt * nl + (l * nt + t) * 2 + 1] = 0;
+        block->commands[ep * 2 * nt * nl + (l * nt + t) * 2] = 0;
+        block->commands[ep * 2 * nt * nl + (l * nt + t) * 2 + 1] = 0;
       }
     }
   }
@@ -392,74 +380,105 @@ struct block *block_parse(xmlDocPtr doc, xmlNsPtr ns, xmlNodePtr cur)
 {
   struct block *block = NULL;
 
-  if ((!xmlStrcmp(cur->name, "block")) && (cur->ns == ns)) {
-    char *prop;
+  if ((!xmlStrcmp(cur->name, (xmlChar *)"block")) && (cur->ns == ns)) {
+    xmlChar *prop;
     int tracks = 4, length = 64, commandpages = 1;
 
     /* Get block properties */
-    prop = xmlGetProp(cur, "tracks");
-    if (prop != NULL)
-      tracks = atoi(prop);
-    prop = xmlGetProp(cur, "length");
-    if (prop != NULL)
-      length = atoi(prop);
-    prop = xmlGetProp(cur, "commandpages");
-    if (prop != NULL)
-      commandpages = atoi(prop);
+    prop = xmlGetProp(cur, (xmlChar *)"tracks");
+    if (prop != NULL) {
+      tracks = atoi((char *)prop);
+      xmlFree(prop);
+    }
+
+    prop = xmlGetProp(cur, (xmlChar *)"length");
+    if (prop != NULL) {
+      length = atoi((char *)prop);
+      xmlFree(prop);
+    }
+
+    prop = xmlGetProp(cur, (xmlChar *)"commandpages");
+    if (prop != NULL) {
+      commandpages = atoi((char *)prop);
+      xmlFree(prop);
+    }
 
     /* Allocate block */
     block = block_alloc(tracks, length, commandpages);
-    block->name = xmlGetProp(cur, "name");
+    prop = xmlGetProp(cur, (xmlChar *)"name");
+    if (prop != NULL) {
+      block->name = strdup((char *)prop);
+      xmlFree(prop);
+    }
 
     /* Get block contents */
     cur = cur->xmlChildrenNode;
     while (cur != NULL) {
       xmlNodePtr temp;
-      if ((!xmlStrcmp(cur->name, "note")) && (cur->ns == ns)) {
-	int line = 0, track = 0, note = 0, instrument = 0;
+      if ((!xmlStrcmp(cur->name, (xmlChar *)"note")) && (cur->ns == ns)) {
+        int line = 0, track = 0, note = 0, instrument = 0;
 
-	/* Get note properties */
-	prop = xmlGetProp(cur, "line");
-	if (prop != NULL)
-	  line = atoi(prop);
-	prop = xmlGetProp(cur, "track");
-	if (prop != NULL)
-	  track = atoi(prop);
-	prop = xmlGetProp(cur, "instrument");
-	if (prop != NULL)
-	  instrument = atoi(prop);
+        /* Get note properties */
+        prop = xmlGetProp(cur, (xmlChar *)"line");
+        if (prop != NULL) {
+          line = atoi((char *)prop);
+          xmlFree(prop);
+        }
 
-	/* Get the note */
-	temp = cur->xmlChildrenNode;
-	if (temp != NULL)
-	  note = atoi(temp->content);
+        prop = xmlGetProp(cur, (xmlChar *)"track");
+        if (prop != NULL) {
+          track = atoi((char *)prop);
+          xmlFree(prop);
+        }
 
-	/* Set the note */
-	block_set_note(block, line, track, 0, note, instrument);
-      } else if ((!xmlStrcmp(cur->name, "command")) && (cur->ns == ns)) {
-	int line = 0, track = 0, commandpage = 0, command = 0, value = 0;
+        prop = xmlGetProp(cur, (xmlChar *)"instrument");
+        if (prop != NULL) {
+          instrument = atoi((char *)prop);
+          xmlFree(prop);
+        }
 
-	/* Get command properties */
-	prop = xmlGetProp(cur, "line");
-	if (prop != NULL)
-	  line = atoi(prop);
-	prop = xmlGetProp(cur, "track");
-	if (prop != NULL)
-	  track = atoi(prop);
-	prop = xmlGetProp(cur, "commandpage");
-	if (prop != NULL)
-	  commandpage = atoi(prop);
-	prop = xmlGetProp(cur, "value");
-	if (prop != NULL)
-	  value = atoi(prop);
+        /* Get the note */
+        temp = cur->xmlChildrenNode;
+        if (temp != NULL)
+          note = atoi((char *)temp->content);
 
-	/* Get the note */
-	temp = cur->xmlChildrenNode;
-	if (temp != NULL)
-	  command = atoi(temp->content);
+        /* Set the note */
+        block_set_note(block, line, track, 0, note, instrument);
+      } else if ((!xmlStrcmp(cur->name, (xmlChar *)"command")) && (cur->ns == ns)) {
+        int line = 0, track = 0, commandpage = 0, command = 0, value = 0;
 
-	/* Set the command */
-	block_set_command_full(block, line, track, commandpage, command, value);
+        /* Get command properties */
+        prop = xmlGetProp(cur, (xmlChar *)"line");
+        if (prop != NULL) {
+          line = atoi((char *)prop);
+          xmlFree(prop);
+        }
+
+        prop = xmlGetProp(cur, (xmlChar *)"track");
+        if (prop != NULL) {
+          track = atoi((char *)prop);
+          xmlFree(prop);
+        }
+
+        prop = xmlGetProp(cur, (xmlChar *)"commandpage");
+        if (prop != NULL) {
+          commandpage = atoi((char *)prop);
+          xmlFree(prop);
+        }
+
+        prop = xmlGetProp(cur, (xmlChar *)"value");
+        if (prop != NULL) {
+          value = atoi((char *)prop);
+          xmlFree(prop);
+        }
+
+        /* Get the note */
+        temp = cur->xmlChildrenNode;
+        if (temp != NULL)
+          command = atoi((char *)temp->content);
+
+        /* Set the command */
+        block_set_command_full(block, line, track, commandpage, command, value);
       }
       cur = cur->next;
     }
@@ -474,57 +493,57 @@ void block_save(struct block *block, int number, xmlNodePtr parent)
 {
   nullcheck_void(block, block_save);
 
-  char c[10];
+  xmlChar c[10];
   int i, j, k;
   xmlNodePtr node, subnode;
 
   /* Set block properties */
-  node = xmlNewChild(parent, NULL, "block", NULL);
-  snprintf(c, 10, "%d", number);
-  xmlSetProp(node, "number", c);
-  xmlSetProp(node, "name", block->name);
-  snprintf(c, 10, "%d", block->tracks);
-  xmlSetProp(node, "tracks", c);
-  snprintf(c, 10, "%d", block->length);
-  xmlSetProp(node, "length", c);
-  snprintf(c, 10, "%d", block->commandpages);
-  xmlSetProp(node, "commandpages", c);
-  xmlAddChild(node, xmlNewText("\n"));
+  node = xmlNewChild(parent, NULL, (xmlChar *)"block", NULL);
+  snprintf((char *)c, 10, "%d", number);
+  xmlSetProp(node, (xmlChar *)"number", c);
+  xmlSetProp(node, (xmlChar *)"name", (xmlChar *)block->name);
+  snprintf((char *)c, 10, "%d", block->tracks);
+  xmlSetProp(node, (xmlChar *)"tracks", c);
+  snprintf((char *)c, 10, "%d", block->length);
+  xmlSetProp(node, (xmlChar *)"length", c);
+  snprintf((char *)c, 10, "%d", block->commandpages);
+  xmlSetProp(node, (xmlChar *)"commandpages", c);
+  xmlAddChild(node, xmlNewText((xmlChar *)"\n"));
 
   /* Notation data */
   for (i = 0; i < block->tracks; i++)
     for (j = 0; j < block->length; j++)
       if (block->notes[2 * (block->tracks * j + i)] != 0 || block->notes[2 * (block->tracks * j + i) + 1] != 0) {
-	snprintf(c, 10, "%d", block->notes[2 * (block->tracks * j + i)]);
-	subnode = xmlNewChild(node, NULL, "note", c);
-	snprintf(c, 10, "%d", j);
-	xmlSetProp(subnode, "line", c);
-	snprintf(c, 10, "%d", i);
-	xmlSetProp(subnode, "track", c);
-	snprintf(c, 10, "%d", block->notes[2 * (block->tracks * j + i) + 1]);
-	xmlSetProp(subnode, "instrument", c);
-	xmlAddChild(node, xmlNewText("\n"));
+        snprintf((char *)c, 10, "%d", block->notes[2 * (block->tracks * j + i)]);
+        subnode = xmlNewChild(node, NULL, (xmlChar *)"note", c);
+        snprintf((char *)c, 10, "%d", j);
+        xmlSetProp(subnode, (xmlChar *)"line", c);
+        snprintf((char *)c, 10, "%d", i);
+        xmlSetProp(subnode, (xmlChar *)"track", c);
+        snprintf((char *)c, 10, "%d", block->notes[2 * (block->tracks * j + i) + 1]);
+        xmlSetProp(subnode, (xmlChar *)"instrument", c);
+        xmlAddChild(node, xmlNewText((xmlChar *)"\n"));
       }
 
   /* Command data */
   for (i = 0; i < block->tracks; i++)
     for (j = 0; j < block->length; j++)
       for (k = 0; k < block->commandpages; k++)
-	if (block->commands[k * 2 * block->tracks * block->length + 2 * (block->tracks * j + i)] != 0 || block->commands[k * 2 * block->tracks * block->length + 2 * (block->tracks * j + i) + 1] != 0) {
-	  snprintf(c, 10, "%d", block->commands[k * 2 * block->tracks * block->length + 2 * (block->tracks * j + i)]);
-	  subnode = xmlNewChild(node, NULL, "command", c);
-	  snprintf(c, 10, "%d", j);
-	  xmlSetProp(subnode, "line", c);
-	  snprintf(c, 10, "%d", i);
-	  xmlSetProp(subnode, "track", c);
-	  snprintf(c, 10, "%d", k);
-	  xmlSetProp(subnode, "commandpage", c);
-	  snprintf(c, 10, "%d", block->commands[k * 2 * block->tracks * block->length + 2 * (block->tracks * j + i) + 1]);
-	  xmlSetProp(subnode, "value", c);
-	  xmlAddChild(node, xmlNewText("\n"));
-	}
+        if (block->commands[k * 2 * block->tracks * block->length + 2 * (block->tracks * j + i)] != 0 || block->commands[k * 2 * block->tracks * block->length + 2 * (block->tracks * j + i) + 1] != 0) {
+          snprintf((char *)c, 10, "%d", block->commands[k * 2 * block->tracks * block->length + 2 * (block->tracks * j + i)]);
+          subnode = xmlNewChild(node, NULL, (xmlChar *)"command", c);
+          snprintf((char *)c, 10, "%d", j);
+          xmlSetProp(subnode, (xmlChar *)"line", c);
+          snprintf((char *)c, 10, "%d", i);
+          xmlSetProp(subnode, (xmlChar *)"track", c);
+          snprintf((char *)c, 10, "%d", k);
+          xmlSetProp(subnode, (xmlChar *)"commandpage", c);
+          snprintf((char *)c, 10, "%d", block->commands[k * 2 * block->tracks * block->length + 2 * (block->tracks * j + i) + 1]);
+          xmlSetProp(subnode, (xmlChar *)"value", c);
+          xmlAddChild(node, xmlNewText((xmlChar *)"\n"));
+        }
 
-  xmlAddChild(parent, xmlNewText("\n"));
+  xmlAddChild(parent, xmlNewText((xmlChar *)"\n"));
 }
 
 /* Transposes a block or a part of it */
@@ -563,30 +582,31 @@ void block_transpose(struct block *block, int instrument, int halfnotes, int sta
     startline = endline;
     endline = l;
   }
-    
+
   /* Either check for correct instrument or transpose all instruments */
   if (instrument < 0)
     for (l = startline; l <= endline; l++)
       for (t = starttrack; t <= endtrack; t++) {
-	if (block->notes[(l * block->tracks + t) * 2] > 0) {
-	  if (block->notes[(l * block->tracks + t) * 2] + halfnotes < 1)
-	    block->notes[(l * block->tracks + t) * 2] = 1;
-	  else if (block->notes[(l * block->tracks + t) * 2] + halfnotes > 127)
-	    block->notes[(l * block->tracks + t) * 2] = 127;
-	  else
-	    block->notes[(l * block->tracks + t) * 2] += halfnotes;
-	}
-  } else
+        if (block->notes[(l * block->tracks + t) * 2] > 0) {
+          if (block->notes[(l * block->tracks + t) * 2] + halfnotes < 1)
+            block->notes[(l * block->tracks + t) * 2] = 1;
+          else if (block->notes[(l * block->tracks + t) * 2] + halfnotes > 127)
+            block->notes[(l * block->tracks + t) * 2] = 127;
+          else
+            block->notes[(l * block->tracks + t) * 2] += halfnotes;
+        }
+      }
+  else
     for (l = startline; l <= endline; l++)
       for (t = starttrack; t <= endtrack; t++) {
-	if (block->notes[(l * block->tracks + t) * 2] > 0 && block->notes[(l * block->tracks + t) * 2 + 1] == instrument + 1) {
-	  if (block->notes[(l * block->tracks + t) * 2] + halfnotes < 0)
-	    block->notes[(l * block->tracks + t) * 2] = 1;
-	  else if(block->notes[(l * block->tracks + t) * 2] + halfnotes > 127)
-	    block->notes[(l * block->tracks + t) * 2] = 127;
-	  else
-	    block->notes[(l * block->tracks + t) * 2] += halfnotes;
-	}
+        if (block->notes[(l * block->tracks + t) * 2] > 0 && block->notes[(l * block->tracks + t) * 2 + 1] == instrument + 1) {
+          if (block->notes[(l * block->tracks + t) * 2] + halfnotes < 0)
+            block->notes[(l * block->tracks + t) * 2] = 1;
+          else if (block->notes[(l * block->tracks + t) * 2] + halfnotes > 127)
+            block->notes[(l * block->tracks + t) * 2] = 127;
+          else
+            block->notes[(l * block->tracks + t) * 2] += halfnotes;
+        }
       }
 }
 
@@ -629,50 +649,50 @@ void block_expandshrink(struct block *block, int factor, int starttrack, int sta
     startline = endline;
     endline = l;
   }
-    
+
   if (factor < 0) {
     /* Shrink */
     for (l = startline; l <= endline; l++)
       if ((l - startline) < (endline + 1 - startline) / -factor) {
-	for (t = starttrack; t <= endtrack; t++) {
-	  block->notes[(l * block->tracks + t) * 2] = block->notes[((startline + (l - startline) * -factor) * block->tracks + t) * 2];
-	  block->notes[(l * block->tracks + t) * 2 + 1] = block->notes[((startline + (l - startline) * -factor) * block->tracks + t) * 2 + 1];
-	  for (p = 0; p < block->commandpages; p++) {
-	    block->commands[p * block->tracks * block->length * 2 + (l * block->tracks + t) * 2] = block->commands[p * block->tracks * block->length * 2 + ((startline + (l - startline) * -factor) * block->tracks + t) * 2];
-	    block->commands[p * block->tracks * block->length * 2 + (l * block->tracks + t) * 2 + 1] = block->commands[p * block->tracks * block->length * 2 + ((startline + (l - startline) * -factor) * block->tracks + t) * 2 + 1];
-	  }
-	}
+        for (t = starttrack; t <= endtrack; t++) {
+          block->notes[(l * block->tracks + t) * 2] = block->notes[((startline + (l - startline) * -factor) * block->tracks + t) * 2];
+          block->notes[(l * block->tracks + t) * 2 + 1] = block->notes[((startline + (l - startline) * -factor) * block->tracks + t) * 2 + 1];
+          for (p = 0; p < block->commandpages; p++) {
+            block->commands[p * block->tracks * block->length * 2 + (l * block->tracks + t) * 2] = block->commands[p * block->tracks * block->length * 2 + ((startline + (l - startline) * -factor) * block->tracks + t) * 2];
+            block->commands[p * block->tracks * block->length * 2 + (l * block->tracks + t) * 2 + 1] = block->commands[p * block->tracks * block->length * 2 + ((startline + (l - startline) * -factor) * block->tracks + t) * 2 + 1];
+          }
+        }
       } else {
-	for (t = starttrack; t <= endtrack; t++) {
-	  block->notes[(l * block->tracks + t) * 2] = 0;
-	  block->notes[(l * block->tracks + t) * 2 + 1] = 0;
-	  for (p = 0; p < block->commandpages; p++) {
-	    block->commands[p * block->tracks * block->length * 2 + (l * block->tracks + t) * 2] = 0;
-	    block->commands[p * block->tracks * block->length * 2 + (l * block->tracks + t) * 2 + 1] = 0;
-	  }
-	}
+        for (t = starttrack; t <= endtrack; t++) {
+          block->notes[(l * block->tracks + t) * 2] = 0;
+          block->notes[(l * block->tracks + t) * 2 + 1] = 0;
+          for (p = 0; p < block->commandpages; p++) {
+            block->commands[p * block->tracks * block->length * 2 + (l * block->tracks + t) * 2] = 0;
+            block->commands[p * block->tracks * block->length * 2 + (l * block->tracks + t) * 2 + 1] = 0;
+          }
+        }
       }
   } else {
     /* Expand */
     for (l = endline; l >= startline; l--)
       if ((l - startline) % factor == 0) {
-	for (t = starttrack; t <= endtrack; t++) {
-	  block->notes[(l * block->tracks + t) * 2] = block->notes[((startline + (l - startline) / factor) * block->tracks + t) * 2];
-	  block->notes[(l * block->tracks + t) * 2 + 1] = block->notes[((startline + (l - startline) / factor) * block->tracks + t) * 2 + 1];
-	  for (p = 0; p < block->commandpages; p++) {
-	    block->commands[p * block->tracks * block->length * 2 + (l * block->tracks + t) * 2] = block->commands[p * block->tracks * block->length * 2 + ((startline + (l - startline) / factor) * block->tracks + t) * 2];
-	    block->commands[p * block->tracks * block->length * 2 + (l * block->tracks + t) * 2 + 1] = block->commands[p * block->tracks * block->length * 2 + ((startline + (l - startline) / factor) * block->tracks + t) * 2 + 1];
-	  }
-	}
+        for (t = starttrack; t <= endtrack; t++) {
+          block->notes[(l * block->tracks + t) * 2] = block->notes[((startline + (l - startline) / factor) * block->tracks + t) * 2];
+          block->notes[(l * block->tracks + t) * 2 + 1] = block->notes[((startline + (l - startline) / factor) * block->tracks + t) * 2 + 1];
+          for (p = 0; p < block->commandpages; p++) {
+            block->commands[p * block->tracks * block->length * 2 + (l * block->tracks + t) * 2] = block->commands[p * block->tracks * block->length * 2 + ((startline + (l - startline) / factor) * block->tracks + t) * 2];
+            block->commands[p * block->tracks * block->length * 2 + (l * block->tracks + t) * 2 + 1] = block->commands[p * block->tracks * block->length * 2 + ((startline + (l - startline) / factor) * block->tracks + t) * 2 + 1];
+          }
+        }
       } else {
-	for (t = starttrack; t <= endtrack; t++) {
-	  block->notes[(l * block->tracks + t) * 2] = 0;
-	  block->notes[(l * block->tracks + t) * 2 + 1] = 0;
-	  for (p = 0; p < block->commandpages; p++) {
-	    block->commands[p * block->tracks * block->length * 2 + (l * block->tracks + t) * 2] = 0;
-	    block->commands[p * block->tracks * block->length * 2 + (l * block->tracks + t) * 2 + 1] = 0;
-	  }
-	}
+        for (t = starttrack; t <= endtrack; t++) {
+          block->notes[(l * block->tracks + t) * 2] = 0;
+          block->notes[(l * block->tracks + t) * 2 + 1] = 0;
+          for (p = 0; p < block->commandpages; p++) {
+            block->commands[p * block->tracks * block->length * 2 + (l * block->tracks + t) * 2] = 0;
+            block->commands[p * block->tracks * block->length * 2 + (l * block->tracks + t) * 2 + 1] = 0;
+          }
+        }
       }
   }
 }
@@ -713,17 +733,18 @@ void block_changeinstrument(struct block *block, int from, int to, int swap, int
     startline = endline;
     endline = l;
   }
-    
+
   if (swap)
     for (l = endline; l >= startline; l--)
       for (t = starttrack; t <= endtrack; t++) {
-	if (block->notes[(l * block->tracks + t) * 2 + 1] == from)
-	  block->notes[(l * block->tracks + t) * 2 + 1] = to;
-	else if (block->notes[(l * block->tracks + t) * 2 + 1] == to)
-	  block->notes[(l * block->tracks + t) * 2 + 1] = from;
-  } else
+        if (block->notes[(l * block->tracks + t) * 2 + 1] == from)
+          block->notes[(l * block->tracks + t) * 2 + 1] = to;
+        else if (block->notes[(l * block->tracks + t) * 2 + 1] == to)
+          block->notes[(l * block->tracks + t) * 2 + 1] = from;
+      }
+  else
     for (l = endline; l >= startline; l--)
       for (t = starttrack; t <= endtrack; t++)
-	if (block->notes[(l * block->tracks + t) * 2 + 1] == from)
-	  block->notes[(l * block->tracks + t) * 2 + 1] = to;
+        if (block->notes[(l * block->tracks + t) * 2 + 1] == from)
+          block->notes[(l * block->tracks + t) * 2 + 1] = to;
 }
