@@ -1,6 +1,6 @@
 /* callbacks.c
  *
- * Copyright 2002-2006 Vesa Halttunen
+ * Copyright 2002-2011 Vesa Halttunen
  *
  * This file is part of Tutka.
  *
@@ -98,7 +98,7 @@ void on_menuitem_file_exit_activate(GtkMenuItem * menuitem, gpointer user_data)
 void on_menuitem_edit_cut_activate(GtkMenuItem * menuitem, gpointer user_data)
 {
   struct gui *gui = (struct gui *)user_data;
-  Tracker *tracker = (Tracker *) glade_xml_get_widget(gui->xml, "tracker");
+  Tracker *tracker = (Tracker *)glade_xml_get_widget(gui->xml, "tracker");
 
   if (tracker_is_in_selection_mode(tracker))
     tracker_mark_selection(tracker, FALSE);
@@ -111,7 +111,7 @@ void on_menuitem_edit_cut_activate(GtkMenuItem * menuitem, gpointer user_data)
 void on_menuitem_edit_copy_activate(GtkMenuItem * menuitem, gpointer user_data)
 {
   struct gui *gui = (struct gui *)user_data;
-  Tracker *tracker = (Tracker *) glade_xml_get_widget(gui->xml, "tracker");
+  Tracker *tracker = (Tracker *)glade_xml_get_widget(gui->xml, "tracker");
 
   if (tracker_is_in_selection_mode(tracker))
     tracker_mark_selection(tracker, FALSE);
@@ -122,7 +122,7 @@ void on_menuitem_edit_copy_activate(GtkMenuItem * menuitem, gpointer user_data)
 void on_menuitem_edit_paste_activate(GtkMenuItem * menuitem, gpointer user_data)
 {
   struct gui *gui = (struct gui *)user_data;
-  Tracker *tracker = (Tracker *) glade_xml_get_widget(gui->xml, "tracker");
+  Tracker *tracker = (Tracker *)glade_xml_get_widget(gui->xml, "tracker");
 
   editor_song_block_current_copyarea_paste(gui->editor, tracker->cursor_ch, tracker->patpos);
   tracker_redraw(tracker);
@@ -131,7 +131,7 @@ void on_menuitem_edit_paste_activate(GtkMenuItem * menuitem, gpointer user_data)
 void on_menuitem_edit_clear_activate(GtkMenuItem * menuitem, gpointer user_data)
 {
   struct gui *gui = (struct gui *)user_data;
-  Tracker *tracker = (Tracker *) glade_xml_get_widget(gui->xml, "tracker");
+  Tracker *tracker = (Tracker *)glade_xml_get_widget(gui->xml, "tracker");
 
   editor_song_block_current_clear(gui->editor, tracker->sel_start_ch, tracker->sel_start_row, tracker->sel_end_ch, tracker->sel_end_row);
   tracker_redraw(tracker);
@@ -288,7 +288,7 @@ void on_menuitem_block_cut_activate(GtkMenuItem * menuitem, gpointer user_data)
 
   editor_song_block_current_copyblock(gui->editor);
   editor_song_block_current_clear(gui->editor, 0, 0, editor_song_block_current_tracks_get(gui->editor) - 1, editor_song_block_current_length_get(gui->editor) - 1);
-  tracker_redraw((Tracker *) glade_xml_get_widget(gui->xml, "tracker"));
+  tracker_redraw((Tracker *)glade_xml_get_widget(gui->xml, "tracker"));
 }
 
 void on_menuitem_block_copy_activate(GtkMenuItem * menuitem, gpointer user_data)
@@ -303,13 +303,13 @@ void on_menuitem_block_paste_activate(GtkMenuItem * menuitem, gpointer user_data
   struct gui *gui = (struct gui *)user_data;
 
   editor_song_block_current_copyblock_paste(gui->editor);
-  tracker_redraw((Tracker *) glade_xml_get_widget(gui->xml, "tracker"));
+  tracker_redraw((Tracker *)glade_xml_get_widget(gui->xml, "tracker"));
 }
 
 void on_menuitem_block_selectall_activate(GtkMenuItem * menuitem, gpointer user_data)
 {
   struct gui *gui = (struct gui *)user_data;
-  Tracker *tracker = (Tracker *) glade_xml_get_widget(gui->xml, "tracker");
+  Tracker *tracker = (Tracker *)glade_xml_get_widget(gui->xml, "tracker");
 
   tracker->sel_start_ch = 0;
   tracker->sel_start_row = 0;
@@ -324,7 +324,7 @@ void on_menuitem_block_clear_activate(GtkMenuItem * menuitem, gpointer user_data
   struct gui *gui = (struct gui *)user_data;
 
   editor_song_block_current_clear(gui->editor, 0, 0, editor_song_block_current_tracks_get(gui->editor) - 1, editor_song_block_current_length_get(gui->editor) - 1);
-  tracker_redraw((Tracker *) glade_xml_get_widget(gui->xml, "tracker"));
+  tracker_redraw((Tracker *)glade_xml_get_widget(gui->xml, "tracker"));
 }
 
 void on_menuitem_block_transpose_activate(GtkMenuItem * menuitem, gpointer user_data)
@@ -381,7 +381,7 @@ void on_menuitem_block_changeinstrument_activate(GtkMenuItem * menuitem, gpointe
 void on_menuitem_track_cut_activate(GtkMenuItem * menuitem, gpointer user_data)
 {
   struct gui *gui = (struct gui *)user_data;
-  Tracker *tracker = (Tracker *) glade_xml_get_widget(gui->xml, "tracker");
+  Tracker *tracker = (Tracker *)glade_xml_get_widget(gui->xml, "tracker");
 
   editor_song_block_current_copytrack(gui->editor, tracker->cursor_ch);
   editor_song_block_current_clear(gui->editor, tracker->cursor_ch, 0, tracker->cursor_ch, editor_song_block_current_length_get(gui->editor) - 1);
@@ -391,7 +391,7 @@ void on_menuitem_track_cut_activate(GtkMenuItem * menuitem, gpointer user_data)
 void on_menuitem_track_copy_activate(GtkMenuItem * menuitem, gpointer user_data)
 {
   struct gui *gui = (struct gui *)user_data;
-  Tracker *tracker = (Tracker *) glade_xml_get_widget(gui->xml, "tracker");
+  Tracker *tracker = (Tracker *)glade_xml_get_widget(gui->xml, "tracker");
 
   editor_song_block_current_copytrack(gui->editor, tracker->cursor_ch);
 }
@@ -399,7 +399,7 @@ void on_menuitem_track_copy_activate(GtkMenuItem * menuitem, gpointer user_data)
 void on_menuitem_track_paste_activate(GtkMenuItem * menuitem, gpointer user_data)
 {
   struct gui *gui = (struct gui *)user_data;
-  Tracker *tracker = (Tracker *) glade_xml_get_widget(gui->xml, "tracker");
+  Tracker *tracker = (Tracker *)glade_xml_get_widget(gui->xml, "tracker");
 
   editor_song_block_current_copytrack_paste(gui->editor, tracker->cursor_ch);
   tracker_redraw(tracker);
@@ -408,7 +408,7 @@ void on_menuitem_track_paste_activate(GtkMenuItem * menuitem, gpointer user_data
 void on_menuitem_track_selectall_activate(GtkMenuItem * menuitem, gpointer user_data)
 {
   struct gui *gui = (struct gui *)user_data;
-  Tracker *tracker = (Tracker *) glade_xml_get_widget(gui->xml, "tracker");
+  Tracker *tracker = (Tracker *)glade_xml_get_widget(gui->xml, "tracker");
 
   tracker->sel_start_ch = tracker->cursor_ch;
   tracker->sel_start_row = 0;
@@ -421,12 +421,44 @@ void on_menuitem_track_selectall_activate(GtkMenuItem * menuitem, gpointer user_
 void on_menuitem_track_clear_activate(GtkMenuItem * menuitem, gpointer user_data)
 {
   struct gui *gui = (struct gui *)user_data;
-  Tracker *tracker = (Tracker *) glade_xml_get_widget(gui->xml, "tracker");
+  Tracker *tracker = (Tracker *)glade_xml_get_widget(gui->xml, "tracker");
 
   editor_song_block_current_clear(gui->editor, tracker->cursor_ch, 0, tracker->cursor_ch, editor_song_block_current_length_get(gui->editor) - 1);
   tracker_redraw(tracker);
 }
 
+void on_menuitem_track_insert_activate(GtkMenuItem * menuitem, gpointer user_data)
+{
+  struct gui *gui = (struct gui *)user_data;
+  Tracker *tracker = (Tracker *)glade_xml_get_widget(gui->xml, "tracker");
+
+  editor_song_block_current_insert_track(gui->editor, tracker->cursor_ch);
+
+  /* No need to call editor_player_trackstatus_create since
+   editor_song_maxtracks_check does the necessary thing automatically */
+  if (editor_song_maxtracks_check(gui->editor))
+    gui_trackvolumes_refresh(gui);
+
+  gui_tracker_refresh(gui, FALSE);
+  gui_blocklist_refresh(gui, TRUE);
+}
+
+void on_menuitem_track_delete_activate(GtkMenuItem * menuitem, gpointer user_data)
+{
+  struct gui *gui = (struct gui *)user_data;
+  Tracker *tracker = (Tracker *)glade_xml_get_widget(gui->xml, "tracker");
+
+  editor_song_block_current_delete_track(gui->editor, tracker->cursor_ch);
+
+  /* No need to call editor_player_trackstatus_create since
+   editor_song_maxtracks_check does the necessary thing automatically */
+  if (editor_song_maxtracks_check(gui->editor))
+    gui_trackvolumes_refresh(gui);
+
+  gui_tracker_refresh(gui, FALSE);
+  gui_blocklist_refresh(gui, TRUE);
+}
+
 void on_menuitem_track_transpose_activate(GtkMenuItem * menuitem, gpointer user_data)
 {
   struct gui *gui = (struct gui *)user_data;
@@ -538,6 +570,13 @@ void on_menuitem_settings_externalsync_jack_activate(GtkMenuItem * menuitem, gpo
   editor_player_set_external_sync(gui->editor, EXTERNAL_SYNC_JACK);
 }
 
+void on_menuitem_settings_externalsync_jack_start_only_activate(GtkMenuItem * menuitem, gpointer user_data)
+{
+  struct gui *gui = (struct gui *)user_data;
+
+  editor_player_set_external_sync(gui->editor, EXTERNAL_SYNC_JACK_START_ONLY);
+}
+
 void on_menuitem_settings_preferences_activate(GtkMenuItem * menuitem, gpointer user_data)
 {
   struct gui *gui = (struct gui *)user_data;
@@ -560,14 +599,13 @@ void on_menuitem_about_activate(GtkMenuItem * menuitem, gpointer user_data)
     return;
   }
 
-  gui->window_about = gnome_about_new("Tutka", VERSION, "Copyright \xc2\xa9 2002-2006 Vesa Halttunen", "http://www.nongnu.org/tutka/", (const gchar **)authors, (const gchar **)authors, NULL, NULL);
+  gui->window_about = gnome_about_new("Tutka", VERSION, "Copyright \xc2\xa9 2002-2011 Vesa Halttunen", "http://www.nongnu.org/tutka/", (const gchar **)authors, (const gchar **)authors, NULL, NULL);
 
   gtk_window_set_destroy_with_parent(GTK_WINDOW(gui->window_about), TRUE);
   g_signal_connect(G_OBJECT(gui->window_about), "destroy", G_CALLBACK(gtk_widget_destroyed), &gui->window_about);
   gtk_widget_show(gui->window_about);
 }
 
-
 GtkWidget *create_tracker(gchar * widget_name, gchar * string1, gchar * string2, gint int1, gint int2)
 {
   return tracker_new();
@@ -800,7 +838,7 @@ void on_button_message_receive_clicked(GtkButton * button, gpointer user_data)
     gtk_widget_show(window);
 
   gui->editor->receivemessage = gui->messagelist_message;
-  
+
   gtk_window_present(GTK_WINDOW(window));
   gui_receive_refresh(gui, -1, 0);
 }
@@ -888,7 +926,6 @@ void on_selection_sectionlist_changed(GtkTreeSelection * selection, gpointer use
   editor_set_section(gui->editor, value);
 }
 
-
 void on_button_sectionlist_delete_clicked(GtkButton * button, gpointer user_data)
 {
   struct gui *gui = (struct gui *)user_data;
@@ -1165,7 +1202,7 @@ gint on_spinbutton_instrument_input(GtkSpinButton * spin_button, gpointer arg1,
   const gchar *c = gtk_entry_get_text(GTK_ENTRY(spin_button));
   gdouble *new_val = arg1;
   char *err;
-  
+
   if (c == NULL)
     return GTK_INPUT_ERROR;
 
@@ -1173,7 +1210,7 @@ gint on_spinbutton_instrument_input(GtkSpinButton * spin_button, gpointer arg1,
   *new_val = strtol(c, &err, 16);
   if (*err != 0)
     return GTK_INPUT_ERROR;
-  
+
   return TRUE;
 }
 
@@ -1228,7 +1265,7 @@ void on_button_transpose_close_clicked(GtkButton * button, gpointer user_data)
 void on_button_transpose_clicked(GtkButton * button, gpointer user_data)
 {
   struct gui *gui = (struct gui *)user_data;
-  Tracker *tracker = (Tracker *) glade_xml_get_widget(gui->xml, "tracker");
+  Tracker *tracker = (Tracker *)glade_xml_get_widget(gui->xml, "tracker");
 
   int area = gtk_combo_box_get_active(GTK_COMBO_BOX(glade_xml_get_widget(gui->xml, "combobox_transpose_area")));
   int instruments = gtk_combo_box_get_active(GTK_COMBO_BOX(glade_xml_get_widget(gui->xml, "combobox_transpose_instrument")));
@@ -1289,7 +1326,7 @@ void on_button_expandshrink_close_clicked(GtkButton * button, gpointer user_data
 void on_button_expandshrink_shrink_clicked(GtkButton * button, gpointer user_data)
 {
   struct gui *gui = (struct gui *)user_data;
-  Tracker *tracker = (Tracker *) glade_xml_get_widget(gui->xml, "tracker");
+  Tracker *tracker = (Tracker *)glade_xml_get_widget(gui->xml, "tracker");
 
   int area = gtk_combo_box_get_active(GTK_COMBO_BOX(glade_xml_get_widget(gui->xml, "combobox_expandshrink_area")));
   int factor = gtk_spin_button_get_value(GTK_SPIN_BUTTON(glade_xml_get_widget(gui->xml, "spinbutton_expandshrink_factor")));
@@ -1321,7 +1358,7 @@ void on_button_expandshrink_shrink_clicked(GtkButton * button, gpointer user_dat
 void on_button_expandshrink_expand_clicked(GtkButton * button, gpointer user_data)
 {
   struct gui *gui = (struct gui *)user_data;
-  Tracker *tracker = (Tracker *) glade_xml_get_widget(gui->xml, "tracker");
+  Tracker *tracker = (Tracker *)glade_xml_get_widget(gui->xml, "tracker");
 
   int area = gtk_combo_box_get_active(GTK_COMBO_BOX(glade_xml_get_widget(gui->xml, "combobox_expandshrink_area")));
   int factor = gtk_spin_button_get_value(GTK_SPIN_BUTTON(glade_xml_get_widget(gui->xml, "spinbutton_expandshrink_factor")));
@@ -1362,7 +1399,7 @@ void on_button_changeinstrument_close_clicked(GtkButton * button, gpointer user_
 void on_button_changeinstrument_swap_clicked(GtkButton * button, gpointer user_data)
 {
   struct gui *gui = (struct gui *)user_data;
-  Tracker *tracker = (Tracker *) glade_xml_get_widget(gui->xml, "tracker");
+  Tracker *tracker = (Tracker *)glade_xml_get_widget(gui->xml, "tracker");
 
   int area = gtk_combo_box_get_active(GTK_COMBO_BOX(glade_xml_get_widget(gui->xml, "combobox_changeinstrument_area")));
   int from = gtk_spin_button_get_value(GTK_SPIN_BUTTON(glade_xml_get_widget(gui->xml, "spinbutton_changeinstrument_from")));
@@ -1395,7 +1432,7 @@ void on_button_changeinstrument_swap_clicked(GtkButton * button, gpointer user_d
 void on_button_changeinstrument_change_clicked(GtkButton * button, gpointer user_data)
 {
   struct gui *gui = (struct gui *)user_data;
-  Tracker *tracker = (Tracker *) glade_xml_get_widget(gui->xml, "tracker");
+  Tracker *tracker = (Tracker *)glade_xml_get_widget(gui->xml, "tracker");
 
   int area = gtk_combo_box_get_active(GTK_COMBO_BOX(glade_xml_get_widget(gui->xml, "combobox_changeinstrument_area")));
   int from = gtk_spin_button_get_value(GTK_SPIN_BUTTON(glade_xml_get_widget(gui->xml, "spinbutton_changeinstrument_from")));
@@ -1545,7 +1582,6 @@ void on_button_quit_cancel_clicked(GtkButton * button, gpointer user_data)
     gtk_widget_hide(window);
 }
 
-
 void on_button_instrumentproperties_close_clicked(GtkButton * button, gpointer user_data)
 {
   struct gui *gui = (struct gui *)user_data;
@@ -1733,18 +1769,17 @@ void on_cell_sectionlist_edited(GtkCellRendererText * cell, gchar * path_string,
 
   /* Update the model */
   switch (column) {
-  default:
-    {
-      char *error;
-      value = (int)strtol(new_text, &error, 10);
-      /* Only update if the entered value was valid */
-      if (*error == 0 && value > 0 && value <= editor_song_numplayseqs_get(gui->editor)) {
-	gtk_list_store_set(GTK_LIST_STORE(model), &iter, column, value, -1);
-	gtk_list_store_set(GTK_LIST_STORE(model), &iter, column + 1, editor_song_playseq_name_get(gui->editor, value - 1), -1);
-	editor_song_section_current_set(gui->editor, value - 1);
-      }
-      break;
+  default: {
+    char *error;
+    value = (int)strtol(new_text, &error, 10);
+    /* Only update if the entered value was valid */
+    if (*error == 0 && value > 0 && value <= editor_song_numplayseqs_get(gui->editor)) {
+      gtk_list_store_set(GTK_LIST_STORE(model), &iter, column, value, -1);
+      gtk_list_store_set(GTK_LIST_STORE(model), &iter, column + 1, editor_song_playseq_name_get(gui->editor, value - 1), -1);
+      editor_song_section_current_set(gui->editor, value - 1);
     }
+    break;
+  }
   }
 }
 
@@ -1805,18 +1840,17 @@ void on_cell_playseq_edited(GtkCellRendererText * cell, gchar * path_string, gch
 
   /* Update the model */
   switch (column) {
-  default:
-    {
-      char *error;
-      value = (int)strtol(new_text, &error, 10);
-      /* Only update if the entered value was valid */
-      if (*error == 0 && value > 0 && value <= editor_song_numblocks_get(gui->editor)) {
-	gtk_list_store_set(GTK_LIST_STORE(model), &iter, column, value, -1);
-	editor_song_playseq_current_set(gui->editor, value - 1);
-	gui_playseq_refresh(gui, TRUE);
-      }
-      break;
+  default: {
+    char *error;
+    value = (int)strtol(new_text, &error, 10);
+    /* Only update if the entered value was valid */
+    if (*error == 0 && value > 0 && value <= editor_song_numblocks_get(gui->editor)) {
+      gtk_list_store_set(GTK_LIST_STORE(model), &iter, column, value, -1);
+      editor_song_playseq_current_set(gui->editor, value - 1);
+      gui_playseq_refresh(gui, TRUE);
     }
+    break;
+  }
   }
 }
 
@@ -1849,17 +1883,16 @@ void on_cell_blocklist_edited(GtkCellRendererText * cell, gchar * path_string, g
     gtk_list_store_set(GTK_LIST_STORE(model), &iter, column, new_text, -1);
     changed = TRUE;
     break;
-  default:
-    {
-      char *error;
-      value = (int)strtol(new_text, &error, 10);
-      /* Only update if the entered value was valid */
-      if (*error == 0 && value > 0) {
-	gtk_list_store_set(GTK_LIST_STORE(model), &iter, column, value, -1);
-	changed = TRUE;
-      }
-      break;
+  default: {
+    char *error;
+    value = (int)strtol(new_text, &error, 10);
+    /* Only update if the entered value was valid */
+    if (*error == 0 && value > 0) {
+      gtk_list_store_set(GTK_LIST_STORE(model), &iter, column, value, -1);
+      changed = TRUE;
     }
+    break;
+  }
   }
 
   /* If the model was not changed no need to do anything */
@@ -1868,44 +1901,40 @@ void on_cell_blocklist_edited(GtkCellRendererText * cell, gchar * path_string, g
 
   /* See what whas changed and act accordingly */
   switch (column) {
-  case GUI_BLOCKLIST_COLUMN_NAME:
-    {
-      editor_song_block_current_name_set(gui->editor, new_text);
-      gui_blocklist_refresh(gui, TRUE);
-      gui_playseq_refresh(gui, TRUE);
-      gui_info_refresh(gui);
+  case GUI_BLOCKLIST_COLUMN_NAME: {
+    editor_song_block_current_name_set(gui->editor, new_text);
+    gui_blocklist_refresh(gui, TRUE);
+    gui_playseq_refresh(gui, TRUE);
+    gui_info_refresh(gui);
 
-      break;
-    }
-  case GUI_BLOCKLIST_COLUMN_TRACKS:
-    {
-      editor_song_block_current_tracks_set(gui->editor, value);
+    break;
+  }
+  case GUI_BLOCKLIST_COLUMN_TRACKS: {
+    editor_song_block_current_tracks_set(gui->editor, value);
 
-      /* No need to call editor_player_trackstatus_create since
-	 editor_song_maxtracks_check does the necessary thing automatically */
-      if (editor_song_maxtracks_check(gui->editor))
-	gui_trackvolumes_refresh(gui);
+    /* No need to call editor_player_trackstatus_create since
+     editor_song_maxtracks_check does the necessary thing automatically */
+    if (editor_song_maxtracks_check(gui->editor))
+      gui_trackvolumes_refresh(gui);
 
+    gui_tracker_refresh(gui, FALSE);
+    break;
+  }
+  case GUI_BLOCKLIST_COLUMN_LENGTH: {
+    editor_song_block_current_length_set(gui->editor, value);
+    gui_tracker_refresh(gui, TRUE);
+
+    break;
+  }
+  case GUI_BLOCKLIST_COLUMN_COMMAND_PAGES: {
+    editor_song_block_current_commandpages_set(gui->editor, value);
+    if (gui->editor->cmdpage >= editor_song_block_current_commandpages_get(gui->editor)) {
+      gui->editor->cmdpage = editor_song_block_current_commandpages_get(gui->editor) - 1;
       gui_tracker_refresh(gui, FALSE);
-      break;
-    }
-  case GUI_BLOCKLIST_COLUMN_LENGTH:
-    {
-      editor_song_block_current_length_set(gui->editor, value);
-      gui_tracker_refresh(gui, TRUE);
-
-      break;
-    }
-  case GUI_BLOCKLIST_COLUMN_COMMAND_PAGES:
-    {
-      editor_song_block_current_commandpages_set(gui->editor, value);
-      if (gui->editor->cmdpage >= editor_song_block_current_commandpages_get(gui->editor)) {
-	gui->editor->cmdpage = editor_song_block_current_commandpages_get(gui->editor) - 1;
-	gui_tracker_refresh(gui, FALSE);
-      }
-      gui_info_refresh(gui);
-      break;
     }
+    gui_info_refresh(gui);
+    break;
+  }
   default:
     break;
   }
@@ -1941,17 +1970,16 @@ void on_cell_messagelist_edited(GtkCellRendererText * cell, gchar * path_string,
       editor_song_message_name_set(gui->editor, gui->messagelist_message, new_text);
     }
     break;
-  default:
-    {
-      char *error;
-      value = (int)strtol(new_text, &error, 10);
-      /* Only update if the entered value was valid */
-      if (*error == 0 && gui->messagelist_message != -1) {
-	gtk_list_store_set(GTK_LIST_STORE(model), &iter, column, value, -1);
-	editor_song_message_length_set(gui->editor, gui->messagelist_message, value);
-      }
-      break;
+  default: {
+    char *error;
+    value = (int)strtol(new_text, &error, 10);
+    /* Only update if the entered value was valid */
+    if (*error == 0 && gui->messagelist_message != -1) {
+      gtk_list_store_set(GTK_LIST_STORE(model), &iter, column, value, -1);
+      editor_song_message_length_set(gui->editor, gui->messagelist_message, value);
     }
+    break;
+  }
   }
 }
 
@@ -1987,8 +2015,8 @@ void on_cell_preferences_midi_interfaces_toggled(GtkCellRendererToggle * cell, g
   GtkTreeIter iter;
   guint column, direction;
   gboolean state;
-  GValue client = { 0 }, port = { 0 };
-  
+  GValue name = { 0 };
+
   /* Column number and direction are passed as renderer object data */
   gpointer columnptr = g_object_get_data(G_OBJECT(cell), "column");
   gpointer directionptr = g_object_get_data(G_OBJECT(cell), "direction");
@@ -2000,7 +2028,7 @@ void on_cell_preferences_midi_interfaces_toggled(GtkCellRendererToggle * cell, g
   if (direction == DIRECTION_OUT)
     treeview = GTK_TREE_VIEW(glade_xml_get_widget(gui->xml, "treeview_preferences_midi_interfaces_output"));
   else
-    treeview = GTK_TREE_VIEW(glade_xml_get_widget(gui->xml, "treeview_preferences_midi_interfaces_input"));    
+    treeview = GTK_TREE_VIEW(glade_xml_get_widget(gui->xml, "treeview_preferences_midi_interfaces_input"));
   model = gtk_tree_view_get_model(treeview);
   gtk_tree_model_get_iter_from_string(model, &iter, path_string);
 
@@ -2011,9 +2039,8 @@ void on_cell_preferences_midi_interfaces_toggled(GtkCellRendererToggle * cell, g
   else
     state = TRUE;
 
-  gtk_tree_model_get_value(model, &iter, GUI_PREFERENCES_MIDI_INTERFACES_COLUMN_CLIENT, &client);
-  gtk_tree_model_get_value(model, &iter, GUI_PREFERENCES_MIDI_INTERFACES_COLUMN_PORT, &port);
-  if (editor_midi_interface_status_set(gui->editor, g_value_get_int(&client), g_value_get_int(&port), direction, state))
+  gtk_tree_model_get_value(model, &iter, GUI_PREFERENCES_MIDI_INTERFACES_COLUMN_NAME, &name);
+  if (editor_midi_interface_status_set(gui->editor, (char *)g_value_get_string(&name), direction, state))
     state = TRUE;
   else
     state = FALSE;
@@ -2021,8 +2048,7 @@ void on_cell_preferences_midi_interfaces_toggled(GtkCellRendererToggle * cell, g
   /* Update the model according to the true status of the device */
   gtk_list_store_set(GTK_LIST_STORE(model), &iter, column, state, -1);
 
-  g_value_unset(&client);
-  g_value_unset(&port);
+  g_value_unset(&name);
 }
 
 void on_combobox_instrumentproperties_midiinterface_changed(GtkComboBox * combobox, gpointer user_data)
@@ -2051,7 +2077,7 @@ void on_dialog_receive_delete_event(GtkWidget * widget, GdkEvent * event, gpoint
 void on_vscrollbar_tracker_value_changed(GtkRange * range, gpointer user_data)
 {
   struct gui *gui = (struct gui *)user_data;
-  Tracker *tracker = (Tracker *) glade_xml_get_widget(gui->xml, "tracker");
+  Tracker *tracker = (Tracker *)glade_xml_get_widget(gui->xml, "tracker");
   int line = (int)(gtk_range_get_value(range) * (editor_song_block_current_length_get(gui->editor) + tracker->disp_rows));
   editor_set_line(gui->editor, line);
 }
@@ -2059,7 +2085,7 @@ void on_vscrollbar_tracker_value_changed(GtkRange * range, gpointer user_data)
 void on_hscrollbar_tracker_value_changed(GtkRange * range, gpointer user_data)
 {
   struct gui *gui = (struct gui *)user_data;
-  Tracker *tracker = (Tracker *) glade_xml_get_widget(gui->xml, "tracker");
+  Tracker *tracker = (Tracker *)glade_xml_get_widget(gui->xml, "tracker");
   int left = (int)(gtk_range_get_value(range) * editor_song_block_current_tracks_get(gui->editor));
   tracker_set_xpanning(tracker, left);
 }
@@ -2067,7 +2093,7 @@ void on_hscrollbar_tracker_value_changed(GtkRange * range, gpointer user_data)
 void on_tracker_patpos(GtkWidget * widget, int current, int length, int visible, gpointer user_data)
 {
   struct gui *gui = (struct gui *)user_data;
-    
+
   GtkRange *r = GTK_RANGE(glade_xml_get_widget(gui->xml, "vscrollbar_tracker"));
   GtkAdjustment *a = gtk_range_get_adjustment(r);
   a->value = (gdouble)current / (length + visible - 1);
@@ -2082,7 +2108,7 @@ void on_tracker_patpos(GtkWidget * widget, int current, int length, int visible,
 void on_tracker_xpanning(GtkWidget * widget, int current, int tracks, int visible, gpointer user_data)
 {
   struct gui *gui = (struct gui *)user_data;
-    
+
   GtkRange *r = GTK_RANGE(glade_xml_get_widget(gui->xml, "hscrollbar_tracker"));
   GtkAdjustment *a = gtk_range_get_adjustment(r);
   a->value = (gdouble)current / tracks;
@@ -2094,9 +2120,10 @@ void on_tracker_xpanning(GtkWidget * widget, int current, int tracks, int visibl
   gtk_adjustment_changed(a);
 }
 
-gboolean on_tracker_scroll_event (GtkWidget *widget, GdkEventScroll *event, gpointer user_data) {
+gboolean on_tracker_scroll_event(GtkWidget *widget, GdkEventScroll *event, gpointer user_data)
+{
   struct gui *gui = (struct gui *)user_data;
-  Tracker *tracker = (Tracker *) glade_xml_get_widget(gui->xml, "tracker");
+  Tracker *tracker = (Tracker *)glade_xml_get_widget(gui->xml, "tracker");
 
   switch (event->direction) {
   case GDK_SCROLL_UP:
