@@ -1,6 +1,6 @@
 /* conversion.c
  *
- * Copyright 2002-2006 Vesa Halttunen
+ * Copyright 2002-2011 Vesa Halttunen
  *
  * This file is part of Tutka.
  *
@@ -30,6 +30,9 @@
 static void MMD2_commands_convert(unsigned char *command, struct MMDMIDICmd3x *mmdcmd3x)
 {
   switch (command[0]) {
+  case COMMAND_PITCH_WHEEL:
+    command[1] = (command[1] + 0x80) / 2;
+    break;
   case COMMAND_PREVIOUS_COMMAND_VALUE:
     if (command[1] != 0) {
       command[0] = 0x80;
@@ -37,11 +40,8 @@ static void MMD2_commands_convert(unsigned char *command, struct MMDMIDICmd3x *m
     }
     break;
   case COMMAND_VELOCITY:
-    /* Fix volume range to 0..127 */
-    if (command[1] == 64)
-      command[1] = 127;
-    else
-      command[1] *= 2;
+    /* MIDI volumes are in range 0-95 in OctaMED */
+    command[1] = command[1] * 0x80 / 0x60;
     break;
   case COMMAND_DELAY:
     /* Command 1F in MMD is both note delay and retrigger */
@@ -55,44 +55,44 @@ static void MMD2_commands_convert(unsigned char *command, struct MMDMIDICmd3x *m
     if (command[1] > 0xf0) {
       switch (command[1]) {
       case 0xf1:
-	/* Makes a single note play twice */
-	command[0] = COMMAND_RETRIGGER;
-	command[1] = 3;
-	break;
+        /* Makes a single note play twice */
+        command[0] = COMMAND_RETRIGGER;
+        command[1] = 3;
+        break;
       case 0xf2:
-	/* Delays the start of a note by half a line */
-	command[0] = COMMAND_DELAY;
-	command[1] = 3;
-	break;
+        /* Delays the start of a note by half a line */
+        command[0] = COMMAND_DELAY;
+        command[1] = 3;
+        break;
       case 0xf3:
-	/* Works like 0FF1 except the note is played three times */
-	command[0] = COMMAND_RETRIGGER;
-	command[1] = 2;
-	break;
+        /* Works like 0FF1 except the note is played three times */
+        command[0] = COMMAND_RETRIGGER;
+        command[1] = 2;
+        break;
       case 0xf4:
-	/* Delays the note one-third of a line */
-	command[0] = COMMAND_DELAY;
-	command[1] = 2;
-	break;
+        /* Delays the note one-third of a line */
+        command[0] = COMMAND_DELAY;
+        command[1] = 2;
+        break;
       case 0xf5:
-	/* Delays the note two thirds of a line */
-	command[0] = COMMAND_DELAY;
-	command[1] = 4;
-	break;
+        /* Delays the note two thirds of a line */
+        command[0] = COMMAND_DELAY;
+        command[1] = 4;
+        break;
       case 0xfe:
-	/* Stops the song playing */
-	command[0] = COMMAND_TEMPO;
-	command[1] = 0;
-	break;
+        /* Stops the song playing */
+        command[0] = COMMAND_TEMPO;
+        command[1] = 0;
+        break;
       case 0xff:
-	/* Stops the note on the current track */
-	command[0] = COMMAND_VELOCITY;
-	command[1] = 0;
-	break;
+        /* Stops the note on the current track */
+        command[0] = COMMAND_VELOCITY;
+        command[1] = 0;
+        break;
       default:
-	command[0] = 0;
-	command[1] = 0;
-	break;
+        command[0] = 0;
+        command[1] = 0;
+        break;
       }
     }
     break;
@@ -150,7 +150,7 @@ struct song *MMD2_convert_song(struct MMD2 *mmd)
   commandconversion[16] = COMMAND_SEND_MESSAGE;
   commandconversion[17] = COMMAND_NOT_DEFINED;
   commandconversion[18] = COMMAND_NOT_DEFINED;
-  commandconversion[19] = COMMAND_NOT_DEFINED;
+  commandconversion[19] = COMMAND_PITCH_WHEEL;
   commandconversion[20] = COMMAND_NOT_DEFINED;
   commandconversion[21] = COMMAND_NOT_DEFINED;
   commandconversion[22] = COMMAND_MIDI_CONTROLLERS + 7;
@@ -188,14 +188,14 @@ struct song *MMD2_convert_song(struct MMD2 *mmd)
       struct PlaySeq *PlaySeq = mmd->song->playseqtable[i];
       song->playseqs[i] = playseq;
       if (PlaySeq->name != NULL) {
-	gchar *name = g_convert(PlaySeq->name, strlen(PlaySeq->name), "UTF-8", "ISO-8859-1", NULL, NULL, NULL);
-	playseq->name = strdup(name);
-	g_free(name);
+        gchar *name = g_convert(PlaySeq->name, strlen(PlaySeq->name), "UTF-8", "ISO-8859-1", NULL, NULL, NULL);
+        playseq->name = strdup(name);
+        g_free(name);
       }
       playseq->length = PlaySeq->length;
       playseq->blocknumbers = (unsigned int *)calloc(playseq->length, sizeof(unsigned int));
       for (j = 0; j < song->playseqs[i]->length; j++)
-	playseq->blocknumbers[j] = PlaySeq->seq[j];
+        playseq->blocknumbers[j] = PlaySeq->seq[j];
     }
     /* Blocks */
     song->numblocks = mmd->song->numblocks;
@@ -205,42 +205,42 @@ struct song *MMD2_convert_song(struct MMD2 *mmd)
       struct MMD1Block *MMD1Block = mmd->blockarr[i];
       unsigned char *notes = (unsigned char *)MMD1Block + sizeof(struct MMD1Block);
       if (MMD1Block->info != NULL && MMD1Block->info->pagetable != NULL)
-	block = block_alloc(MMD1Block->numtracks, MMD1Block->lines + 1, MMD1Block->info->pagetable->num_pages + 1);
+        block = block_alloc(MMD1Block->numtracks, MMD1Block->lines + 1, MMD1Block->info->pagetable->num_pages + 1);
       else
-	block = block_alloc(MMD1Block->numtracks, MMD1Block->lines + 1, 1);
+        block = block_alloc(MMD1Block->numtracks, MMD1Block->lines + 1, 1);
 
       song->blocks[i] = block;
 
       if (MMD1Block->info != NULL && MMD1Block->info->blockname != NULL) {
-	gchar *name = g_convert(MMD1Block->info->blockname, strlen(MMD1Block->info->blockname), "UTF-8", "ISO-8859-1", NULL, NULL, NULL);
-	block->name = strdup(name);
-	g_free(name);
+        gchar *name = g_convert(MMD1Block->info->blockname, strlen(MMD1Block->info->blockname), "UTF-8", "ISO-8859-1", NULL, NULL, NULL);
+        block->name = strdup(name);
+        g_free(name);
       }
 
       for (j = 0; j < block->length; j++)
-	for (k = 0; k < block->tracks; k++) {
-	  block->notes[(j * block->tracks + k) * 2] = notes[(j * block->tracks + k) * 4];
-	  block->notes[(j * block->tracks + k) * 2 + 1] = notes[(j * block->tracks + k) * 4 + 1];
-	  block->commands[(j * block->tracks + k) * 2] = commandconversion[notes[(j * block->tracks + k) * 4 + 2]];
-	  block->commands[(j * block->tracks + k) * 2 + 1] = notes[(j * block->tracks + k) * 4 + 3];
+        for (k = 0; k < block->tracks; k++) {
+          block->notes[(j * block->tracks + k) * 2] = notes[(j * block->tracks + k) * 4];
+          block->notes[(j * block->tracks + k) * 2 + 1] = notes[(j * block->tracks + k) * 4 + 1];
+          block->commands[(j * block->tracks + k) * 2] = commandconversion[notes[(j * block->tracks + k) * 4 + 2]];
+          block->commands[(j * block->tracks + k) * 2 + 1] = notes[(j * block->tracks + k) * 4 + 3];
 
-	  /* Command values may need special handling */
-	  MMD2_commands_convert(&(block->commands[(j * block->tracks + k) * 2]), mmd->expdata->mmdcmd3x);
-	}
+          /* Command values may need special handling */
+          MMD2_commands_convert(&(block->commands[(j * block->tracks + k) * 2]), mmd->expdata->mmdcmd3x);
+        }
 
       if (MMD1Block->info != NULL && MMD1Block->info->pagetable != NULL) {
-	for (j = 0; j < MMD1Block->info->pagetable->num_pages; j++) {
-	  for (k = 0; k < (MMD1Block->lines + 1); k++)
-	    for (l = 0; l < MMD1Block->numtracks; l++) {
-	      unsigned char *commands = (unsigned char *)MMD1Block->info->pagetable->page[j];
+        for (j = 0; j < MMD1Block->info->pagetable->num_pages; j++) {
+          for (k = 0; k < (MMD1Block->lines + 1); k++)
+            for (l = 0; l < MMD1Block->numtracks; l++) {
+              unsigned char *commands = (unsigned char *)MMD1Block->info->pagetable->page[j];
 
-	      block->commands[(j + 1) * 2 * block->tracks * block->length + (k * block->tracks + l) * 2] = commandconversion[commands[(k * MMD1Block->numtracks + l) * 2]];
-	      block->commands[(j + 1) * 2 * block->tracks * block->length + (k * block->tracks + l) * 2 + 1] = commands[(k * MMD1Block->numtracks + l) * 2 + 1];
+              block->commands[(j + 1) * 2 * block->tracks * block->length + (k * block->tracks + l) * 2] = commandconversion[commands[(k * MMD1Block->numtracks + l) * 2]];
+              block->commands[(j + 1) * 2 * block->tracks * block->length + (k * block->tracks + l) * 2 + 1] = commands[(k * MMD1Block->numtracks + l) * 2 + 1];
 
-	      /* Command values may need special handling */
-	      MMD2_commands_convert(&(block->commands[(j + 1) * 2 * block->tracks * block->length + (k * block->tracks + l) * 2]), mmd->expdata->mmdcmd3x);
-	    }
-	}
+              /* Command values may need special handling */
+              MMD2_commands_convert(&(block->commands[(j + 1) * 2 * block->tracks * block->length + (k * block->tracks + l) * 2]), mmd->expdata->mmdcmd3x);
+            }
+        }
       }
     }
     /* Instruments */
@@ -249,18 +249,18 @@ struct song *MMD2_convert_song(struct MMD2 *mmd)
     for (i = 0; i < song->numinstruments; i++) {
       song->instruments[i] = instrument_alloc();
       if (mmd->expdata) {
-	gchar *name = g_convert(mmd->expdata->iinfo[i].name, strlen(mmd->expdata->iinfo[i].name), "UTF-8", "ISO-8859-1", NULL, NULL, NULL);
-	song->instruments[i]->name = strdup(name);
-	song->instruments[i]->outputs[0]->hold = mmd->expdata->exp_smp[i].hold;
-	g_free(name);
+        gchar *name = g_convert((char *)mmd->expdata->iinfo[i].name, strlen((char *)mmd->expdata->iinfo[i].name), "UTF-8", "ISO-8859-1", NULL, NULL, NULL);
+        song->instruments[i]->name = strdup(name);
+        song->instruments[i]->outputs[0]->hold = mmd->expdata->exp_smp[i].hold;
+        g_free(name);
       }
       song->instruments[i]->outputs[0]->midichannel = mmd->song->sample[i].midich - 1;
       song->instruments[i]->outputs[0]->midipreset = mmd->song->sample[i].midipreset;
       song->instruments[i]->outputs[0]->transpose = mmd->song->sample[i].strans;
       if (mmd->song->sample[i].svol == 64)
-	song->instruments[i]->outputs[0]->defaultvelocity = 127;
+        song->instruments[i]->outputs[0]->defaultvelocity = 127;
       else
-	song->instruments[i]->outputs[0]->defaultvelocity = mmd->song->sample[i].svol * 2;
+        song->instruments[i]->outputs[0]->defaultvelocity = mmd->song->sample[i].svol * 2;
     }
 
     /* Track volumes */
@@ -269,9 +269,9 @@ struct song *MMD2_convert_song(struct MMD2 *mmd)
     for (i = 0; i < song->maxtracks; i++) {
       song->tracks[i] = (struct track *)calloc(1, sizeof(struct track));
       if (mmd->song->trackvols[i] < 64)
-	song->tracks[i]->volume = mmd->song->trackvols[i] * 2;
+        song->tracks[i]->volume = mmd->song->trackvols[i] * 2;
       else
-	song->tracks[i]->volume = 127;
+        song->tracks[i]->volume = 127;
     }
     if (mmd->song->mastervol < 64)
       song->mastervolume = mmd->song->mastervol * 2;
@@ -287,16 +287,16 @@ struct song *MMD2_convert_song(struct MMD2 *mmd)
       song->nummessages = mmd->expdata->dumps->numdumps;
       song->messages = (struct message **)calloc(song->nummessages, sizeof(struct message *));
       for (i = 0; i < song->nummessages; i++) {
-	song->messages[i] = (struct message *)calloc(1, sizeof(struct message));
-	song->messages[i]->length = dumps[i]->length;
-	song->messages[i]->data = (unsigned char *)calloc(song->messages[i]->length, sizeof(unsigned char));
-	memcpy(song->messages[i]->data, dumps[i]->data, song->messages[i]->length);
+        song->messages[i] = (struct message *)calloc(1, sizeof(struct message));
+        song->messages[i]->length = dumps[i]->length;
+        song->messages[i]->data = (unsigned char *)calloc(song->messages[i]->length, sizeof(unsigned char));
+        memcpy(song->messages[i]->data, dumps[i]->data, song->messages[i]->length);
 
-	if (dumps[i]->ext_len >= 20) {
-	  gchar *name = g_convert(dumps[i]->name, strlen(dumps[i]->name), "UTF-8", "ISO-8859-1", NULL, NULL, NULL);
-	  song->messages[i]->name = strdup(name);
-	  g_free(name);
-	}
+        if (dumps[i]->ext_len >= 20) {
+          gchar *name = g_convert((char *)dumps[i]->name, strlen((char *)dumps[i]->name), "UTF-8", "ISO-8859-1", NULL, NULL, NULL);
+          song->messages[i]->name = strdup(name);
+          g_free(name);
+        }
       }
     }
   } else {
@@ -313,8 +313,15 @@ static void song_commands_convert(unsigned char *command)
 {
   switch (command[0]) {
   case 0x0c:
-    /* Fix volume range to 0..64 */
-    command[1] = (command[1] + 1) / 2;
+    /* If there is no note convert C00 to FFF */
+    if (command[-2] == 0 && command[1] == 0) {
+      command[0] = 0x0f;
+      command[1] = 0xff;
+    } else
+      command[1] = command[1] * 0x60 / 0x80;
+    break;
+  case 0x13:
+    command[1] = command[1] * 2 - 0x80;
     break;
   case 0x20:
     /* Delay */
@@ -356,7 +363,7 @@ struct MMD2 *song_convert_MMD2(struct song *song)
   /* Create a table for converting song commands; values need special handling */
   for (i = 0; i < 256; i++)
     commandconversion[i] = i;
-  commandconversion[COMMAND_PITCH_WHEEL] = 0x03;
+  commandconversion[COMMAND_PITCH_WHEEL] = 0x13;
   commandconversion[COMMAND_END_BLOCK] = 0x1d;
   commandconversion[COMMAND_PLAYSEQ_POSITION] = 0x0b;
   commandconversion[COMMAND_SEND_MESSAGE] = 0x10;
@@ -382,7 +389,7 @@ struct MMD2 *song_convert_MMD2(struct song *song)
   mmd->song->channels = 4;
   mmd->song->mix_echodepth = 2;
   mmd->song->mix_echolen = 150;
-  
+
   /* Song name */
   if (song->name != NULL) {
     gchar *name = g_convert(song->name, strlen(song->name), "ISO-8859-1", "UTF-8", NULL, NULL, NULL);
@@ -445,50 +452,50 @@ struct MMD2 *song_convert_MMD2(struct song *song)
 
     for (j = 0; j < (MMD1Block->lines + 1); j++)
       for (k = 0; k < MMD1Block->numtracks; k++) {
-	notes[(j * block->tracks + k) * 4] = block->notes[(j * block->tracks + k) * 2];
-	notes[(j * block->tracks + k) * 4 + 1] = block->notes[(j * block->tracks + k) * 2 + 1];
-	notes[(j * block->tracks + k) * 4 + 2] = commandconversion[block->commands[(j * block->tracks + k) * 2]];
-	notes[(j * block->tracks + k) * 4 + 3] = block->commands[(j * block->tracks + k) * 2 + 1];
+        notes[(j * block->tracks + k) * 4] = block->notes[(j * block->tracks + k) * 2];
+        notes[(j * block->tracks + k) * 4 + 1] = block->notes[(j * block->tracks + k) * 2 + 1];
+        notes[(j * block->tracks + k) * 4 + 2] = commandconversion[block->commands[(j * block->tracks + k) * 2]];
+        notes[(j * block->tracks + k) * 4 + 3] = block->commands[(j * block->tracks + k) * 2 + 1];
 
-	/* Convert MIDI controllers to MMD3x commands */
-	if (notes[(j * block->tracks + k) * 4 + 2] >= 0x80) {
-	  if (mmd->expdata->mmdcmd3x == NULL) {
-	    mmd->expdata->mmdcmd3x = (struct MMDMIDICmd3x *)calloc(1, sizeof(struct MMDMIDICmd3x));
-	    mmd->expdata->mmdcmd3x->num_of_settings = 15;
-	    mmd->expdata->mmdcmd3x->ctrlr_types = (unsigned char *)calloc(15, sizeof(unsigned char));
-	    mmd->expdata->mmdcmd3x->ctrlr_numbers = (unsigned short *)calloc(15, sizeof(unsigned short));
-	  }
+        /* Convert MIDI controllers to MMD3x commands */
+        if (notes[(j * block->tracks + k) * 4 + 2] >= 0x80) {
+          if (mmd->expdata->mmdcmd3x == NULL) {
+            mmd->expdata->mmdcmd3x = (struct MMDMIDICmd3x *)calloc(1, sizeof(struct MMDMIDICmd3x));
+            mmd->expdata->mmdcmd3x->num_of_settings = 15;
+            mmd->expdata->mmdcmd3x->ctrlr_types = (unsigned char *)calloc(15, sizeof(unsigned char));
+            mmd->expdata->mmdcmd3x->ctrlr_numbers = (unsigned short *)calloc(15, sizeof(unsigned short));
+          }
 
-	  /* Check if the controller already has a MMD3x command */
-	  for (l = 0; l < controllers_used; l++)
-	    if (mmd->expdata->mmdcmd3x->ctrlr_numbers[l] == notes[(j * block->tracks + k) * 4 + 2] - 0x80)
-	      break;
+          /* Check if the controller already has a MMD3x command */
+          for (l = 0; l < controllers_used; l++)
+            if (mmd->expdata->mmdcmd3x->ctrlr_numbers[l] == notes[(j * block->tracks + k) * 4 + 2] - 0x80)
+              break;
 
-	  if (l == controllers_used) {
-	    if (controllers_used < 15) {
-	      mmd->expdata->mmdcmd3x->ctrlr_numbers[l] = notes[(j * block->tracks + k) * 4 + 2] - 0x80;
-	      controllers_used++;
-	      notes[(j * block->tracks + k) * 4 + 2] = 0x31 + l;
-	    }
-	  } else
-	    notes[(j * block->tracks + k) * 4 + 2] = 0x31 + l;
-	}
+          if (l == controllers_used) {
+            if (controllers_used < 15) {
+              mmd->expdata->mmdcmd3x->ctrlr_numbers[l] = notes[(j * block->tracks + k) * 4 + 2] - 0x80;
+              controllers_used++;
+              notes[(j * block->tracks + k) * 4 + 2] = 0x31 + l;
+            }
+          } else
+            notes[(j * block->tracks + k) * 4 + 2] = 0x31 + l;
+        }
 
-	/* Command values may need special handling */
-	song_commands_convert(&(notes[(j * block->tracks + k) * 4 + 2]));
+        /* Command values may need special handling */
+        song_commands_convert(&(notes[(j * block->tracks + k) * 4 + 2]));
       }
 
     if (MMD1Block->info != NULL && MMD1Block->info->pagetable != NULL) {
       for (j = 0; j < MMD1Block->info->pagetable->num_pages; j++) {
-	MMD1Block->info->pagetable->page[j] = (unsigned short *)calloc(MMD1Block->numtracks * (MMD1Block->lines + 1) * 2, sizeof(unsigned short));
-	for (k = 0; k < (MMD1Block->lines + 1); k++)
-	  for (l = 0; l < MMD1Block->numtracks; l++) {
-	    unsigned char *commands = (unsigned char *)MMD1Block->info->pagetable->page[j];
-	    commands[(k * MMD1Block->numtracks + l) * 2] = commandconversion[block->commands[(j + 1) * 2 * block->tracks * block->length + (k * block->tracks + l) * 2]];
-	    commands[(k * MMD1Block->numtracks + l) * 2 + 1] = block->commands[(j + 1) * 2 * block->tracks * block->length + (k * block->tracks + l) * 2 + 1];
-	    /* Command values may need special handling */
-	    song_commands_convert(&(commands[(k * MMD1Block->numtracks + l) * 2]));
-	  }
+        MMD1Block->info->pagetable->page[j] = (unsigned short *)calloc(MMD1Block->numtracks * (MMD1Block->lines + 1) * 2, sizeof(unsigned short));
+        for (k = 0; k < (MMD1Block->lines + 1); k++)
+          for (l = 0; l < MMD1Block->numtracks; l++) {
+            unsigned char *commands = (unsigned char *)MMD1Block->info->pagetable->page[j];
+            commands[(k * MMD1Block->numtracks + l) * 2] = commandconversion[block->commands[(j + 1) * 2 * block->tracks * block->length + (k * block->tracks + l) * 2]];
+            commands[(k * MMD1Block->numtracks + l) * 2 + 1] = block->commands[(j + 1) * 2 * block->tracks * block->length + (k * block->tracks + l) * 2 + 1];
+            /* Command values may need special handling */
+            song_commands_convert(&(commands[(k * MMD1Block->numtracks + l) * 2]));
+          }
       }
     }
   }
@@ -506,7 +513,7 @@ struct MMD2 *song_convert_MMD2(struct song *song)
   mmd->expdata->iinfo = (struct MMDInstrInfo *)calloc(mmd->expdata->i_ext_entries, sizeof(struct MMDInstrInfo));
   for (i = 0; i < mmd->song->numsamples; i++) {
     gchar *name = g_convert(song->instruments[i]->name, strlen(song->instruments[i]->name), "ISO-8859-1", "UTF-8", NULL, NULL, NULL);
-    strncpy(mmd->expdata->iinfo[i].name, name, 40);
+    strncpy((char *)mmd->expdata->iinfo[i].name, name, 40);
     mmd->expdata->iinfo[i].name[39] = 0;
     mmd->expdata->exp_smp[i].hold = song->instruments[i]->outputs[0]->hold;
     mmd->song->sample[i].midich = song->instruments[i]->outputs[0]->midichannel + 1;
@@ -519,7 +526,7 @@ struct MMD2 *song_convert_MMD2(struct song *song)
   /* Track volumes */
   mmd->song->numtracks = song->maxtracks;
   mmd->song->trackvols = (unsigned char *)calloc(mmd->song->numtracks, sizeof(unsigned char));
-  mmd->song->trackpans = (unsigned char *)calloc(mmd->song->numtracks, sizeof(unsigned char));
+  mmd->song->trackpans = (char *)calloc(mmd->song->numtracks, sizeof(char));
   for (i = 0; i < mmd->song->numtracks; i++)
     mmd->song->trackvols[i] = (song->tracks[i]->volume + 1) / 2;
   mmd->song->mastervol = (song->mastervolume + 1) / 2;
@@ -535,11 +542,11 @@ struct MMD2 *song_convert_MMD2(struct song *song)
       dumps[i]->data = (unsigned char *)calloc(dumps[i]->length, sizeof(unsigned char));
       memcpy(dumps[i]->data, song->messages[i]->data, dumps[i]->length);
       if (song->messages[i]->name != NULL) {
-	gchar *name = g_convert(song->messages[i]->name, strlen(song->messages[i]->name), "ISO-8859-1", "UTF-8", NULL, NULL, NULL);
-	strncpy(dumps[i]->name, name, 20);
-	dumps[i]->name[19] = 0;
-	dumps[i]->ext_len = strlen(name);
-	g_free(name);
+        gchar *name = g_convert(song->messages[i]->name, strlen(song->messages[i]->name), "ISO-8859-1", "UTF-8", NULL, NULL, NULL);
+        strncpy((char *)dumps[i]->name, name, 20);
+        dumps[i]->name[19] = 0;
+        dumps[i]->ext_len = strlen(name);
+        g_free(name);
       }
     }
   }
@@ -556,7 +563,7 @@ struct smf *song_convert_smf(struct song *song)
   struct midi_interface *midi;
   int i;
   unsigned char endMTrk[3];
-  
+
   smf = calloc(1, sizeof(struct smf));
 
   /* Allocate the MThd chunk */
@@ -569,7 +576,7 @@ struct smf *song_convert_smf(struct song *song)
   smf->MThd->Format = 0;
   smf->MThd->NumTracks = 1;
   smf->MThd->Division = 24;
-  
+
   /* Allocate the MTrk chunks */
   smf->MTrks = calloc(smf->MThd->NumTracks, sizeof(struct smf_MTrk *));
   for (i = 0; i < smf->MThd->NumTracks; i++) {
@@ -579,7 +586,7 @@ struct smf *song_convert_smf(struct song *song)
     smf->MTrks[i]->ID[2] = 'r';
     smf->MTrks[i]->ID[3] = 'k';
   }
-  
+
   midi = midi_interface_alloc(MIDI_BUFFER, NULL);
   player_midi_export(song, midi);
   endMTrk[0] = 0xff;
@@ -589,6 +596,6 @@ struct smf *song_convert_smf(struct song *song)
   smf->MTrks[0]->Length = midi_get_buffer_length(midi);
   smf->MTrks[0]->Data = midi_get_buffer(midi);
   midi_interface_free(midi);
-  
+
   return smf;
 }
