@@ -1,6 +1,6 @@
 /* editor.c
  *
- * Copyright 2002-2006 Vesa Halttunen
+ * Copyright 2002-2011 Vesa Halttunen
  *
  * This file is part of Tutka.
  *
@@ -64,6 +64,10 @@ struct editor *editor_open()
 
   /* Open the MIDI subsystem */
   editor->midi = midi_open(editor, editor_midi_interface_get(editor, DIRECTION_IN, MIDIINTERFACES_AVAILABLE), editor_midi_interface_get(editor, DIRECTION_OUT, MIDIINTERFACES_AVAILABLE), editor_midi_interface_get(editor, DIRECTION_IN, MIDIINTERFACES_UNAVAILABLE), editor_midi_interface_get(editor, DIRECTION_OUT, MIDIINTERFACES_UNAVAILABLE));
+  if (editor->midi == NULL) {
+    editor_close(editor);
+    return NULL;
+  }
 
   /* Some interfaces may have been removed so refresh preferences */
   editor_midi_interface_set_from_current(editor);
@@ -86,10 +90,8 @@ void editor_song_open(struct editor *editor, char *filename)
     player_close(editor->player);
 
   /* Free the old song */
-  if (editor->song != NULL)
-    song_free(editor->song);
-  if (editor->filename != NULL)
-    free(editor->filename);
+  song_free(editor->song);
+  free(editor->filename);
 
   /* If a filename was supplied open the file */
   if (filename != NULL) {
@@ -119,7 +121,7 @@ void editor_song_open(struct editor *editor, char *filename)
       /* Figure out default MIDI interface number for new instruments */
       int i = 0;
       if (editor->midi->numoutputs > 1)
-	i = 1;
+        i = 1;
 
       editor->song = song_alloc();
       song_check_instrument(editor->song, 0, i);
@@ -174,7 +176,7 @@ void editor_close(struct editor *editor)
 #ifdef HAVE_JACK
   /* Close Jack */
   if (editor->jack != NULL)
-    jack_close(editor->jack);
+  jack_close(editor->jack);
 #endif
 
   /* Close MIDI */
@@ -194,20 +196,15 @@ void editor_close(struct editor *editor)
     g_mutex_free(editor->songmutex);
 
   /* Free the song */
-  if (editor->song != NULL)
-    song_free(editor->song);
+  song_free(editor->song);
 
   /* Free the filename */
-  if (editor->filename != NULL)
-    free(editor->filename);
+  free(editor->filename);
 
   /* Free the copy & paste buffers */
-  if (editor->copyblock != NULL)
-    block_free(editor->copyblock);
-  if (editor->copytrack != NULL)
-    block_free(editor->copytrack);
-  if (editor->copyarea != NULL)
-    block_free(editor->copyarea);
+  block_free(editor->copyblock);
+  block_free(editor->copytrack);
+  block_free(editor->copyarea);
 
   /* Unref GConf client */
   if (editor->gconfclient != NULL)
@@ -249,8 +246,7 @@ void editor_song_midi_interface_names_set(struct editor *editor)
     struct instrument *instrument = editor->song->instruments[i];
     for (j = 0; j < instrument->numoutputs; j++) {
       struct instrument_output *output = instrument->outputs[j];
-      if (output->midiinterfacename != NULL)
-	free(output->midiinterfacename);
+      free(output->midiinterfacename);
       output->midiinterfacename = midi_interface_name_get_by_number(editor->midi, output->midiinterface, DIRECTION_OUT);
     }
   }
@@ -389,8 +385,7 @@ void editor_set_line(struct editor *editor, int line)
     /* Bounds checking */
     while (line < 0)
       line += editor->song->blocks[editor->block]->length;
-    while (line >= editor->song->blocks[editor->block]->length)
-      line -= editor->song->blocks[editor->block]->length;
+    line %= editor->song->blocks[editor->block]->length;
 
     editor->line = line;
 
@@ -425,13 +420,13 @@ void editor_seek(struct editor *editor, int ticks)
     /* Look for commands that alter the playing position */
     for (i = 0; i < b->tracks; i++) {
       for (j = 0; j < b->commandpages; j++) {
-	unsigned char command = b->commands[j * b->tracks * b->length * 2 + (b->tracks * line + i) * 2];
-	unsigned char value = b->commands[j * b->tracks * b->length * 2 + (b->tracks * line + i) * 2 + 1];
+        unsigned char command = b->commands[j * b->tracks * b->length * 2 + (b->tracks * line + i) * 2];
+        unsigned char value = b->commands[j * b->tracks * b->length * 2 + (b->tracks * line + i) * 2 + 1];
 
-	if (tick == tpl - 1 && (command == COMMAND_END_BLOCK || command == COMMAND_PLAYSEQ_POSITION || command == COMMAND_TPL)) {
-	  postcommand = command;
-	  postvalue = value;
-	}
+        if (tick == tpl - 1 && (command == COMMAND_END_BLOCK || command == COMMAND_PLAYSEQ_POSITION || command == COMMAND_TPL)) {
+          postcommand = command;
+          postvalue = value;
+        }
       }
     }
 
@@ -445,60 +440,60 @@ void editor_seek(struct editor *editor, int ticks)
       /* Check if a playing position altering command was used */
       switch (postcommand) {
       case COMMAND_END_BLOCK:
-	line = postvalue;
-	position++;
-	if (position >= editor->song->playseqs[playseq]->length) {
-	  position = 0;
-	  section++;
-	  if (section >= editor->song->numsections)
-	    section = 0;
-	}
-	changeblock = 1;
-	break;
+        line = postvalue;
+        position++;
+        if (position >= editor->song->playseqs[playseq]->length) {
+          position = 0;
+          section++;
+          if (section >= editor->song->numsections)
+            section = 0;
+        }
+        changeblock = 1;
+        break;
       case COMMAND_PLAYSEQ_POSITION:
-	line = 0;
-	position = postvalue;
-	if (position >= editor->song->playseqs[playseq]->length) {
-	  position = 0;
-	  section++;
-	  if (section >= editor->song->numsections)
-	    section = 0;
-	}
-	changeblock = 1;
-	break;
+        line = 0;
+        position = postvalue;
+        if (position >= editor->song->playseqs[playseq]->length) {
+          position = 0;
+          section++;
+          if (section >= editor->song->numsections)
+            section = 0;
+        }
+        changeblock = 1;
+        break;
       case COMMAND_TPL:
-	tpl = postvalue;
-	/* Fall through to default: */
+        tpl = postvalue;
+        /* Fall through to default: */
       default:
-	if (line >= b->length) {
-	  line = 0;
-	  position++;
-	  if (position >= editor->song->playseqs[playseq]->length) {
-	    position = 0;
-	    section++;
-	    if (section >= editor->song->numsections)
-	      section = 0;
-	  }
-	  changeblock = 1;
-	}
-	break;
+        if (line >= b->length) {
+          line = 0;
+          position++;
+          if (position >= editor->song->playseqs[playseq]->length) {
+            position = 0;
+            section++;
+            if (section >= editor->song->numsections)
+              section = 0;
+          }
+          changeblock = 1;
+        }
+        break;
       }
 
       if (changeblock == 1) {
-	if (section >= editor->song->numsections)
-	  section = 0;
-	playseq = editor->song->sections[section];
+        if (section >= editor->song->numsections)
+          section = 0;
+        playseq = editor->song->sections[section];
+
+        if (position >= editor->song->playseqs[playseq]->length)
+          position = 0;
 
-	if (position >= editor->song->playseqs[playseq]->length)
-	  position = 0;
-          
         block = editor->song->playseqs[playseq]->blocknumbers[position];
       }
     }
 
     ticks--;
   }
-  
+
   /* Set the position in editor, player and gui */
   editor->section = section;
   editor->playseq = playseq;
@@ -583,7 +578,7 @@ void editor_midi_input(struct editor *editor, unsigned char *data)
   nullcheck_void(editor, editor_midi_input);
 
   /* FIX: gui->tracker should not be used here directly (or at all) */
-  Tracker *tracker = (Tracker *) glade_xml_get_widget(editor->gui->xml, "tracker");
+  Tracker *tracker = (Tracker *)glade_xml_get_widget(editor->gui->xml, "tracker");
   struct song *song = editor->song;
 
   switch (data[0] & 0xf0) {
@@ -591,16 +586,16 @@ void editor_midi_input(struct editor *editor, unsigned char *data)
     /* Note off */
     if (editor->edit) {
       if (editor->chord) {
-	/* Do not allow negative chord status: it should never happen anyway though */
-	if (editor->chordstatus > 0)
-	  editor->chordstatus--;
+        /* Do not allow negative chord status: it should never happen anyway though */
+        if (editor->chordstatus > 0)
+          editor->chordstatus--;
 
-	/* Move the cursor to the previous channel */
-	tracker_step_cursor_channel(tracker, -1);
+        /* Move the cursor to the previous channel */
+        tracker_step_cursor_channel(tracker, -1);
 
-	/* If all notes of the chord have been released move to the next line */
-	if (editor->chordstatus == 0)
-	  editor_set_line(editor, editor->line + editor->space);
+        /* If all notes of the chord have been released move to the next line */
+        if (editor->chordstatus == 0)
+          editor_set_line(editor, editor->line + editor->space);
       }
     }
     break;
@@ -611,12 +606,12 @@ void editor_midi_input(struct editor *editor, unsigned char *data)
       block_set_command_full(song->blocks[editor->block], editor->line, tracker->cursor_ch, editor->cmdpage, COMMAND_VELOCITY, data[2]);
       tracker_redraw_row(tracker, editor->line);
       if (editor->chord) {
-	editor->chordstatus++;
+        editor->chordstatus++;
 
-	/* Move the cursor to the next channel */
-	tracker_step_cursor_channel(tracker, 1);
+        /* Move the cursor to the next channel */
+        tracker_step_cursor_channel(tracker, 1);
       } else
-	editor_set_line(editor, editor->line + editor->space);
+        editor_set_line(editor, editor->line + editor->space);
     }
     break;
   case 0xb0:
@@ -672,7 +667,7 @@ void editor_song_unlock(struct editor *editor)
 {
   nullcheck_void(editor, editor_song_unlock)
 
-    g_mutex_unlock(editor->songmutex);
+  g_mutex_unlock(editor->songmutex);
 }
 
 /* Checks if the maximum number of tracks in the song has changed */
@@ -722,7 +717,6 @@ unsigned short editor_song_instrument_current_midiinterface_get(struct editor *e
   return editor->song->instruments[editor->instrument]->outputs[0]->midiinterface;
 }
 
-
 /* Sets the default velocity of the current instrument */
 void editor_song_instrument_current_defaultvelocity_set(struct editor *editor, unsigned char defaultvelocity)
 {
@@ -811,7 +805,8 @@ unsigned int editor_song_block_current_length_get(struct editor *editor)
   nullcheck_int(editor, editor_song_block_current_length_get);
 
   return editor->song->blocks[editor->block]->length;
-};
+}
+;
 
 /* Returns the number of tracks in the current block */
 unsigned int editor_song_block_current_tracks_get(struct editor *editor)
@@ -819,7 +814,8 @@ unsigned int editor_song_block_current_tracks_get(struct editor *editor)
   nullcheck_int(editor, editor_song_block_current_tracks_get);
 
   return editor->song->blocks[editor->block]->tracks;
-};
+}
+;
 
 /* Returns the number of command pages in the current block */
 unsigned int editor_song_block_current_commandpages_get(struct editor *editor)
@@ -827,7 +823,8 @@ unsigned int editor_song_block_current_commandpages_get(struct editor *editor)
   nullcheck_int(editor, editor_song_block_current_commandpages_get);
 
   return editor->song->blocks[editor->block]->commandpages;
-};
+}
+;
 
 /* Sets a note in the current block */
 void editor_song_block_current_note_set(struct editor *editor, unsigned int line, unsigned int track, unsigned char octave, unsigned char note, unsigned char instrument)
@@ -993,6 +990,56 @@ void editor_song_block_current_delete_line_track_all(struct editor *editor)
   g_mutex_unlock(editor->songmutex);
 }
 
+/* Inserts a track into the current block */
+void editor_song_block_current_insert_track(struct editor *editor, unsigned int track)
+{
+  nullcheck_void(editor, editor_song_block_current_insert_track);
+
+  struct song *song;
+  struct block *b;
+
+  g_mutex_lock(editor->songmutex);
+  song = editor->song;
+
+  /* Add a new track */
+  block_set_tracks(song->blocks[editor->block], song->blocks[editor->block]->tracks + 1);
+
+  /* Move tracks to the right */
+  b = block_copy(song->blocks[editor->block], track, 0, song->blocks[editor->block]->tracks - 1, song->blocks[editor->block]->length - 1);
+  block_paste(song->blocks[editor->block], b, track + 1, 0);
+
+  /* Clear the inserted track */
+  block_clear(song->blocks[editor->block], track, 0, track, song->blocks[editor->block]->length - 1);
+  block_free(b);
+  g_mutex_unlock(editor->songmutex);
+}
+
+/* Deletes a track from the current block */
+void editor_song_block_current_delete_track(struct editor *editor, unsigned int track)
+{
+  nullcheck_void(editor, editor_song_block_current_delete_track);
+
+  struct song *song;
+  struct block *b;
+
+  g_mutex_lock(editor->songmutex);
+  song = editor->song;
+
+  if (song->blocks[editor->block]->tracks > 1 && track < song->blocks[editor->block]->tracks) {
+    if (track < song->blocks[editor->block]->tracks - 1) {
+      /* Move tracks to the left */
+      b = block_copy(song->blocks[editor->block], track + 1, 0, song->blocks[editor->block]->tracks - 1, song->blocks[editor->block]->length - 1);
+      block_paste(song->blocks[editor->block], b, track, 0);
+      block_free(b);
+    }
+
+    /* Remove a track */
+    block_set_tracks(song->blocks[editor->block], song->blocks[editor->block]->tracks - 1);
+  }
+
+  g_mutex_unlock(editor->songmutex);
+}
+
 /* Sets the volume of a track */
 void editor_song_track_volume_set(struct editor *editor, unsigned int track, unsigned int volume)
 {
@@ -1023,7 +1070,7 @@ void editor_song_track_mute_set(struct editor *editor, unsigned int track, unsig
   player_lock(editor->player);
   player_stop_muted(editor->player);
   player_unlock(editor->player);
-  
+
   /* Refresh the tracker */
   gui_tracker_refresh(editor->gui, FALSE);
 }
@@ -1053,7 +1100,7 @@ void editor_song_track_solo_set(struct editor *editor, unsigned int track, unsig
   player_lock(editor->player);
   player_stop_muted(editor->player);
   player_unlock(editor->player);
-  
+
   /* Refresh the tracker */
   gui_tracker_refresh(editor->gui, FALSE);
 }
@@ -1097,8 +1144,7 @@ void editor_song_block_current_copyarea(struct editor *editor, int starttrack, i
 
   g_mutex_lock(editor->songmutex);
   /* Free the previous copy buffer if one exists */
-  if (editor->copyarea)
-    block_free(editor->copyarea);
+  block_free(editor->copyarea);
 
   editor->copyarea = block_copy(editor->song->blocks[editor->block], starttrack, startline, endtrack, endline);
   g_mutex_unlock(editor->songmutex);
@@ -1122,8 +1168,7 @@ void editor_song_block_current_copytrack(struct editor *editor, int track)
 
   g_mutex_lock(editor->songmutex);
   /* Free the previous copy buffer if one exists */
-  if (editor->copytrack)
-    block_free(editor->copytrack);
+  block_free(editor->copytrack);
 
   editor->copytrack = block_copy(editor->song->blocks[editor->block], track, 0, track, editor->song->blocks[editor->block]->length - 1);
   g_mutex_unlock(editor->songmutex);
@@ -1146,8 +1191,7 @@ void editor_song_block_current_copyblock(struct editor *editor)
   nullcheck_void(editor, editor_song_block_current_copyblock);
 
   g_mutex_lock(editor->songmutex);
-  if (editor->copyblock)
-    block_free(editor->copyblock);
+  block_free(editor->copyblock);
   editor->copyblock = block_copy(editor->song->blocks[editor->block], 0, 0, editor->song->blocks[editor->block]->tracks - 1, editor->song->blocks[editor->block]->length - 1);
   g_mutex_unlock(editor->songmutex);
 }
@@ -1229,8 +1273,8 @@ unsigned int editor_song_message_receive(struct editor *editor)
     } else {
       /* No bytes received: if autostop is not used and as many bytes received as requested, quit */
       if (!editor->receiveautostop && editor->receivedbytes == message->length) {
-	on_button_receive_stop_clicked(NULL, editor->gui);
-	return 1;
+        on_button_receive_stop_clicked(NULL, editor->gui);
+        return 1;
       }
     }
   }
@@ -1379,8 +1423,7 @@ void editor_song_instrument_current_name_set(struct editor *editor, char *name)
 
   g_mutex_lock(editor->songmutex);
   /* Free the old name if there is one */
-  if (editor->song->instruments[editor->instrument]->name != NULL)
-    free(editor->song->instruments[editor->instrument]->name);
+  free(editor->song->instruments[editor->instrument]->name);
 
   editor->song->instruments[editor->instrument]->name = strdup(name);
   g_mutex_unlock(editor->songmutex);
@@ -1405,8 +1448,7 @@ void editor_song_save(struct editor *editor, char *filename)
   /* If a filename was specified store it */
   if (filename != NULL) {
     /* Store the filename */
-    if (editor->filename != NULL)
-      free(editor->filename);
+    free(editor->filename);
     editor->filename = strdup(filename);
   } else if (editor->filename == NULL)
     /* Unable to save if there was no name and no name was specified */
@@ -1445,8 +1487,7 @@ void editor_song_name_set(struct editor *editor, char *name)
 
   g_mutex_lock(editor->songmutex);
   /* Free the old name if one exists */
-  if (editor->song->name != NULL)
-    free(editor->song->name);
+  free(editor->song->name);
 
   editor->song->name = strdup(name);
   g_mutex_unlock(editor->songmutex);
@@ -1635,8 +1676,7 @@ void editor_song_playseq_current_name_set(struct editor *editor, char *name)
 
   g_mutex_lock(editor->songmutex);
   /* Free the old name if one exists */
-  if (editor->song->playseqs[editor->playseq]->name)
-    free(editor->song->playseqs[editor->playseq]->name);
+  free(editor->song->playseqs[editor->playseq]->name);
 
   editor->song->playseqs[editor->playseq]->name = (char *)strdup(name);
   g_mutex_unlock(editor->songmutex);
@@ -1754,8 +1794,7 @@ void editor_song_message_name_set(struct editor *editor, unsigned int message, c
   nullcheck_void(editor, editor_song_message_name_set);
 
   g_mutex_lock(editor->songmutex);
-  if (editor->song->messages[message]->name)
-    free(editor->song->messages[message]->name);
+  free(editor->song->messages[message]->name);
 
   editor->song->messages[message]->name = (char *)strdup(name);
   g_mutex_unlock(editor->songmutex);
@@ -1963,10 +2002,14 @@ void editor_midi_interface_set_from_current(struct editor *editor)
   editor_midi_interface_set(editor, outputs_ok, DIRECTION_OUT, MIDIINTERFACES_AVAILABLE);
   editor_midi_interface_set(editor, inputs_error, DIRECTION_IN, MIDIINTERFACES_UNAVAILABLE);
   editor_midi_interface_set(editor, outputs_error, DIRECTION_OUT, MIDIINTERFACES_UNAVAILABLE);
+  free(outputs_ok);
+  free(inputs_ok);
+  free(outputs_error);
+  free(inputs_error);
 }
 
 /* Enables or disables a MIDI interface by client and port */
-unsigned int editor_midi_interface_status_set(struct editor *editor, unsigned int client, unsigned int port, unsigned int direction, unsigned int enabled)
+unsigned int editor_midi_interface_status_set(struct editor *editor, char *name, unsigned int direction, unsigned int enabled)
 {
   nullcheck_int(editor, editor_midi_interface_status_set);
 
@@ -1974,13 +2017,13 @@ unsigned int editor_midi_interface_status_set(struct editor *editor, unsigned in
 
   if (enabled)
     /* Subscribe */
-    result = midi_subscribe(editor->midi, client, port, direction);
+    result = midi_subscribe(editor->midi, name, direction);
   else {
     /* Store the old MIDI interface names */
     editor_song_midi_interface_names_set(editor);
 
     /* Unsubscribe */
-    result = 1 - midi_unsubscribe(editor->midi, client, port, direction);
+    result = 1 - midi_unsubscribe(editor->midi, name, direction);
 
     /* Make sure the MIDI interface numbers are still valid */
     editor_song_midi_interface_numbers_set(editor);
@@ -2081,7 +2124,7 @@ void editor_player_set_scheduler(struct editor *editor)
   if (editor->player == NULL)
     return;
 
-  if (editor->externalsync != EXTERNAL_SYNC_OFF)
+  if (editor->externalsync != EXTERNAL_SYNC_OFF && editor->externalsync != EXTERNAL_SYNC_JACK_START_ONLY)
     player_set_scheduler(editor->player, SCHED_EXTERNAL_SYNC);
   else {
     player_set_scheduler(editor->player, editor_scheduler_get(editor));
@@ -2090,7 +2133,6 @@ void editor_player_set_scheduler(struct editor *editor)
   }
 }
 
-
 unsigned int editor_player_get_external_sync(struct editor *editor)
 {
   nullcheck_int(editor, editor_player_get_external_sync);
