@@ -1,6 +1,6 @@
 /* gui.c
  *
- * Copyright 2002-2006 Vesa Halttunen
+ * Copyright 2002-2011 Vesa Halttunen
  *
  * This file is part of Tutka.
  *
@@ -20,7 +20,7 @@
  */
 
 #ifdef HAVE_CONFIG_H
-#  include <config.h>
+#include <config.h>
 #endif
 
 #include <string.h>
@@ -58,7 +58,7 @@ void gui_gladexmlconnectfunc(const gchar * handler_name, GObject * object, const
   GCallback func;
 
   /* Use the gmodule interface to resolve the function name to a pointer */
-  if (!g_module_symbol(g_module_open(NULL, 0), handler_name, (gpointer *) & func))
+  if (!g_module_symbol(g_module_open(NULL, 0), handler_name, (gpointer *)&func))
     g_warning("could not find signal handler '%s'.", handler_name);
 
   /* Connect the signal */
@@ -66,8 +66,7 @@ void gui_gladexmlconnectfunc(const gchar * handler_name, GObject * object, const
     g_signal_connect_after(object, signal_name, func, user_data);
   else
     g_signal_connect(object, signal_name, func, user_data);
-
-};
+}
 
 void gtk_main_quit()
 {
@@ -229,6 +228,12 @@ struct gui *gui_open(struct editor *editor)
   g_object_set_data(G_OBJECT(renderer), "direction", GUINT_TO_POINTER(DIRECTION_OUT));
   g_signal_connect(renderer, "toggled", G_CALLBACK(on_cell_preferences_midi_interfaces_toggled), gui);
   gtk_tree_view_append_column(treeview, new_column("Enabled", renderer, "active", GUI_PREFERENCES_MIDI_INTERFACES_COLUMN_ENABLED, NULL));
+  renderer = gtk_cell_renderer_text_new();
+  g_object_set_data(G_OBJECT(renderer), "column", GUINT_TO_POINTER(GUI_PREFERENCES_MIDI_INTERFACES_COLUMN_CLIENT));
+  gtk_tree_view_append_column(treeview, new_column("Client", renderer, "text", GUI_PREFERENCES_MIDI_INTERFACES_COLUMN_CLIENT, NULL));
+  renderer = gtk_cell_renderer_text_new();
+  g_object_set_data(G_OBJECT(renderer), "column", GUINT_TO_POINTER(GUI_PREFERENCES_MIDI_INTERFACES_COLUMN_PORT));
+  gtk_tree_view_append_column(treeview, new_column("Port", renderer, "text", GUI_PREFERENCES_MIDI_INTERFACES_COLUMN_PORT, NULL));
 
   store = gtk_list_store_new(4, G_TYPE_STRING, G_TYPE_BOOLEAN, G_TYPE_INT, G_TYPE_INT);
   treeview = GTK_TREE_VIEW(glade_xml_get_widget(gui->xml, "treeview_preferences_midi_interfaces_input"));
@@ -244,6 +249,12 @@ struct gui *gui_open(struct editor *editor)
   g_object_set_data(G_OBJECT(renderer), "direction", GUINT_TO_POINTER(DIRECTION_IN));
   g_signal_connect(renderer, "toggled", G_CALLBACK(on_cell_preferences_midi_interfaces_toggled), gui);
   gtk_tree_view_append_column(treeview, new_column("Enabled", renderer, "active", GUI_PREFERENCES_MIDI_INTERFACES_COLUMN_ENABLED, NULL));
+  renderer = gtk_cell_renderer_text_new();
+  g_object_set_data(G_OBJECT(renderer), "column", GUINT_TO_POINTER(GUI_PREFERENCES_MIDI_INTERFACES_COLUMN_CLIENT));
+  gtk_tree_view_append_column(treeview, new_column("Client", renderer, "text", GUI_PREFERENCES_MIDI_INTERFACES_COLUMN_CLIENT, NULL));
+  renderer = gtk_cell_renderer_text_new();
+  g_object_set_data(G_OBJECT(renderer), "column", GUINT_TO_POINTER(GUI_PREFERENCES_MIDI_INTERFACES_COLUMN_PORT));
+  gtk_tree_view_append_column(treeview, new_column("Port", renderer, "text", GUI_PREFERENCES_MIDI_INTERFACES_COLUMN_PORT, NULL));
 
   /* Set the keyboard octave */
   gui->editor->octave = 3;
@@ -269,11 +280,13 @@ struct gui *gui_open(struct editor *editor)
   if (songpath != NULL) {
     gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(glade_xml_get_widget(gui->xml, "filechooserdialog_file_open")), songpath);
     gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(glade_xml_get_widget(gui->xml, "filechooserdialog_file_saveas")), songpath);
+    free(songpath);
   }
   messagepath = editor_message_path_get(gui->editor);
   if (messagepath != NULL) {
     gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(glade_xml_get_widget(gui->xml, "filechooserdialog_message_load")), messagepath);
     gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(glade_xml_get_widget(gui->xml, "filechooserdialog_message_save")), messagepath);
+    free(messagepath);
   }
 
   /* Add icons */
@@ -284,7 +297,7 @@ struct gui *gui_open(struct editor *editor)
     gtk_window_set_default_icon_list(g_list_append(NULL, pixbuf));
 
   /* Set tracker properties */
-  tracker = (Tracker *) glade_xml_get_widget(gui->xml, "tracker");
+  tracker = (Tracker *)glade_xml_get_widget(gui->xml, "tracker");
   fontname = editor_tracker_font_get(gui->editor);
   if (fontname != NULL && strlen(fontname) > 0)
     tracker_set_font(tracker, fontname);
@@ -292,9 +305,13 @@ struct gui *gui_open(struct editor *editor)
     tracker_set_font(tracker, "Monospace 9");
 
 #ifdef HAVE_JACK
-  if (editor->jack == NULL)
+  if (editor->jack == NULL) {
+#endif
+  gtk_widget_hide(glade_xml_get_widget(gui->xml, "menuitem_settings_externalsync_jack"));
+  gtk_widget_hide(glade_xml_get_widget(gui->xml, "menuitem_settings_externalsync_jack_start_only"));
+#ifdef HAVE_JACK
+}
 #endif
-    gtk_widget_hide(glade_xml_get_widget(gui->xml, "menuitem_settings_externalsync_jack"));
 
   return gui;
 }
@@ -327,7 +344,7 @@ void gui_set_saveas_filename(struct gui *gui, char *filename)
 void gui_refresh_all(struct gui *gui)
 {
   /* Reset the tracker */
-  Tracker *tracker = (Tracker *) glade_xml_get_widget(gui->xml, "tracker");
+  Tracker *tracker = (Tracker *)glade_xml_get_widget(gui->xml, "tracker");
 
   tracker_reset(tracker);
   tracker_set_song(tracker, gui->editor->song);
@@ -367,12 +384,12 @@ void gui_refresh_all(struct gui *gui)
 
 Tracker *gui_get_tracker(struct gui *gui)
 {
-  return (Tracker *) glade_xml_get_widget(gui->xml, "tracker");
+  return (Tracker *)glade_xml_get_widget(gui->xml, "tracker");
 }
 
 int gui_get_tracker_position(struct gui *gui)
 {
-  Tracker *tracker = (Tracker *) glade_xml_get_widget(gui->xml, "tracker");
+  Tracker *tracker = (Tracker *)glade_xml_get_widget(gui->xml, "tracker");
 
   return tracker->patpos;
 }
@@ -593,26 +610,26 @@ void gui_instrument_refresh(struct gui *gui)
 
     if (gui->editor->midi->outputs[i]->type != MIDI_ALSA) {
       if (gui->editor->midi->outputs[i]->name != NULL)
-	snprintf(midiinterface, 256, "%s", gui->editor->midi->outputs[i]->name);
+        snprintf(midiinterface, 256, "%s", gui->editor->midi->outputs[i]->name);
       else
-	snprintf(midiinterface, 256, "Unknown");
+        snprintf(midiinterface, 256, "Unknown");
       gtk_list_store_append(store, &iter);
       gtk_list_store_set(store, &iter, 0, midiinterface, 1, i, -1);
       if (i == current)
-	selected = n;
+        selected = n;
       n++;
     } else {
       struct midi_alsa *alsa = (struct midi_alsa *)gui->editor->midi->outputs[i]->data;
       if (alsa->subs != NULL) {
-	if (gui->editor->midi->outputs[i]->name != NULL)
-	  snprintf(midiinterface, 256, "%s (%d:%d)", gui->editor->midi->outputs[i]->name, alsa->client, alsa->port);
-	else
-	  snprintf(midiinterface, 256, "Unknown (%d:%d)", alsa->client, alsa->port);
-	gtk_list_store_append(store, &iter);
-	gtk_list_store_set(store, &iter, 0, midiinterface, 1, i, -1);
-	if (i == current)
-	  selected = n;
-	n++;
+        if (gui->editor->midi->outputs[i]->name != NULL)
+          snprintf(midiinterface, 256, "%s (%d:%d)", gui->editor->midi->outputs[i]->name, alsa->client, alsa->port);
+        else
+          snprintf(midiinterface, 256, "Unknown (%d:%d)", alsa->client, alsa->port);
+        gtk_list_store_append(store, &iter);
+        gtk_list_store_set(store, &iter, 0, midiinterface, 1, i, -1);
+        if (i == current)
+          selected = n;
+        n++;
       }
     }
   }
@@ -770,6 +787,7 @@ void gui_blocklist_refresh(struct gui *gui, gboolean recreate)
   gtk_tree_selection_select_path(gtk_tree_view_get_selection(treeview), treepath);
   gtk_tree_view_scroll_to_cell(treeview, treepath, NULL, TRUE, 0.5, 0);
   gtk_tree_view_set_cursor(treeview, treepath, NULL, FALSE);
+  gtk_tree_path_free(treepath);
 
   /* Make the delete button sensitive/insensitive */
   if (editor_song_numblocks_get(gui->editor) > 1)
@@ -809,6 +827,7 @@ void gui_playseqlist_refresh(struct gui *gui, gboolean recreate)
   gtk_tree_selection_select_path(gtk_tree_view_get_selection(treeview), treepath);
   gtk_tree_view_scroll_to_cell(treeview, treepath, NULL, TRUE, 0.5, 0);
   gtk_tree_view_set_cursor(treeview, treepath, NULL, FALSE);
+  gtk_tree_path_free(treepath);
 
   /* Make the delete playing sequence button sensitive/insensitive */
   if (editor_song_numplayseqs_get(gui->editor) > 1)
@@ -864,6 +883,7 @@ void gui_playseq_refresh(struct gui *gui, gboolean recreate)
   gtk_tree_selection_select_path(gtk_tree_view_get_selection(treeview), treepath);
   gtk_tree_view_scroll_to_cell(treeview, treepath, NULL, TRUE, 0.5, 0);
   gtk_tree_view_set_cursor(treeview, treepath, NULL, FALSE);
+  gtk_tree_path_free(treepath);
 
   /* Set window title */
   gtk_window_set_title(GTK_WINDOW(glade_xml_get_widget(gui->xml, "dialog_playseq")), title);
@@ -908,6 +928,7 @@ void gui_sectionlist_refresh(struct gui *gui, gboolean recreate)
   gtk_tree_selection_select_path(gtk_tree_view_get_selection(treeview), treepath);
   gtk_tree_view_scroll_to_cell(treeview, treepath, NULL, TRUE, 0.5, 0);
   gtk_tree_view_set_cursor(treeview, treepath, NULL, FALSE);
+  gtk_tree_path_free(treepath);
 
   /* Make the delete button sensitive/insensitive */
   if (editor_song_numsections_get(gui->editor) > 1)
@@ -973,7 +994,7 @@ void gui_trackvolumes_refresh(struct gui *gui)
   char track[10];
 
   /* Destroy all children */
-  gtk_container_foreach(GTK_CONTAINER(table), (GtkCallback) gtk_widget_destroy, NULL);
+  gtk_container_foreach(GTK_CONTAINER(table), (GtkCallback)gtk_widget_destroy, NULL);
   gtk_table_resize(GTK_TABLE(table), 5, n);
 
   /* Create widgets for each channel */
@@ -990,8 +1011,8 @@ void gui_trackvolumes_refresh(struct gui *gui)
     gtk_table_attach(GTK_TABLE(table), x, i, i + 1, 0, 1, GTK_EXPAND | GTK_FILL, GTK_FILL, 2, 4);
 
     /* Volume sliders */
-    w = (GtkWidget *) gtk_adjustment_new(volume, 0, 127, 1, 8, 0);
-    g_object_set_data(G_OBJECT(w), "track", (gpointer) i);
+    w = (GtkWidget *)gtk_adjustment_new(volume, 0, 127, 1, 8, 0);
+    g_object_set_data(G_OBJECT(w), "track", (gpointer)i);
     g_signal_connect(G_OBJECT(w), "value_changed", G_CALLBACK(on_adjustment_trackvolumes_volume_changed), gui);
     x = gtk_vscale_new(GTK_ADJUSTMENT(w));
     gtk_range_set_inverted(GTK_RANGE(x), TRUE);
@@ -1000,7 +1021,7 @@ void gui_trackvolumes_refresh(struct gui *gui)
 
     /* Mute buttons */
     w = gtk_check_button_new_with_label("Mute");
-    g_object_set_data(G_OBJECT(w), "track", (gpointer) i);
+    g_object_set_data(G_OBJECT(w), "track", (gpointer)i);
     if (mute)
       gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w), TRUE);
     g_signal_connect(G_OBJECT(w), "toggled", G_CALLBACK(on_togglebutton_trackvolumes_mute_toggled), gui);
@@ -1008,7 +1029,7 @@ void gui_trackvolumes_refresh(struct gui *gui)
 
     /* Solo buttons */
     w = gtk_check_button_new_with_label("Solo");
-    g_object_set_data(G_OBJECT(w), "track", (gpointer) i);
+    g_object_set_data(G_OBJECT(w), "track", (gpointer)i);
     if (solo)
       gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w), TRUE);
     g_signal_connect(G_OBJECT(w), "toggled", G_CALLBACK(on_togglebutton_trackvolumes_solo_toggled), gui);
@@ -1017,7 +1038,7 @@ void gui_trackvolumes_refresh(struct gui *gui)
     /* Track names */
     x = gtk_entry_new();
     gtk_entry_set_width_chars(GTK_ENTRY(x), 7);
-    g_object_set_data(G_OBJECT(x), "track", (gpointer) i);
+    g_object_set_data(G_OBJECT(x), "track", (gpointer)i);
     if (name != NULL)
       gtk_entry_set_text(GTK_ENTRY(x), name);
     else
@@ -1054,21 +1075,15 @@ void gui_preferences_refresh(struct gui *gui)
       struct midi_alsa *alsa = (struct midi_alsa *)gui->editor->midi->outputs[i]->data;
 
       if (alsa->subs != NULL)
-	enabled = TRUE;
+        enabled = TRUE;
 
       client = alsa->client;
       port = alsa->port;
-
-      if (gui->editor->midi->outputs[i]->name != NULL)
-	snprintf(name, 256, "%s (%d:%d)", gui->editor->midi->outputs[i]->name, client, port);
-      else
-	snprintf(name, 256, "Unknown (%d:%d)", client, port);
-    } else {
-      if (gui->editor->midi->outputs[i]->name != NULL)
-	snprintf(name, 256, "%s", gui->editor->midi->outputs[i]->name);
-      else
-	snprintf(name, 256, "Unknown");
     }
+    if (gui->editor->midi->outputs[i]->name != NULL)
+      snprintf(name, 256, "%s", gui->editor->midi->outputs[i]->name);
+    else
+      snprintf(name, 256, "Unknown");
 
     gtk_list_store_append(store, &iter);
     gtk_list_store_set(store, &iter, GUI_PREFERENCES_MIDI_INTERFACES_COLUMN_NAME, name, GUI_PREFERENCES_MIDI_INTERFACES_COLUMN_ENABLED, enabled, GUI_PREFERENCES_MIDI_INTERFACES_COLUMN_CLIENT, client, GUI_PREFERENCES_MIDI_INTERFACES_COLUMN_PORT, port, -1);
@@ -1086,21 +1101,15 @@ void gui_preferences_refresh(struct gui *gui)
       struct midi_alsa *alsa = (struct midi_alsa *)gui->editor->midi->inputs[i]->data;
 
       if (alsa->subs != NULL)
-	enabled = TRUE;
+        enabled = TRUE;
 
       client = alsa->client;
       port = alsa->port;
-
-      if (gui->editor->midi->inputs[i]->name != NULL)
-	snprintf(name, 256, "%s (%d:%d)", gui->editor->midi->inputs[i]->name, client, port);
-      else
-	snprintf(name, 256, "Unknown (%d:%d)", client, port);
-    } else {
-      if (gui->editor->midi->inputs[i]->name != NULL)
-	snprintf(name, 256, "%s", gui->editor->midi->inputs[i]->name);
-      else
-	snprintf(name, 256, "Unknown");
     }
+    if (gui->editor->midi->inputs[i]->name != NULL)
+      snprintf(name, 256, "%s", gui->editor->midi->inputs[i]->name);
+    else
+      snprintf(name, 256, "Unknown");
 
     gtk_list_store_append(store, &iter);
     gtk_list_store_set(store, &iter, GUI_PREFERENCES_MIDI_INTERFACES_COLUMN_NAME, name, GUI_PREFERENCES_MIDI_INTERFACES_COLUMN_ENABLED, enabled, GUI_PREFERENCES_MIDI_INTERFACES_COLUMN_CLIENT, client, GUI_PREFERENCES_MIDI_INTERFACES_COLUMN_PORT, port, -1);
@@ -1127,7 +1136,7 @@ void gui_receive_refresh(struct gui *gui, int bytes, int total)
   } else {
     if (bytes >= 0) {
       snprintf(b, 40, "Bytes received: %d/%d", bytes, total);
-      gtk_progress_bar_set_fraction(pbar, (gdouble) bytes / total);
+      gtk_progress_bar_set_fraction(pbar, (gdouble)bytes / total);
     } else
       snprintf(b, 40, "Bytes received: 0/0");
   }
