@@ -1,6 +1,6 @@
 /* midi.c
  *
- * Copyright 2002-2006 Vesa Halttunen
+ * Copyright 2002-2011 Vesa Halttunen
  *
  * This file is part of Tutka.
  *
@@ -29,8 +29,11 @@
 #include "player.h"
 
 static char *string_remove_string(char *from, char *toremove);
-static char *string_append_char(char *to, char append);
 static char *string_append_string(char *to, char *append);
+/* Not used */
+/*
+static char *string_append_char(char *to, char append);
+*/
 
 /* Open the MIDI subsystem */
 struct midi *midi_open(struct editor *editor, char *in, char *out, char *unavailablein, char *unavailableout)
@@ -40,7 +43,7 @@ struct midi *midi_open(struct editor *editor, char *in, char *out, char *unavail
   snd_seq_port_subscribe_t *subs;
 
   midi->editor = editor;
-  
+
   int err = snd_seq_open(&midi->seq, "default", SND_SEQ_OPEN_DUPLEX, SND_SEQ_NONBLOCK);
   if (err < 0) {
     fprintf(stderr, "Couldn't create client: %s", snd_strerror(err));
@@ -85,7 +88,7 @@ struct midi *midi_open(struct editor *editor, char *in, char *out, char *unavail
 
   /* Get MIDI interfaces */
   midi_interfaces_refresh(midi);
-  
+
   return midi;
 }
 
@@ -94,59 +97,69 @@ void midi_close(struct midi *midi)
 {
   nullcheck_void(midi, midi_close);
 
-  snd_seq_addr_t sender, dest;
-  snd_seq_port_subscribe_t *subs;
+  if (midi->seq != NULL) {
+    snd_seq_addr_t sender, dest;
+    snd_seq_port_subscribe_t *subs;
 
-  /* Unsubscribe the announce port */
-  sender.client = SND_SEQ_CLIENT_SYSTEM;
-  sender.port = SND_SEQ_PORT_SYSTEM_ANNOUNCE;
-  dest.client = midi->client;
-  dest.port = midi->port;
-  snd_seq_port_subscribe_alloca(&subs);
-  snd_seq_port_subscribe_set_sender(subs, &sender);
-  snd_seq_port_subscribe_set_dest(subs, &dest);
-  snd_seq_unsubscribe_port(midi->seq, subs);
+    /* Unsubscribe the announce port */
+    sender.client = SND_SEQ_CLIENT_SYSTEM;
+    sender.port = SND_SEQ_PORT_SYSTEM_ANNOUNCE;
+    dest.client = midi->client;
+    dest.port = midi->port;
+    snd_seq_port_subscribe_alloca(&subs);
+    snd_seq_port_subscribe_set_sender(subs, &sender);
+    snd_seq_port_subscribe_set_dest(subs, &dest);
+    snd_seq_unsubscribe_port(midi->seq, subs);
 
-  /* Unsubscribe and free all interfaces */
-  midi_unsubscribe_all(midi);
+    /* Unsubscribe and free all interfaces */
+    midi_unsubscribe_all(midi);
 
-  /* Free MIDI event encoder and decoder */
-  if (midi->encoder != NULL)
-    snd_midi_event_free(midi->encoder);
-  if (midi->decoder != NULL)
-    snd_midi_event_free(midi->decoder);
+    /* Free MIDI event encoder and decoder */
+    if (midi->encoder != NULL)
+      snd_midi_event_free(midi->encoder);
+    if (midi->decoder != NULL)
+      snd_midi_event_free(midi->decoder);
 
-  /* Delete port */
-  if (midi->port >= 0)
-    snd_seq_delete_simple_port(midi->seq, midi->port);
+    /* Delete port */
+    if (midi->port >= 0)
+      snd_seq_delete_simple_port(midi->seq, midi->port);
 
-  /* Delete client */
-  if (midi->seq != NULL)
+    /* Delete client */
     snd_seq_close(midi->seq);
+  }
+
+  /* Free interface names */
+  free(midi->outputs_ok);
+  free(midi->inputs_ok);
+  free(midi->outputs_error);
+  free(midi->inputs_error);
 
   free(midi);
 }
 
 /* Subscribe to a client/port */
-unsigned int midi_subscribe(struct midi *midi, int client, int port, unsigned int direction)
+unsigned int midi_subscribe(struct midi *midi, char *name, unsigned int direction)
 {
   nullcheck_int(midi, midi_subscribe);
-  struct midi_interface *iface = midi_interface_get_by_client_and_port(midi, client, port, direction);
+  nullcheck_int(name, midi_subscribe);
+
+  struct midi_interface *iface = midi_interface_get_by_name(midi, name, direction);
   int result = 0;
-  char temp[13];
+  char *temp;
 
   if (iface != NULL) {
     snd_seq_addr_t sender, dest;
     snd_seq_port_subscribe_t *subs;
+    struct midi_alsa *alsa = (struct midi_alsa *)iface->data;
 
     if (direction == DIRECTION_OUT) {
       sender.client = midi->client;
       sender.port = midi->port;
-      dest.client = client;
-      dest.port = port;
+      dest.client = alsa->client;
+      dest.port = alsa->port;
     } else {
-      sender.client = client;
-      sender.port = port;
+      sender.client = alsa->client;
+      sender.port = alsa->port;
       dest.client = midi->client;
       dest.port = midi->port;
     }
@@ -156,13 +169,15 @@ unsigned int midi_subscribe(struct midi *midi, int client, int port, unsigned in
 
     /* Try subscribing to the port */
     if (snd_seq_subscribe_port(midi->seq, subs) == 0) {
-      struct midi_alsa *alsa = (struct midi_alsa *)iface->data;
       alsa->subs = subs;
       result = 1;
     }
   }
 
-  snprintf(temp, 13, "%d:%d,", client, port);
+  temp = malloc(strlen(name) + 2);
+  strcpy(temp, name);
+  strcat(temp, ",");
+
   if (result) {
     /* Add to the subscribed strings */
     if (direction == DIRECTION_OUT)
@@ -177,16 +192,18 @@ unsigned int midi_subscribe(struct midi *midi, int client, int port, unsigned in
       midi->inputs_error = string_append_string(midi->inputs_error, temp);
   }
 
+  free(temp);
+
   return result;
 }
 
 /* Unsubscribe from a client/port */
-unsigned int midi_unsubscribe(struct midi *midi, int client, int port, unsigned int direction)
+unsigned int midi_unsubscribe(struct midi *midi, char *name, unsigned int direction)
 {
   nullcheck_int(midi, midi_unsubscribe);
 
-  struct midi_interface *iface = midi_interface_get_by_client_and_port(midi, client, port, direction);
-  char temp[13];
+  struct midi_interface *iface = midi_interface_get_by_name(midi, name, direction);
+  char *temp;
 
   if (iface == NULL)
     return 0;
@@ -199,11 +216,11 @@ unsigned int midi_unsubscribe(struct midi *midi, int client, int port, unsigned
     if (direction == DIRECTION_OUT) {
       sender.client = midi->client;
       sender.port = midi->port;
-      dest.client = client;
-      dest.port = port;
+      dest.client = alsa->client;
+      dest.port = alsa->port;
     } else {
-      sender.client = client;
-      sender.port = port;
+      sender.client = alsa->client;
+      sender.port = alsa->port;
       dest.client = midi->client;
       dest.port = midi->port;
     }
@@ -216,11 +233,16 @@ unsigned int midi_unsubscribe(struct midi *midi, int client, int port, unsigned
       alsa->subs = NULL;
 
       /* Remove from the subscribed strings */
-      snprintf(temp, 13, "%d:%d,", client, port);
+      temp = malloc(strlen(name) + 2);
+      strcpy(temp, name);
+      strcat(temp, ",");
+
       if (direction == DIRECTION_OUT)
-	midi->outputs_ok = string_remove_string(midi->outputs_ok, temp);
+        midi->outputs_ok = string_remove_string(midi->outputs_ok, temp);
       else
-	midi->inputs_ok = string_remove_string(midi->inputs_ok, temp);
+        midi->inputs_ok = string_remove_string(midi->inputs_ok, temp);
+
+      free(temp);
 
       return 1;
     }
@@ -252,12 +274,11 @@ struct midi_interface *midi_interface_alloc(unsigned int type, char *name)
 /* Free a MIDI interface */
 void midi_interface_free(struct midi_interface *midi)
 {
-  nullcheck_void(midi, midi_interface_free);
-
-  if (midi->name != NULL)
+  if (midi != NULL) {
     free(midi->name);
-  if (midi->data != NULL)
     free(midi->data);
+    free(midi);
+  }
 }
 
 /* Get MIDI interface from available interfaces by client and port */
@@ -280,33 +301,54 @@ struct midi_interface *midi_interface_get_by_client_and_port(struct midi *midi,
     if (interfaces[i]->type == MIDI_ALSA) {
       struct midi_alsa *alsa = (struct midi_alsa *)interfaces[i]->data;
       if (alsa->client == client && alsa->port == port)
-	return interfaces[i];
+        return interfaces[i];
     }
   }
 
   return NULL;
 }
 
+/* Gets an MIDI interface by name from available interfaces */
+struct midi_interface *midi_interface_get_by_name(struct midi *midi, char *name, unsigned int direction)
+{
+  nullcheck_pointer(midi, midi_interface_get_by_name);
+
+  /* Use the null interface if no name specified */
+  if (name == NULL)
+    return NULL;
+
+  struct midi_interface **interfaces;
+  int i, n;
+
+  if (direction == DIRECTION_OUT) {
+    interfaces = midi->outputs;
+    n = midi->numoutputs;
+  } else {
+    interfaces = midi->inputs;
+    n = midi->numinputs;
+  }
+
+  for (i = 0; i < n; i++) {
+    struct midi_interface *iface = interfaces[i];
+    if (iface->name != NULL && strcmp(iface->name, name) == 0)
+      return interfaces[i];
+  }
+
+  /* If no interface is found use the null interface */
+  return NULL;
+}
+
 /* Gets an MIDI interface number by name from the subscribed interfaces */
 int midi_interface_number_get_by_name(struct midi *midi, char *name, unsigned int direction)
 {
-  nullcheck_int(midi, midi_interface_get_by_name);
+  nullcheck_int(midi, midi_interface_number_get_by_name);
 
   /* Use the null interface if no name specified */
   if (name == NULL)
     return 0;
 
-  int i, n, client = 0, port = 0;
+  int i, n;
   struct midi_interface **interfaces;
-  char *c = name;
-
-  while (*c != 0 && *c != ':')
-    c++;
-  if (*c == ':') {
-    *c++ = 0;
-    port = atoi(c);
-  }
-  client = atoi(name);
 
   if (direction == DIRECTION_OUT) {
     n = midi->numoutputs;
@@ -318,11 +360,8 @@ int midi_interface_number_get_by_name(struct midi *midi, char *name, unsigned in
 
   for (i = 1; i < n; i++) {
     struct midi_interface *iface = interfaces[i];
-    if (iface->type == MIDI_ALSA) {
-      struct midi_alsa *alsa = (struct midi_alsa *)iface->data;
-      if (alsa->client == client && alsa->port == port && alsa->subs != NULL)
-	return i;
-    }
+    if (iface->name != NULL && strcmp(iface->name, name) == 0)
+      return i;
   }
 
   /* If no interface is found use the null interface */
@@ -341,11 +380,8 @@ char *midi_interface_name_get_by_number(struct midi *midi, unsigned int number,
   else
     iface = midi->inputs[number];
 
-  if (iface->type == MIDI_ALSA) {
-    struct midi_alsa *alsa = (struct midi_alsa *)iface->data;
-    char name[256];
-    snprintf(name, 256, "%d:%d", alsa->client, alsa->port);
-    return strdup(name);
+  if (iface->name != NULL) {
+    return strdup(iface->name);
   } else
     return NULL;
 }
@@ -355,9 +391,9 @@ void midi_subscribe_all(struct midi *midi)
 {
   nullcheck_void(midi, midi_subscribe_all);
 
-  int j, last, client, port;
+  int j, last;
   unsigned int direction = 0;
-  char *current = NULL, *next = NULL, *portptr = NULL;
+  char *current = NULL, *next = NULL;
   char *outputs_ok = midi->outputs_ok;
   char *inputs_ok = midi->inputs_ok;
   char *outputs_error = midi->outputs_error;
@@ -393,29 +429,22 @@ void midi_subscribe_all(struct midi *midi)
 
     if (current != NULL) {
       while (!last) {
-	/* Find the end of the device name */
-	while (*next != ',' && *next != 0) {
-	  if (*next == ':') {
-	    *next = 0;
-	    portptr = next + 1;
-	  }
-	  next++;
-	}
-	if (*next == 0)
-	  last = 1;
-	else
-	  *next = 0;
+        /* Find the end of the device name */
+        while (*next != ',' && *next != 0)
+          next++;
+        if (*next == 0)
+          last = 1;
+        else
+          *next = 0;
 
-	/* Empty string? */
-	if (strlen(current) == 0)
-	  break;
+        /* Empty string? */
+        if (strlen(current) == 0)
+          break;
 
-	/* Try to subscribe */
-	client = atoi(current);
-	port = atoi(portptr);
-	midi_subscribe(midi, client, port, direction);
+        /* Try to subscribe */
+        midi_subscribe(midi, current, direction);
 
-	current = ++next;
+        current = ++next;
       }
     }
   }
@@ -438,14 +467,14 @@ void midi_unsubscribe_all(struct midi *midi)
     if (iface->type == MIDI_ALSA) {
       struct midi_alsa *alsa = iface->data;
       if (alsa->subs != NULL) {
-	sender.client = midi->client;
-	sender.port = midi->port;
-	dest.client = alsa->client;
-	dest.port = alsa->port;
-	snd_seq_port_subscribe_alloca(&subs);
-	snd_seq_port_subscribe_set_sender(subs, &sender);
-	snd_seq_port_subscribe_set_dest(subs, &dest);
-	snd_seq_unsubscribe_port(midi->seq, subs);
+        sender.client = midi->client;
+        sender.port = midi->port;
+        dest.client = alsa->client;
+        dest.port = alsa->port;
+        snd_seq_port_subscribe_alloca(&subs);
+        snd_seq_port_subscribe_set_sender(subs, &sender);
+        snd_seq_port_subscribe_set_dest(subs, &dest);
+        snd_seq_unsubscribe_port(midi->seq, subs);
       }
     }
     midi_interface_free(iface);
@@ -456,19 +485,19 @@ void midi_unsubscribe_all(struct midi *midi)
     if (iface->type == MIDI_ALSA) {
       struct midi_alsa *alsa = iface->data;
       if (alsa->subs != NULL) {
-	sender.client = alsa->client;
-	sender.port = alsa->port;
-	dest.client = midi->client;
-	dest.port = midi->port;
-	snd_seq_port_subscribe_alloca(&subs);
-	snd_seq_port_subscribe_set_sender(subs, &sender);
-	snd_seq_port_subscribe_set_dest(subs, &dest);
-	snd_seq_unsubscribe_port(midi->seq, subs);
+        sender.client = alsa->client;
+        sender.port = alsa->port;
+        dest.client = midi->client;
+        dest.port = midi->port;
+        snd_seq_port_subscribe_alloca(&subs);
+        snd_seq_port_subscribe_set_sender(subs, &sender);
+        snd_seq_port_subscribe_set_dest(subs, &dest);
+        snd_seq_unsubscribe_port(midi->seq, subs);
       }
     }
     midi_interface_free(iface);
   }
-  
+
   midi->numoutputs = 0;
   midi->numinputs = 0;
   free(midi->outputs);
@@ -505,28 +534,28 @@ void midi_interfaces_get(struct midi *midi, unsigned int direction)
       int port = snd_seq_port_info_get_port(pinfo);
       /* Check whether the capabilities match the current direction */
       if (snd_seq_port_info_get_capability(pinfo) & bits && !(client == midi->client && port == midi->port)) {
-	/* Create an interface structure for the port */
-	struct midi_interface *iface;
-	struct midi_alsa *alsa;
-	alsa = calloc(1, sizeof(struct midi_alsa));
-	alsa->client = client;
-	alsa->port = port;
-	iface = calloc(1, sizeof(struct midi_interface));
-	iface->midi = midi;
-	iface->type = MIDI_ALSA;
-	iface->data = alsa;
-	iface->name = strdup(snd_seq_port_info_get_name(pinfo));
+        /* Create an interface structure for the port */
+        struct midi_interface *iface;
+        struct midi_alsa *alsa;
+        alsa = calloc(1, sizeof(struct midi_alsa));
+        alsa->client = client;
+        alsa->port = port;
+        iface = calloc(1, sizeof(struct midi_interface));
+        iface->midi = midi;
+        iface->type = MIDI_ALSA;
+        iface->data = alsa;
+        iface->name = strdup(snd_seq_port_info_get_name(pinfo));
 
-	/* Append the interface to the interface array */
-	if (direction == DIRECTION_OUT) {
-	  midi->numoutputs++;
-	  midi->outputs = (struct midi_interface **)realloc(midi->outputs, midi->numoutputs * sizeof(struct midi_interface *));
-	  midi->outputs[midi->numoutputs - 1] = iface;
-	} else {
-	  midi->numinputs++;
-	  midi->inputs = (struct midi_interface **)realloc(midi->inputs, midi->numinputs * sizeof(struct midi_interface *));
-	  midi->inputs[midi->numinputs - 1] = iface;
-	}
+        /* Append the interface to the interface array */
+        if (direction == DIRECTION_OUT) {
+          midi->numoutputs++;
+          midi->outputs = (struct midi_interface **)realloc(midi->outputs, midi->numoutputs * sizeof(struct midi_interface *));
+          midi->outputs[midi->numoutputs - 1] = iface;
+        } else {
+          midi->numinputs++;
+          midi->inputs = (struct midi_interface **)realloc(midi->inputs, midi->numinputs * sizeof(struct midi_interface *));
+          midi->inputs[midi->numinputs - 1] = iface;
+        }
       }
     }
   }
@@ -609,39 +638,38 @@ static inline int midi_read(struct midi *midi, unsigned char *data, unsigned int
       case SND_SEQ_EVENT_PORT_START:
       case SND_SEQ_EVENT_PORT_EXIT:
       case SND_SEQ_EVENT_PORT_CHANGE:
-	/* Ports have been changed */
-	midi->changed = 1;
-	return 0;
-      default:
-	{
-	  /* Get the event to the incoming buffer */
-	  int n = snd_seq_event_length(ev);
-	  midi->incoming = malloc(n);
-	  midi->incominglength = snd_midi_event_decode(midi->decoder, midi->incoming, n, ev);
-	  if (midi->incominglength < 0) {
-	    /* Some sort of an error occurred */
-	    free(midi->incoming);
-	    midi->incoming = NULL;
-	    return 0;
-	  }
-	  midi->incomingposition = 0;
+        /* Ports have been changed */
+        midi->changed = 1;
+        return 0;
+      default: {
+        /* Get the event to the incoming buffer */
+        int n = snd_seq_event_length(ev);
+        midi->incoming = malloc(n);
+        midi->incominglength = snd_midi_event_decode(midi->decoder, midi->incoming, n, ev);
+        if (midi->incominglength < 0) {
+          /* Some sort of an error occurred */
+          free(midi->incoming);
+          midi->incoming = NULL;
+          return 0;
+        }
+        midi->incomingposition = 0;
 
-	  /* Copy up to requested number of bytes from the buffer */
-	  n = midi->incominglength;
-	  if (length < n)
-	    n = length;
+        /* Copy up to requested number of bytes from the buffer */
+        n = midi->incominglength;
+        if (length < n)
+          n = length;
 
-	  memcpy(data, midi->incoming, n);
-	  midi->incomingposition += n;
+        memcpy(data, midi->incoming, n);
+        midi->incomingposition += n;
 
-	  /* If all bytes are used clear the buffer */
-	  if (midi->incomingposition == midi->incominglength) {
-	    free(midi->incoming);
-	    midi->incoming = NULL;
-	  }
+        /* If all bytes are used clear the buffer */
+        if (midi->incomingposition == midi->incominglength) {
+          free(midi->incoming);
+          midi->incoming = NULL;
+        }
 
-	  return n;
-	}
+        return n;
+      }
       }
     } else
       return 0;
@@ -654,74 +682,72 @@ static inline int midi_read(struct midi *midi, unsigned char *data, unsigned int
 static inline void midi_write(struct midi_interface *midi, unsigned char *data, unsigned int length)
 {
   nullcheck_void(midi, midi_write);
-  
+
   unsigned int delta = midi->tick - midi->oldtick;
   midi->oldtick = midi->tick;
 
   switch (midi->type) {
-  case MIDI_ALSA:
-    {
-      struct midi_alsa *alsa = midi->data;
-      int l = 0;
-      
-      /* Create event */
-      snd_seq_event_t ev;
-      snd_seq_ev_clear(&ev);
-      snd_seq_ev_set_source(&ev, midi->midi->port);
-      snd_seq_ev_set_dest(&ev, alsa->client, alsa->port);
-      snd_seq_ev_set_direct(&ev);
-      /* The encoder may send the data in multiple packets */
-      while (l < length) {
-        l += snd_midi_event_encode(midi->midi->encoder, data + l, length - l, &ev);
-        snd_seq_event_output(midi->midi->seq, &ev);
-      }
-      snd_seq_drain_output(midi->midi->seq);
-      break;
+  case MIDI_ALSA: {
+    struct midi_alsa *alsa = midi->data;
+    int l = 0;
+
+    /* Create event */
+    snd_seq_event_t ev;
+    snd_seq_ev_clear(&ev);
+    snd_seq_ev_set_source(&ev, midi->midi->port);
+    snd_seq_ev_set_dest(&ev, alsa->client, alsa->port);
+    snd_seq_ev_set_direct(&ev);
+    /* The encoder may send the data in multiple packets */
+    while (l < length) {
+      l += snd_midi_event_encode(midi->midi->encoder, data + l, length - l, &ev);
+      snd_seq_event_output(midi->midi->seq, &ev);
     }
-  case MIDI_BUFFER:
-    {
-      struct midi_buffer *buffer = midi->data;
-      unsigned int curpos = buffer->ptr - buffer->data;
-      unsigned long value = delta;
-      unsigned long varlen = value & 0x7F;
-      int i;
+    snd_seq_drain_output(midi->midi->seq);
+    break;
+  }
+  case MIDI_BUFFER: {
+    struct midi_buffer *buffer = midi->data;
+    unsigned int curpos = buffer->ptr - buffer->data;
+    unsigned long value = delta;
+    unsigned long varlen = value & 0x7F;
+    int i;
 
-      /* Create a variable length version of the tick delta */
-      while ((value >>= 7)) {
-	varlen <<= 8;
-	varlen |= ((value & 0x7F) | 0x80);
-      }
-
-      /* Check how many bytes the varlen version requires */
-      value = varlen;
-      for (i = 1; i <= 4; i++) {
-	if (value & 0x80)
-	  value >>= 8;
-	else
-	  break;
-      }
-
-      /* Allocate more space if required */
-      if (curpos + length + i > buffer->allocated) {
-	buffer->allocated += MIDI_BUFFER_BLOCK_SIZE;
-	buffer->data = realloc(buffer->data, buffer->allocated);
-	buffer->ptr = buffer->data + curpos;
-      }
-
-      /* Write the varlen tick delta value */
-      for (i = 1; i <= 4; i++) {
-	*buffer->ptr++ = (unsigned char)(varlen & 0xff);
-	if (varlen & 0x80)
-	  varlen >>= 8;
-	else
-	  break;
-      }
-
-      /* Write the actual payload */
-      memcpy(buffer->ptr, data, length);
-      buffer->ptr += length;
-      break;
+    /* Create a variable length version of the tick delta */
+    while ((value >>= 7)) {
+      varlen <<= 8;
+      varlen |= ((value & 0x7F) | 0x80);
     }
+
+    /* Check how many bytes the varlen version requires */
+    value = varlen;
+    for (i = 1; i <= 4; i++) {
+      if (value & 0x80)
+        value >>= 8;
+      else
+        break;
+    }
+
+    /* Allocate more space if required */
+    if (curpos + length + i > buffer->allocated) {
+      buffer->allocated += MIDI_BUFFER_BLOCK_SIZE;
+      buffer->data = realloc(buffer->data, buffer->allocated);
+      buffer->ptr = buffer->data + curpos;
+    }
+
+    /* Write the varlen tick delta value */
+    for (i = 1; i <= 4; i++) {
+      *buffer->ptr++ = (unsigned char)(varlen & 0xff);
+      if (varlen & 0x80)
+        varlen >>= 8;
+      else
+        break;
+    }
+
+    /* Write the actual payload */
+    memcpy(buffer->ptr, data, length);
+    buffer->ptr += length;
+    break;
+  }
   default:
     break;
   }
@@ -848,9 +874,9 @@ void midi_write_raw(struct midi_interface *midi, unsigned char *message, unsigne
     value = varlen;
     for (l = 1; l <= 4; l++) {
       if (value & 0x80)
-	value >>= 8;
+        value >>= 8;
       else
-	break;
+        break;
     }
 
     /* Allocate buffer */
@@ -861,9 +887,9 @@ void midi_write_raw(struct midi_interface *midi, unsigned char *message, unsigne
     for (i = 1; i <= 4; i++) {
       newmessage[i] = (unsigned char)(varlen & 0xff);
       if (varlen & 0x80)
-	varlen >>= 8;
+        varlen >>= 8;
       else
-	break;
+        break;
     }
 
     /* Copy the rest of the data */
@@ -912,7 +938,8 @@ unsigned char *midi_read_raw(struct midi *midi)
   unsigned char c = 0, *d = NULL;
   const struct timespec nobusywait = { 0, 40000 };
 
-  while ((status = midi_read(midi, &c, 1)) == 1 && c < 0x80);
+  while ((status = midi_read(midi, &c, 1)) == 1 && c < 0x80)
+    ;
 
   if (status != 1)
     return NULL;
@@ -922,26 +949,25 @@ unsigned char *midi_read_raw(struct midi *midi)
   case 0x90:
   case 0xa0:
   case 0xb0:
-  case 0xe0:
-    {
-      /* Number of bytes read */
-      int r = 0;
+  case 0xe0: {
+    /* Number of bytes read */
+    int r = 0;
 
-      d = (unsigned char *)calloc(3, sizeof(unsigned char));
-      d[0] = c;
+    d = (unsigned char *)calloc(3, sizeof(unsigned char));
+    d[0] = c;
 
-      /* Read until 2 bytes have been read. If for some reason these two bytes will never come Tutka hangs here! */
-      while (r < 2) {
-	status = midi_read(midi, d + 1 + r, 2 - r);
-	if (status >= 0)
-	  r += status;
-        
-        /* Sleep for a while to avoid busy looping */
-        nanosleep(&nobusywait, 0);
-      }
+    /* Read until 2 bytes have been read. If for some reason these two bytes will never come Tutka hangs here! */
+    while (r < 2) {
+      status = midi_read(midi, d + 1 + r, 2 - r);
+      if (status >= 0)
+        r += status;
 
-      return d;
+      /* Sleep for a while to avoid busy looping */
+      nanosleep(&nobusywait, 0);
     }
+
+    return d;
+  }
   default:
     break;
   }
@@ -963,15 +989,15 @@ int midi_read_system_exclusive(struct midi *midi, struct message *message, unsig
       status = midi_read(midi, &c, 1);
 
       if (status == 0)
-	return i - read;
+        return i - read;
 
       if (i >= message->length) {
-	message->length += 65536;
-	message->data = (unsigned char *)realloc(message->data, message->length);
+        message->length += 65536;
+        message->data = (unsigned char *)realloc(message->data, message->length);
       }
 
       if (c != 0xf1 && c != 0xf8 && c != 0xf9 && c != 0xfe)
-	message->data[i++] = c;
+        message->data[i++] = c;
     }
 
     return -(i - read);
@@ -981,10 +1007,10 @@ int midi_read_system_exclusive(struct midi *midi, struct message *message, unsig
       status = midi_read(midi, &c, 1);
 
       if (status == 0)
-	return i - read;
+        return i - read;
 
       if (c != 0xf1 && c != 0xf8 && c != 0xf9 && c != 0xfe)
-	message->data[i++] = c;
+        message->data[i++] = c;
     }
 
     return -(i - read);
@@ -1047,9 +1073,9 @@ struct message *message_parse(xmlDocPtr doc, xmlNsPtr ns, xmlNodePtr cur)
 {
   struct message *message = NULL;
 
-  if ((!xmlStrcmp(cur->name, "message")) && (cur->ns == ns)) {
+  if ((!xmlStrcmp(cur->name, (xmlChar *)"message")) && (cur->ns == ns)) {
     xmlNodePtr temp = cur->xmlChildrenNode;
-    char c[3], *prop;
+    xmlChar c[3], *prop;
     int i;
     unsigned int d;
 
@@ -1058,17 +1084,26 @@ struct message *message_parse(xmlDocPtr doc, xmlNsPtr ns, xmlNodePtr cur)
 
     /* Allocate message */
     message = (struct message *)calloc(1, sizeof(struct message));
-    message->name = xmlGetProp(cur, "name");
+    prop = xmlGetProp(cur, (xmlChar *)"name");
+    if (prop != NULL) {
+      message->name = strdup((char *)prop);
+      xmlFree(prop);
+    }
+
     if (temp != NULL)
-      message->length = strlen(temp->content) / 2;
-    prop = xmlGetProp(cur, "autosend");
-    if (prop != NULL)
-      message->autosend = atoi(prop);
+      message->length = strlen((char *)temp->content) / 2;
+
+    prop = xmlGetProp(cur, (xmlChar *)"autosend");
+    if (prop != NULL) {
+      message->autosend = atoi((char *)prop);
+      xmlFree(prop);
+    }
+
     message->data = (unsigned char *)calloc(message->length, sizeof(unsigned char));
     for (i = 0; i < message->length; i++) {
       c[0] = temp->content[i * 2];
       c[1] = temp->content[i * 2 + 1];
-      sscanf(c, "%X", &d);
+      sscanf((char *)c, "%X", &d);
       message->data[i] = d;
     }
   } else if (cur->type != XML_COMMENT_NODE)
@@ -1082,22 +1117,22 @@ void message_save(struct message *message, int number, xmlNodePtr parent)
 {
   nullcheck_void(message, message_save);
 
-  char *c, t[10];
+  xmlChar *c, t[10];
   xmlNodePtr node;
   int i;
 
-  c = (char *)calloc(2 * message->length + 1, sizeof(char));
+  c = (xmlChar *)calloc(2 * message->length + 1, sizeof(xmlChar));
   for (i = 0; i < message->length; i++)
-    sprintf(c + (i * 2), "%.2X", message->data[i]);
+    sprintf((char *)c + (i * 2), "%.2X", message->data[i]);
 
-  node = xmlNewChild(parent, NULL, "message", c);
-  snprintf(t, 10, "%d", number);
-  xmlSetProp(node, "number", t);
-  xmlSetProp(node, "name", message->name);
-  snprintf(t, 10, "%d", message->autosend);
-  xmlSetProp(node, "autosend", t);
+  node = xmlNewChild(parent, NULL, (xmlChar *)"message", c);
+  snprintf((char *)t, 10, "%d", number);
+  xmlSetProp(node, (xmlChar *)"number", t);
+  xmlSetProp(node, (xmlChar *)"name", (xmlChar *)message->name);
+  snprintf((char *)t, 10, "%d", message->autosend);
+  xmlSetProp(node, (xmlChar *)"autosend", t);
 
-  xmlAddChild(parent, xmlNewText("\n"));
+  xmlAddChild(parent, xmlNewText((xmlChar *)"\n"));
 
   free(c);
 }
@@ -1123,8 +1158,7 @@ void message_load_binary(struct message *message, char *filename)
     fread(data, sizeof(unsigned char), length, file);
     fclose(file);
 
-    if (message->data != NULL)
-      free(message->data);
+    free(message->data);
 
     message->data = data;
     message->length = length;
@@ -1173,20 +1207,6 @@ static char *string_remove_string(char *from, char *toremove)
   return from;
 }
 
-static char *string_append_char(char *to, char append)
-{
-  int l = 0;
-
-  if (to != NULL)
-    l = strlen(to);
-
-  to = realloc(to, l + 2);
-  to[l++] = append;
-  to[l] = 0;
-
-  return to;
-}
-
 static char *string_append_string(char *to, char *append)
 {
   int l1 = 0;
@@ -1200,3 +1220,20 @@ static char *string_append_string(char *to, char *append)
 
   return to;
 }
+
+/* Not used */
+/*
+static char *string_append_char(char *to, char append)
+{
+  int l = 0;
+
+  if (to != NULL)
+    l = strlen(to);
+
+  to = realloc(to, l + 2);
+  to[l++] = append;
+  to[l] = 0;
+
+  return to;
+}
+*/
