@@ -1,6 +1,6 @@
 /* mmd.c
  *
- * Copyright 2002-2006 Vesa Halttunen
+ * Copyright 2002-2011 Vesa Halttunen
  *
  * This file is part of Tutka.
  *
@@ -61,19 +61,19 @@ int MMD2_length_get(struct MMD2 *mmd)
   for (i = 0; i < mmd->song->numblocks; i++) {
     if (mmd->blockarr[i]->info != NULL) {
       if (mmd->blockarr[i]->info->hlmask != NULL) {
-	/* hlmask */
-	length += (1 + mmd->blockarr[i]->lines / 32) * sizeof(unsigned int);
+        /* hlmask */
+        length += (1 + mmd->blockarr[i]->lines / 32) * sizeof(unsigned int);
       }
       if (mmd->blockarr[i]->info->blockname != NULL) {
-	length += (mmd->blockarr[i]->info->blocknamelen + 1) & 0xfffffffe;
+        length += (mmd->blockarr[i]->info->blocknamelen + 1) & 0xfffffffe;
       }
       if (mmd->blockarr[i]->info->pagetable != NULL) {
-	for (j = 0; j < mmd->blockarr[i]->info->pagetable->num_pages; j++) {
-	  /* page */
-	  length += (mmd->blockarr[i]->numtracks * (mmd->blockarr[i]->lines + 1)) * sizeof(unsigned short);
-	}
-	/* pagetable */
-	length += sizeof(struct BlockCmdPageTable) + mmd->blockarr[i]->info->pagetable->num_pages * sizeof(int);
+        for (j = 0; j < mmd->blockarr[i]->info->pagetable->num_pages; j++) {
+          /* page */
+          length += (mmd->blockarr[i]->numtracks * (mmd->blockarr[i]->lines + 1)) * sizeof(unsigned short);
+        }
+        /* pagetable */
+        length += sizeof(struct BlockCmdPageTable) + mmd->blockarr[i]->info->pagetable->num_pages * sizeof(int);
       }
       /* blockinfo */
       length += sizeof(struct BlockInfo);
@@ -90,12 +90,12 @@ int MMD2_length_get(struct MMD2 *mmd)
     /* instrext */
     if (mmd->expdata->exp_smp != NULL) {
       for (i = 0; i < mmd->expdata->s_ext_entries; i++)
-	length += mmd->expdata->s_ext_entrsz;
+        length += mmd->expdata->s_ext_entrsz;
     }
     /* instrinfo */
     if (mmd->expdata->iinfo != NULL) {
       for (i = 0; i < mmd->expdata->i_ext_entries; i++)
-	length += mmd->expdata->i_ext_entrsz;
+        length += mmd->expdata->i_ext_entrsz;
     }
     /* notationinfo */
     if (mmd->expdata->n_info != NULL)
@@ -110,33 +110,33 @@ int MMD2_length_get(struct MMD2 *mmd)
       struct MMDDump **dumps = (struct MMDDump **)((char *)mmd->expdata->dumps + sizeof(struct MMDDumpData));
       /* dump data */
       for (i = 0; i < mmd->expdata->dumps->numdumps; i++) {
-	/* dump */
-	length += ((dumps[i]->length + 1) & 0xfffffffe);
-	length += 10 + dumps[i]->ext_len;
+        /* dump */
+        length += ((dumps[i]->length + 1) & 0xfffffffe);
+        length += 10 + dumps[i]->ext_len;
       }
       length += sizeof(struct MMDDumpData) + mmd->expdata->dumps->numdumps * sizeof(int);
     }
     if (mmd->expdata->mmdrexx != NULL) {
       if (mmd->expdata->mmdrexx->trigcmd != NULL) {
-	struct MMDARexxTrigCmd *trig = mmd->expdata->mmdrexx->trigcmd;
-	while (trig != NULL) {
-	  trig = trig->next;
-	  if (trig->cmd != NULL)
-	    length += (trig->cmd_len + 2) & 0xfffffffe;
-	  if (trig->port != NULL)
-	    length += (trig->port_len + 2) & 0xfffffffe;
-	  /* mmdarexxtrigcmd */
-	  length += sizeof(struct MMDARexxTrigCmd);
-	}
-	/* mmdarexx */
-	length += sizeof(struct MMDARexx);
+        struct MMDARexxTrigCmd *trig = mmd->expdata->mmdrexx->trigcmd;
+        while (trig != NULL) {
+          trig = trig->next;
+          if (trig->cmd != NULL)
+            length += (trig->cmd_len + 2) & 0xfffffffe;
+          if (trig->port != NULL)
+            length += (trig->port_len + 2) & 0xfffffffe;
+          /* mmdarexxtrigcmd */
+          length += sizeof(struct MMDARexxTrigCmd);
+        }
+        /* mmdarexx */
+        length += sizeof(struct MMDARexx);
       }
     }
     if (mmd->expdata->mmdcmd3x != NULL) {
       /* types */
       length += mmd->expdata->mmdcmd3x->num_of_settings;
       if ((mmd->expdata->mmdcmd3x->num_of_settings & 1) == 1)
-	length++;
+        length++;
       /* numbers */
       length += 2 * mmd->expdata->mmdcmd3x->num_of_settings;
       /* mmdcmd3x */
@@ -145,7 +145,7 @@ int MMD2_length_get(struct MMD2 *mmd)
     /* MMD0exp */
     length += sizeof(struct MMD0exp);
   }
-  
+
   return length;
 }
 
@@ -209,7 +209,8 @@ void MMD2_save(struct MMD2 *mmd, char *filename)
     memcpy(data + offsets[0], mmd->song->playseqtable[i]->name, 32);
     putword(data + offsets[0] + 40, mmd->song->playseqtable[i]->length);
     for (j = 0; j < mmd->song->playseqtable[i]->length; j++) {
-      putword(data + offsets[0] + 42 + j * sizeof(unsigned short), mmd->song->playseqtable[i]->seq[j]);
+      putword(data + offsets[0] + 42 + j * sizeof(unsigned short), mmd->song->playseqtable[i]->seq[j])
+      ;
     }
     putlong(data + offsets[1] + i * sizeof(int), offsets[0]);
     offsets[0] += sizeof(struct PlaySeq) + (mmd->song->playseqtable[i]->length - 1) * sizeof(unsigned short);
@@ -217,7 +218,8 @@ void MMD2_save(struct MMD2 *mmd, char *filename)
   offsets[0] += mmd->song->numpseqs * sizeof(int);
   /* sectiontable */
   for (i = 0; i < mmd->song->songlen; i++) {
-    putword(data + offsets[0] + i * sizeof(unsigned short), mmd->song->sectiontable[i]);
+    putword(data + offsets[0] + i * sizeof(unsigned short), mmd->song->sectiontable[i])
+    ;
   }
   offsets[0] += mmd->song->songlen * sizeof(unsigned short);
   /* trackvols */
@@ -230,7 +232,7 @@ void MMD2_save(struct MMD2 *mmd, char *filename)
     putbyte(data + offsets[0] + i, mmd->song->trackpans[i]);
   }
   offsets[0] += ((mmd->song->numtracks + 1) & 0xfffffffe);
-  
+
   /* MMD2Song */
   putlong(data + 8, offsets[0]);
   for (i = 0; i < 63; i++) {
@@ -279,11 +281,11 @@ void MMD2_save(struct MMD2 *mmd, char *filename)
     if (mmd->blockarr[i]->info != NULL) {
       offsets[1] += sizeof(struct BlockInfo);
       if (mmd->blockarr[i]->info->hlmask != NULL)
-	offsets[1] += (1 + mmd->blockarr[i]->lines / 32) * sizeof(unsigned int);
+        offsets[1] += (1 + mmd->blockarr[i]->lines / 32) * sizeof(unsigned int);
       if (mmd->blockarr[i]->info->blockname != NULL)
-	offsets[1] += (mmd->blockarr[i]->info->blocknamelen + 1) & 0xfffffffe;
+        offsets[1] += (mmd->blockarr[i]->info->blocknamelen + 1) & 0xfffffffe;
       if (mmd->blockarr[i]->info->pagetable != NULL)
-	offsets[1] += sizeof(struct BlockCmdPageTable) + mmd->blockarr[i]->info->pagetable->num_pages * sizeof(unsigned short *) + mmd->blockarr[i]->info->pagetable->num_pages * (mmd->blockarr[i]->numtracks * (mmd->blockarr[i]->lines + 1)) * sizeof(unsigned short);
+        offsets[1] += sizeof(struct BlockCmdPageTable) + mmd->blockarr[i]->info->pagetable->num_pages * sizeof(unsigned short *) + mmd->blockarr[i]->info->pagetable->num_pages * (mmd->blockarr[i]->numtracks * (mmd->blockarr[i]->lines + 1)) * sizeof(unsigned short);
     }
   }
 
@@ -296,34 +298,35 @@ void MMD2_save(struct MMD2 *mmd, char *filename)
       offsets[4] = 0;
       offsets[5] = 0;
       if (mmd->blockarr[i]->info->hlmask != NULL) {
-	/* hlmask */
-	offsets[3] = offsets[0];
-	for (j = 0; j < 1 + mmd->blockarr[i]->lines / 32; j++) {
-	  putlong(data + offsets[0] + j * sizeof(int), mmd->blockarr[i]->info->hlmask[j]);
-	}
-	offsets[0] += (1 + mmd->blockarr[i]->lines / 32) * sizeof(unsigned int);
+        /* hlmask */
+        offsets[3] = offsets[0];
+        for (j = 0; j < 1 + mmd->blockarr[i]->lines / 32; j++) {
+          putlong(data + offsets[0] + j * sizeof(int), mmd->blockarr[i]->info->hlmask[j])
+          ;
+        }
+        offsets[0] += (1 + mmd->blockarr[i]->lines / 32) * sizeof(unsigned int);
       }
       if (mmd->blockarr[i]->info->blockname != NULL) {
-	offsets[4] = offsets[0];
-	memcpy(data + offsets[0], mmd->blockarr[i]->info->blockname, mmd->blockarr[i]->info->blocknamelen);
-	offsets[0] += (mmd->blockarr[i]->info->blocknamelen + 1) & 0xfffffffe;
+        offsets[4] = offsets[0];
+        memcpy(data + offsets[0], mmd->blockarr[i]->info->blockname, mmd->blockarr[i]->info->blocknamelen);
+        offsets[0] += (mmd->blockarr[i]->info->blocknamelen + 1) & 0xfffffffe;
       }
       if (mmd->blockarr[i]->info->pagetable != NULL) {
-	offsets[2] = offsets[0];
-	for (j = 0; j < mmd->blockarr[i]->info->pagetable->num_pages; j++)
-	  offsets[2] += (mmd->blockarr[i]->numtracks * (mmd->blockarr[i]->lines + 1)) * sizeof(unsigned short);
+        offsets[2] = offsets[0];
+        for (j = 0; j < mmd->blockarr[i]->info->pagetable->num_pages; j++)
+          offsets[2] += (mmd->blockarr[i]->numtracks * (mmd->blockarr[i]->lines + 1)) * sizeof(unsigned short);
 
-	for (j = 0; j < mmd->blockarr[i]->info->pagetable->num_pages; j++) {
-	  /* page */
-	  memcpy(data + offsets[0], mmd->blockarr[i]->info->pagetable->page[j], (mmd->blockarr[i]->numtracks * (mmd->blockarr[i]->lines + 1)) * sizeof(unsigned short));
-	  putlong(data + offsets[2] + 4 + sizeof(int) * j, offsets[0]);
-	  offsets[0] += (mmd->blockarr[i]->numtracks * (mmd->blockarr[i]->lines + 1)) * sizeof(unsigned short);
-	}
-	/* pagetable */
-	offsets[5] = offsets[0];
-	putword(data + offsets[0] + 0, mmd->blockarr[i]->info->pagetable->num_pages);
-	putword(data + offsets[0] + 2, mmd->blockarr[i]->info->pagetable->reserved);
-	offsets[0] += sizeof(struct BlockCmdPageTable) + mmd->blockarr[i]->info->pagetable->num_pages * sizeof(int);
+        for (j = 0; j < mmd->blockarr[i]->info->pagetable->num_pages; j++) {
+          /* page */
+          memcpy(data + offsets[0], mmd->blockarr[i]->info->pagetable->page[j], (mmd->blockarr[i]->numtracks * (mmd->blockarr[i]->lines + 1)) * sizeof(unsigned short));
+          putlong(data + offsets[2] + 4 + sizeof(int) * j, offsets[0]);
+          offsets[0] += (mmd->blockarr[i]->numtracks * (mmd->blockarr[i]->lines + 1)) * sizeof(unsigned short);
+        }
+        /* pagetable */
+        offsets[5] = offsets[0];
+        putword(data + offsets[0] + 0, mmd->blockarr[i]->info->pagetable->num_pages);
+        putword(data + offsets[0] + 2, mmd->blockarr[i]->info->pagetable->reserved);
+        offsets[0] += sizeof(struct BlockCmdPageTable) + mmd->blockarr[i]->info->pagetable->num_pages * sizeof(int);
       }
       /* blockinfo */
       offsets[2] = offsets[0];
@@ -355,23 +358,24 @@ void MMD2_save(struct MMD2 *mmd, char *filename)
     offsets[2] = offsets[0];
     if (mmd->expdata->exp_smp != NULL) {
       for (i = 0; i < mmd->expdata->s_ext_entries; i++) {
-	putbyte(data + offsets[0] + 0, mmd->expdata->exp_smp[i].hold);
-	putbyte(data + offsets[0] + 1, mmd->expdata->exp_smp[i].decay);
-	putbyte(data + offsets[0] + 2, mmd->expdata->exp_smp[i].suppress_midi_off);
-	putbyte(data + offsets[0] + 3, mmd->expdata->exp_smp[i].finetune);
-	if (mmd->expdata->s_ext_entrsz > 4) {
-	  putbyte(data + offsets[0] + 4, mmd->expdata->exp_smp[i].default_pitch);
-	  putbyte(data + offsets[0] + 5, mmd->expdata->exp_smp[i].instr_flags);
-	  putword(data + offsets[0] + 6, mmd->expdata->exp_smp[i].long_midi_preset);
-	  if (mmd->expdata->s_ext_entrsz > 8) {
-	    putbyte(data + offsets[0] + 8, mmd->expdata->exp_smp[i].output_device);
-	    if (mmd->expdata->s_ext_entrsz > 10) {
-	      putlong(data + offsets[0] + 10, mmd->expdata->exp_smp[i].long_repeat);
-	      putlong(data + offsets[0] + 14, mmd->expdata->exp_smp[i].long_replen);
-	    }
-	  }
-	}
-	offsets[0] += mmd->expdata->s_ext_entrsz;
+        putbyte(data + offsets[0] + 0, mmd->expdata->exp_smp[i].hold);
+        putbyte(data + offsets[0] + 1, mmd->expdata->exp_smp[i].decay);
+        putbyte(data + offsets[0] + 2, mmd->expdata->exp_smp[i].suppress_midi_off);
+        putbyte(data + offsets[0] + 3, mmd->expdata->exp_smp[i].finetune);
+        if (mmd->expdata->s_ext_entrsz > 4) {
+          putbyte(data + offsets[0] + 4, mmd->expdata->exp_smp[i].default_pitch);
+          putbyte(data + offsets[0] + 5, mmd->expdata->exp_smp[i].instr_flags);
+          putword(data + offsets[0] + 6, mmd->expdata->exp_smp[i].long_midi_preset);
+          if (mmd->expdata->s_ext_entrsz > 8) {
+            putbyte(data + offsets[0] + 8, mmd->expdata->exp_smp[i].output_device);
+            if (mmd->expdata->s_ext_entrsz > 10) {
+              putlong(data + offsets[0] + 10, mmd->expdata->exp_smp[i].long_repeat);
+              putlong(data + offsets[0] + 14, mmd->expdata->exp_smp[i].long_replen)
+              ;
+            }
+          }
+        }
+        offsets[0] += mmd->expdata->s_ext_entrsz;
       }
     }
     /* instrinfo */
@@ -379,8 +383,8 @@ void MMD2_save(struct MMD2 *mmd, char *filename)
     if (mmd->expdata->iinfo != NULL) {
       offsets[3] = offsets[0];
       for (i = 0; i < mmd->expdata->i_ext_entries; i++) {
-	memcpy(data + offsets[0], mmd->expdata->iinfo[i].name, sizeof(struct MMDInstrInfo));
-	offsets[0] += mmd->expdata->i_ext_entrsz;
+        memcpy(data + offsets[0], mmd->expdata->iinfo[i].name, sizeof(struct MMDInstrInfo));
+        offsets[0] += mmd->expdata->i_ext_entrsz;
       }
     }
     /* notationinfo */
@@ -421,21 +425,21 @@ void MMD2_save(struct MMD2 *mmd, char *filename)
       offsets[7] = offsets[0];
 
       for (i = 0; i < mmd->expdata->dumps->numdumps; i++)
-	offsets[7] += ((dumps[i]->length + 1) & 0xfffffffe) + 10 + dumps[i]->ext_len;
+        offsets[7] += ((dumps[i]->length + 1) & 0xfffffffe) + 10 + dumps[i]->ext_len;
       putword(data + offsets[7] + 0, mmd->expdata->dumps->numdumps);
 
       /* dump data */
       for (i = 0; i < mmd->expdata->dumps->numdumps; i++) {
-	/* dump */
-	memcpy(data + offsets[0], dumps[i]->data, dumps[i]->length);
-	offsets[0] += ((dumps[i]->length + 1) & 0xfffffffe);
-	putlong(data + offsets[0] + 0, dumps[i]->length);
-	putlong(data + offsets[0] + 4, offsets[0] - ((dumps[i]->length + 1) & 0xfffffffe));
-	putword(data + offsets[0] + 8, dumps[i]->ext_len);
-	putlong(data + offsets[7] + 8 + i * sizeof(int), offsets[0]);
-	if (dumps[i]->ext_len >= 20)
-	  memcpy(data + offsets[0] + 10, dumps[i]->name, dumps[i]->ext_len);
-	offsets[0] += 10 + dumps[i]->ext_len;
+        /* dump */
+        memcpy(data + offsets[0], dumps[i]->data, dumps[i]->length);
+        offsets[0] += ((dumps[i]->length + 1) & 0xfffffffe);
+        putlong(data + offsets[0] + 0, dumps[i]->length);
+        putlong(data + offsets[0] + 4, offsets[0] - ((dumps[i]->length + 1) & 0xfffffffe));
+        putword(data + offsets[0] + 8, dumps[i]->ext_len);
+        putlong(data + offsets[7] + 8 + i * sizeof(int), offsets[0]);
+        if (dumps[i]->ext_len >= 20)
+          memcpy(data + offsets[0] + 10, dumps[i]->name, dumps[i]->ext_len);
+        offsets[0] += 10 + dumps[i]->ext_len;
       }
       offsets[0] += sizeof(struct MMDDumpData) + mmd->expdata->dumps->numdumps * sizeof(int);
     }
@@ -443,52 +447,52 @@ void MMD2_save(struct MMD2 *mmd, char *filename)
     offsets[8] = 0;
     if (mmd->expdata->mmdrexx != NULL) {
       if (mmd->expdata->mmdrexx->trigcmd != NULL) {
-	struct MMDARexxTrigCmd *trig = mmd->expdata->mmdrexx->trigcmd;
-	struct MMDARexxTrigCmd **trigs;
-	i = 0;
-	while (trig != NULL) {
-	  trig = trig->next;
-	  i++;
-	}
-	trigs = (struct MMDARexxTrigCmd **)calloc(i + 1, sizeof(struct MMDARexxTrigCmd *));
-	trig = mmd->expdata->mmdrexx->trigcmd;
-	while (trig != NULL) {
-	  trigs[--i] = trig;
-	  trig = trig->next;
-	}
-	offsets[13] = 0;
-	for (i = 0; trigs[i] != NULL; i++) {
-	  offsets[14] = 0;
-	  offsets[15] = 0;
-	  if (trigs[i]->cmd != NULL) {
-	    offsets[14] = offsets[0];
-	    /* command */
-	    memcpy(data + offsets[0], trigs[i]->cmd, trigs[i]->cmd_len);
-	    offsets[0] += (trigs[i]->cmd_len + 2) & 0xfffffffe;
-	  }
-	  if (trigs[i]->port != NULL) {
-	    offsets[15] = offsets[0];
-	    /* port */
-	    memcpy(data + offsets[0], trigs[i]->port, trigs[i]->port_len);
-	    offsets[0] += (trigs[i]->port_len + 2) & 0xfffffffe;
-	  }
-	  /* mmdarexxtrigcmd */
-	  putlong(data + offsets[0], offsets[13]);
-	  putbyte(data + offsets[0] + 4, trigs[i]->cmdnum);
-	  putword(data + offsets[0] + 6, trigs[i]->cmdtype);
-	  putlong(data + offsets[0] + 8, offsets[14]);
-	  putlong(data + offsets[0] + 12, offsets[15]);
-	  putword(data + offsets[0] + 16, trigs[i]->cmd_len);
-	  putword(data + offsets[0] + 18, trigs[i]->port_len);
-	  offsets[13] = offsets[0];
-	  offsets[0] += sizeof(struct MMDARexxTrigCmd);
-	}
-	free(trigs);
-	/* mmdarexx */
-	offsets[8] = offsets[0];
-	putword(data + offsets[0] + 2, mmd->expdata->mmdrexx->trigcmdlen);
-	putlong(data + offsets[0] + 4, offsets[13]);
-	offsets[0] += sizeof(struct MMDARexx);
+        struct MMDARexxTrigCmd *trig = mmd->expdata->mmdrexx->trigcmd;
+        struct MMDARexxTrigCmd **trigs;
+        i = 0;
+        while (trig != NULL) {
+          trig = trig->next;
+          i++;
+        }
+        trigs = (struct MMDARexxTrigCmd **)calloc(i + 1, sizeof(struct MMDARexxTrigCmd *));
+        trig = mmd->expdata->mmdrexx->trigcmd;
+        while (trig != NULL) {
+          trigs[--i] = trig;
+          trig = trig->next;
+        }
+        offsets[13] = 0;
+        for (i = 0; trigs[i] != NULL; i++) {
+          offsets[14] = 0;
+          offsets[15] = 0;
+          if (trigs[i]->cmd != NULL) {
+            offsets[14] = offsets[0];
+            /* command */
+            memcpy(data + offsets[0], trigs[i]->cmd, trigs[i]->cmd_len);
+            offsets[0] += (trigs[i]->cmd_len + 2) & 0xfffffffe;
+          }
+          if (trigs[i]->port != NULL) {
+            offsets[15] = offsets[0];
+            /* port */
+            memcpy(data + offsets[0], trigs[i]->port, trigs[i]->port_len);
+            offsets[0] += (trigs[i]->port_len + 2) & 0xfffffffe;
+          }
+          /* mmdarexxtrigcmd */
+          putlong(data + offsets[0], offsets[13]);
+          putbyte(data + offsets[0] + 4, trigs[i]->cmdnum);
+          putword(data + offsets[0] + 6, trigs[i]->cmdtype);
+          putlong(data + offsets[0] + 8, offsets[14]);
+          putlong(data + offsets[0] + 12, offsets[15]);
+          putword(data + offsets[0] + 16, trigs[i]->cmd_len);
+          putword(data + offsets[0] + 18, trigs[i]->port_len);
+          offsets[13] = offsets[0];
+          offsets[0] += sizeof(struct MMDARexxTrigCmd);
+        }
+        free(trigs);
+        /* mmdarexx */
+        offsets[8] = offsets[0];
+        putword(data + offsets[0] + 2, mmd->expdata->mmdrexx->trigcmdlen);
+        putlong(data + offsets[0] + 4, offsets[13]);
+        offsets[0] += sizeof(struct MMDARexx);
       }
     }
     /* MMDCmd3x */
@@ -497,16 +501,16 @@ void MMD2_save(struct MMD2 *mmd, char *filename)
       putlong(data + offsets[0] + ((mmd->expdata->mmdcmd3x->num_of_settings + 1) & 0xfffffffe) + mmd->expdata->mmdcmd3x->num_of_settings * sizeof(unsigned short) + 4, offsets[0]);
       /* types */
       for (i = 0; i < mmd->expdata->mmdcmd3x->num_of_settings; i++) {
-	putbyte(data + offsets[0], mmd->expdata->mmdcmd3x->ctrlr_types[i]);
-	offsets[0]++;
+        putbyte(data + offsets[0], mmd->expdata->mmdcmd3x->ctrlr_types[i]);
+        offsets[0]++;
       }
       if ((mmd->expdata->mmdcmd3x->num_of_settings & 1) == 1)
-	offsets[0]++;
+        offsets[0]++;
       putlong(data + offsets[0] + mmd->expdata->mmdcmd3x->num_of_settings * sizeof(unsigned short) + 8, offsets[0]);
       /* numbers */
       for (i = 0; i < mmd->expdata->mmdcmd3x->num_of_settings; i++) {
-	putword(data + offsets[0], mmd->expdata->mmdcmd3x->ctrlr_numbers[i]);
-	offsets[0] += 2;
+        putword(data + offsets[0], mmd->expdata->mmdcmd3x->ctrlr_numbers[i]);
+        offsets[0] += 2;
       }
       offsets[9] = offsets[0];
       /* mmdcmd3x */
@@ -551,66 +555,56 @@ void MMD2_save(struct MMD2 *mmd, char *filename)
 /* Frees an MMD0 structure and all associated data */
 void MMD2_free(struct MMD2 *mmd)
 {
-  nullcheck_void(mmd, MMD2_free);
+  if (mmd != NULL) {
+    int i;
 
-  int i;
-
-  for (i = 0; i < mmd->song->numblocks - 1; i++) {
-    if (mmd->blockarr[i])
+    for (i = 0; i < mmd->song->numblocks - 1; i++) {
       MMD1Block_free(mmd->blockarr[i]);
-  }
-  if (mmd->blockarr)
+    }
     free(mmd->blockarr);
-
-  if (mmd->song)
     MMD2song_free(mmd->song);
-
-  if (mmd->expdata)
     free(mmd->expdata);
-
-  free(mmd);
+    free(mmd);
+  }
 }
 
 /* Frees an MMD2song structure and all associated data */
 void MMD2song_free(struct MMD2song *song)
 {
-  nullcheck_void(song, MMD2song_free);
+  if (song != NULL) {
+    unsigned short i;
 
-  unsigned short i;
-
-  for (i = 0; i < song->numpseqs; i++) {
-    if (song->playseqtable[i]) {
-      free(song->playseqtable[i]);
-      song->playseqtable[i] = 0;
+    for (i = 0; i < song->numpseqs; i++) {
+      if (song->playseqtable[i]) {
+        free(song->playseqtable[i]);
+        song->playseqtable[i] = 0;
+      }
     }
+    free(song);
   }
-  free(song);
 }
 
 /* Frees an MMD1Block structure and all associated data */
 void MMD1Block_free(struct MMD1Block *block)
 {
-  nullcheck_void(block, MMD1block_free);
+  if (block != NULL) {
+    int i;
 
-  int i;
-
-  if (block->info) {
-    if (block->info->pagetable) {
-      for (i = 0; i < block->info->pagetable->num_pages - 1; i++) {
-	if (block->info->pagetable->page[i])
-	  free(block->info->pagetable->page[i]);
+    if (block->info) {
+      if (block->info->pagetable) {
+        for (i = 0; i < block->info->pagetable->num_pages - 1; i++) {
+          free(block->info->pagetable->page[i]);
+        }
+        free(block->info->pagetable);
       }
-      free(block->info->pagetable);
+      free(block->info);
     }
-    free(block->info);
+    free(block);
   }
-  free(block);
 }
 
 void PlaySeq_free(struct PlaySeq *playseq)
 {
-  nullcheck_void(playseq, PlaySeq_free);
-
   free(playseq);
 }
 
@@ -633,17 +627,17 @@ struct MMD2 *MMD2_parse(unsigned char *base, int offset)
     for (i = 0; i < mmd->song->numblocks; i++)
       mmd->blockarr[i] = MMD1Block_parse(base, getlong(base + getlong(data + 16) + i * 4));
     mmd->mmdflags = getbyte(data + 20);
-    if(getlong(data + 24) != 0) {
+    if (getlong(data + 24) != 0) {
       mmd->smplarr = (struct InstrHdr **)calloc(mmd->song->numsamples, sizeof(struct InstrHdr *));
       data2 = base + getlong(data + 24);
       /* samplearr */
       for (i = 0; i < mmd->song->numsamples; i++) {
-	mmd->smplarr[i] = (struct InstrHdr *)calloc(1, sizeof(struct InstrHdr));
-	if(getlong(data2 + i * 4) != 0) {
-	  data3 = base + getlong(data2 + i * 4);
-	  mmd->smplarr[i]->length = getlong(data3);
-	  mmd->smplarr[i]->type = getword(data3 + 4);
-	}
+        mmd->smplarr[i] = (struct InstrHdr *)calloc(1, sizeof(struct InstrHdr));
+        if (getlong(data2 + i * 4) != 0) {
+          data3 = base + getlong(data2 + i * 4);
+          mmd->smplarr[i]->length = getlong(data3);
+          mmd->smplarr[i]->type = getword(data3 + 4);
+        }
       }
     }
     mmd->expdata = MMD0exp_parse(base, getlong(data + 32));
@@ -741,7 +735,7 @@ struct MMD1Block *MMD1Block_parse(unsigned char *base, int offset)
       data3 = base + getlong(data2);
       block->info->hlmask = (unsigned int *)calloc(1 + block->lines / 32, sizeof(unsigned int));
       for (i = 0; i < 1 + block->lines / 32; i++)
-	block->info->hlmask[i] = getlong(data3 + i * sizeof(unsigned int));
+        block->info->hlmask[i] = getlong(data3 + i * sizeof(unsigned int));
     }
     /* blockname */
     block->info->blocknamelen = getlong(data2 + 8);
@@ -755,15 +749,15 @@ struct MMD1Block *MMD1Block_parse(unsigned char *base, int offset)
       block->info->pagetable = (struct BlockCmdPageTable *)calloc(1, sizeof(struct BlockCmdPageTable) + getword(data2) * sizeof(unsigned short *));
       block->info->pagetable->num_pages = getword(data2);
       for (i = 0; i < block->info->pagetable->num_pages; i++) {
-	data3 = base + getlong(data2 + 4 + i * 4);
-	/* page */
-	block->info->pagetable->page[i] = (unsigned short *)calloc(block->numtracks * (block->lines + 1) * 2, sizeof(unsigned short));
-	for (j = 0; j < (block->lines + 1); j++)
-	  for (k = 0; k < block->numtracks; k++) {
-	    unsigned char *commands = (unsigned char *)block->info->pagetable->page[i];
-	    commands[(j * block->numtracks + k) * 2] = getbyte(data3 + (j * block->numtracks + k) * 2);
-	    commands[(j * block->numtracks + k) * 2 + 1] = getbyte(data3 + (j * block->numtracks + k) * 2 + 1);
-	  }
+        data3 = base + getlong(data2 + 4 + i * 4);
+        /* page */
+        block->info->pagetable->page[i] = (unsigned short *)calloc(block->numtracks * (block->lines + 1) * 2, sizeof(unsigned short));
+        for (j = 0; j < (block->lines + 1); j++)
+          for (k = 0; k < block->numtracks; k++) {
+            unsigned char *commands = (unsigned char *)block->info->pagetable->page[i];
+            commands[(j * block->numtracks + k) * 2] = getbyte(data3 + (j * block->numtracks + k) * 2);
+            commands[(j * block->numtracks + k) * 2 + 1] = getbyte(data3 + (j * block->numtracks + k) * 2 + 1);
+          }
       }
     }
   }
@@ -800,16 +794,16 @@ struct MMD0exp *MMD0exp_parse(unsigned char *base, int offset)
       mmd0exp->exp_smp[i].suppress_midi_off = getbyte(data2 + i * mmd0exp->s_ext_entrsz + 2);
       mmd0exp->exp_smp[i].finetune = getbyte(data2 + i * mmd0exp->s_ext_entrsz + 3);
       if (mmd0exp->s_ext_entrsz > 4) {
-	mmd0exp->exp_smp[i].default_pitch = getbyte(data2 + i * mmd0exp->s_ext_entrsz + 4);
-	mmd0exp->exp_smp[i].instr_flags = getbyte(data2 + i * mmd0exp->s_ext_entrsz + 5);
-	mmd0exp->exp_smp[i].long_midi_preset = getword(data2 + i * mmd0exp->s_ext_entrsz + 6);
-	if (mmd0exp->s_ext_entrsz > 8)
-	  mmd0exp->exp_smp[i].output_device = getbyte(data2 + i * mmd0exp->s_ext_entrsz + 8);
+        mmd0exp->exp_smp[i].default_pitch = getbyte(data2 + i * mmd0exp->s_ext_entrsz + 4);
+        mmd0exp->exp_smp[i].instr_flags = getbyte(data2 + i * mmd0exp->s_ext_entrsz + 5);
+        mmd0exp->exp_smp[i].long_midi_preset = getword(data2 + i * mmd0exp->s_ext_entrsz + 6);
+        if (mmd0exp->s_ext_entrsz > 8)
+          mmd0exp->exp_smp[i].output_device = getbyte(data2 + i * mmd0exp->s_ext_entrsz + 8);
       }
     }
   }
   mmd0exp->annolen = getlong(data + 16);
-  if(getlong(data + 12) != 0) {
+  if (getlong(data + 12) != 0) {
     mmd0exp->annotxt = (unsigned char *)calloc(mmd0exp->annolen, sizeof(unsigned char));
     memcpy(mmd0exp->annotxt, base + getlong(data + 12), mmd0exp->annolen);
   }
@@ -869,10 +863,10 @@ struct MMD0exp *MMD0exp_parse(unsigned char *base, int offset)
 
       dumps[i]->ext_len = getword(data3 + 8);
       if (dumps[i]->ext_len >= 20)
-	memcpy(dumps[i]->name, data3 + 10, 20);
+        memcpy(dumps[i]->name, data3 + 10, 20);
     }
   }
-  
+
   /* mmdinfo */
   if (getlong(data + 56) != 0) {
     data2 = base + getlong(data + 56);
@@ -894,26 +888,26 @@ struct MMD0exp *MMD0exp_parse(unsigned char *base, int offset)
       mmd0exp->mmdrexx->trigcmd = (struct MMDARexxTrigCmd *)calloc(1, sizeof(struct MMDARexxTrigCmd));
       trig = mmd0exp->mmdrexx->trigcmd;
       while (trig != NULL) {
-	trig->cmdnum = getbyte(data2 + 4);
-	trig->cmdtype = getword(data2 + 6);
-	trig->cmd_len = getword(data2 + 16);
-	trig->port_len = getword(data2 + 18);
-	if (getlong(data2 + 8) != 0) {
-	  trig->cmd = (char *)calloc(trig->cmd_len, sizeof(char));
-	  /* command */
-	  memcpy(trig->cmd, base + getlong(data2 + 8), trig->cmd_len);
-	}
-	if (getlong(data2 + 12) != 0) {
-	  trig->port = (char *)calloc(trig->port_len, sizeof(char));
-	  /* command */
-	  memcpy(trig->port, base + getlong(data2 + 12), trig->port_len);
-	}
-	if (getlong(data2) != 0) {
-	  trig->next = (struct MMDARexxTrigCmd *)calloc(1, sizeof(struct MMDARexxTrigCmd));
-	  /* mmdarexxtrigcmd */
-	  data2 = base + getlong(data2);
-	}
-	trig = trig->next;
+        trig->cmdnum = getbyte(data2 + 4);
+        trig->cmdtype = getword(data2 + 6);
+        trig->cmd_len = getword(data2 + 16);
+        trig->port_len = getword(data2 + 18);
+        if (getlong(data2 + 8) != 0) {
+          trig->cmd = (char *)calloc(trig->cmd_len, sizeof(char));
+          /* command */
+          memcpy(trig->cmd, base + getlong(data2 + 8), trig->cmd_len);
+        }
+        if (getlong(data2 + 12) != 0) {
+          trig->port = (char *)calloc(trig->port_len, sizeof(char));
+          /* command */
+          memcpy(trig->port, base + getlong(data2 + 12), trig->port_len);
+        }
+        if (getlong(data2) != 0) {
+          trig->next = (struct MMDARexxTrigCmd *)calloc(1, sizeof(struct MMDARexxTrigCmd));
+          /* mmdarexxtrigcmd */
+          data2 = base + getlong(data2);
+        }
+        trig = trig->next;
       }
     }
   }
