@@ -1,6 +1,6 @@
 /* playseq.c
  *
- * Copyright 2002-2006 Vesa Halttunen
+ * Copyright 2002-2011 Vesa Halttunen
  *
  * This file is part of Tutka.
  *
@@ -20,6 +20,7 @@
  */
 
 #include <stdio.h>
+#include <string.h>
 #include "tutka.h"
 #include "playseq.h"
 
@@ -36,15 +37,11 @@ struct playseq *playseq_alloc()
 
 void playseq_free(struct playseq *playseq)
 {
-  nullcheck_void(playseq, playseq_free);
-
-  if (playseq->name)
+  if (playseq != NULL) {
     free(playseq->name);
-
-  if (playseq->blocknumbers)
     free(playseq->blocknumbers);
-
-  free(playseq);
+    free(playseq);
+  }
 }
 
 /* Inserts a new block in the block array in the given position */
@@ -110,39 +107,45 @@ struct playseq *playseq_parse(xmlDocPtr doc, xmlNsPtr ns, xmlNodePtr cur)
 {
   struct playseq *playseq = NULL;
 
-  if ((!xmlStrcmp(cur->name, "playingsequence")) && (cur->ns == ns)) {
-    char *prop;
+  if ((!xmlStrcmp(cur->name, (xmlChar *)"playingsequence")) && (cur->ns == ns)) {
+    xmlChar *prop;
 
     /* Allocate playseq */
     playseq = (struct playseq *)calloc(1, sizeof(struct playseq));
-    playseq->name = xmlGetProp(cur, "name");
+    prop = xmlGetProp(cur, (xmlChar *)"name");
+    if (prop != NULL) {
+      playseq->name = strdup((char *)prop);
+      xmlFree(prop);
+    }
 
     /* Get playseq contents */
     cur = cur->xmlChildrenNode;
     while (cur != NULL) {
-      if ((!xmlStrcmp(cur->name, "position")) && (cur->ns == ns)) {
-	xmlNodePtr temp;
-	int block = 0, number = 0;
+      if ((!xmlStrcmp(cur->name, (xmlChar *)"position")) && (cur->ns == ns)) {
+        xmlNodePtr temp;
+        int block = 0, number = 0;
 
-	/* Get the position */
-	prop = xmlGetProp(cur, "number");
-	if (prop != NULL)
-	  number = atoi(prop);
+        /* Get the position */
+        prop = xmlGetProp(cur, (xmlChar *)"number");
+        if (prop != NULL) {
+          number = atoi((char *)prop);
+          xmlFree(prop);
+        }
 
-	/* Get block number */
-	temp = cur->xmlChildrenNode;
-	if (temp != NULL)
-	  block = atoi(temp->content);
+        /* Get block number */
+        temp = cur->xmlChildrenNode;
+        if (temp != NULL)
+          block = atoi((char *)temp->content);
 
-	if (number >= playseq->length) {
-	  int i;
+        if (number >= playseq->length) {
+          int i;
 
-	  playseq->blocknumbers = realloc(playseq->blocknumbers, (number + 1) * sizeof(unsigned int));
-	  for (i = playseq->length; i <= number; i++)
-	    playseq->blocknumbers[i] = 0;
-	}
-	playseq->blocknumbers[number] = block;
-	playseq->length = number + 1;
+          playseq->blocknumbers = realloc(playseq->blocknumbers, (number + 1) * sizeof(unsigned int));
+          for (i = playseq->length; i <= number; i++)
+            playseq->blocknumbers[i] = 0;
+        }
+        playseq->blocknumbers[number] = block;
+        playseq->length = number + 1;
       }
       cur = cur->next;
     }
@@ -157,22 +160,22 @@ void playseq_save(struct playseq *playseq, int number, xmlNodePtr parent)
 {
   nullcheck_void(playseq, playseq_save);
 
-  char c[10];
+  xmlChar c[10];
   int i;
   xmlNodePtr node, subnode;
 
-  node = xmlNewChild(parent, NULL, "playingsequence", NULL);
-  snprintf(c, 10, "%d", number);
-  xmlSetProp(node, "number", c);
-  xmlSetProp(node, "name", playseq->name);
-  xmlAddChild(node, xmlNewText("\n"));
+  node = xmlNewChild(parent, NULL, (xmlChar *)"playingsequence", NULL);
+  snprintf((char *)c, 10, "%d", number);
+  xmlSetProp(node, (xmlChar *)"number", c);
+  xmlSetProp(node, (xmlChar *)"name", (xmlChar *)playseq->name);
+  xmlAddChild(node, xmlNewText((xmlChar *)"\n"));
   /* Add all blocks */
   for (i = 0; i < playseq->length; i++) {
-    snprintf(c, 10, "%d", playseq->blocknumbers[i]);
-    subnode = xmlNewChild(node, NULL, "position", c);
-    snprintf(c, 10, "%d", i);
-    xmlSetProp(subnode, "number", c);
-    xmlAddChild(node, xmlNewText("\n"));
+    snprintf((char *)c, 10, "%d", playseq->blocknumbers[i]);
+    subnode = xmlNewChild(node, NULL, (xmlChar *)"position", c);
+    snprintf((char *)c, 10, "%d", i);
+    xmlSetProp(subnode, (xmlChar *)"number", c);
+    xmlAddChild(node, xmlNewText((xmlChar *)"\n"));
   }
-  xmlAddChild(parent, xmlNewText("\n"));
+  xmlAddChild(parent, xmlNewText((xmlChar *)"\n"));
 }
