@@ -1,6 +1,6 @@
 /* smf.c
  *
- * Copyright 2002-2006 Vesa Halttunen
+ * Copyright 2002-2011 Vesa Halttunen
  *
  * This file is part of Tutka.
  *
@@ -38,12 +38,12 @@ int smf_length_get(struct smf *smf)
   nullcheck_int(smf, smf_length_get);
 
   int i, length = 14;
-  
+
   for (i = 0; i < smf->MThd->NumTracks; i++) {
     length += 8;
     length += smf->MTrks[i]->Length;
   }
-  
+
   return length;
 }
 
@@ -57,7 +57,7 @@ void smf_save(struct smf *smf, char *filename)
   int i, length = smf_length_get(smf);
   unsigned char *data = malloc(length);
   unsigned char *ptr = data;
-  
+
   /* Copy MThd structure */
   putbyte(ptr, smf->MThd->ID[0]);
   putbyte(ptr + 1, smf->MThd->ID[1]);
@@ -68,8 +68,8 @@ void smf_save(struct smf *smf, char *filename)
   putword(ptr + 10, smf->MThd->NumTracks);
   putword(ptr + 12, smf->MThd->Division);
   ptr += 14;
-  
-  /* Copy MTrk structures */ 
+
+  /* Copy MTrk structures */
   for (i = 0; i < smf->MThd->NumTracks; i++) {
     putbyte(ptr, smf->MTrks[i]->ID[0]);
     putbyte(ptr + 1, smf->MTrks[i]->ID[1]);
@@ -81,7 +81,7 @@ void smf_save(struct smf *smf, char *filename)
     ptr += smf->MTrks[i]->Length;
   }
 
-  /* Write to file */  
+  /* Write to file */
   if ((file = fopen(filename, "w")) == NULL) {
     free(data);
     return;
@@ -94,24 +94,20 @@ void smf_save(struct smf *smf, char *filename)
 /* Frees an smf structure and all associated data */
 void smf_free(struct smf *smf)
 {
-  nullcheck_void(smf, smf_free);
-  
-  if (smf->MTrks != NULL) {
-    if (smf->MThd != NULL) {
-      int i;
-      
-      for (i = 0; i < smf->MThd->NumTracks; i++) {
-        if (smf->MTrks[i]->Data != NULL)
-          free (smf->MTrks[i]->Data);
-        
-        free (smf->MTrks[i]);
+  if (smf != NULL) {
+    if (smf->MTrks != NULL) {
+      if (smf->MThd != NULL) {
+        int i;
+
+        for (i = 0; i < smf->MThd->NumTracks; i++) {
+          free(smf->MTrks[i]->Data);
+          free(smf->MTrks[i]);
+        }
       }
+      free(smf->MTrks);
     }
-    free (smf->MTrks);
+
+    free(smf->MThd);
+    free(smf);
   }
-  
-  if (smf->MThd != NULL)
-    free (smf->MThd);
-  
-  free(smf);
 }
