@@ -1,6 +1,6 @@
 /* song.c
  *
- * Copyright 2002-2006 Vesa Halttunen
+ * Copyright 2002-2011 Vesa Halttunen
  *
  * This file is part of Tutka.
  *
@@ -61,45 +61,55 @@ struct song *song_alloc(void)
 /* Frees a song structure and its contents */
 void song_free(struct song *song)
 {
-  nullcheck_void(song, song_free);
+  if (song != NULL) {
+    int i;
 
-  int i;
-
-  if (song->name)
     free(song->name);
 
-  if (song->numsections > 0)
-    free(song->sections);
+    if (song->numsections > 0)
+      free(song->sections);
 
-  if (song->numplayseqs > 0) {
-    for (i = 0; i < song->numplayseqs; i++)
-      playseq_free(song->playseqs[i]);
+    if (song->numplayseqs > 0) {
+      for (i = 0; i < song->numplayseqs; i++)
+        playseq_free(song->playseqs[i]);
 
-    free(song->playseqs);
+      free(song->playseqs);
+    }
+
+    if (song->numblocks > 0) {
+      for (i = 0; i < song->numblocks; i++)
+        block_free(song->blocks[i]);
+
+      free(song->blocks);
+    }
+
+    if (song->numinstruments > 0) {
+      for (i = 0; i < song->numinstruments; i++)
+        instrument_free(song->instruments[i]);
+
+      free(song->instruments);
+    }
+
+    if (song->maxtracks > 0) {
+      for (i = 0; i < song->maxtracks; i++) {
+        free(song->tracks[i]->name);
+        free(song->tracks[i]);
+      }
+      free(song->tracks);
+    }
+
+    if (song->nummessages > 0) {
+      for (i = 0; i < song->nummessages; i++) {
+        if (song->messages[i] != NULL) {
+          free(song->messages[i]->data);
+          free(song->messages[i]);
+        }
+      }
+      free(song->messages);
+    }
+
+    free(song);
   }
-
-  if (song->numblocks > 0) {
-    for (i = 0; i < song->numblocks; i++)
-      block_free(song->blocks[i]);
-
-    free(song->blocks);
-  }
-
-  if (song->numinstruments > 0) {
-    for (i = 0; i < song->numinstruments; i++)
-      if (song->instruments[i] != NULL)
-	instrument_free(song->instruments[i]);
-
-    free(song->instruments);
-  }
-
-  if (song->maxtracks > 0) {
-    for (i = 0; i < song->maxtracks; i++)
-      free(song->tracks[i]);
-    free(song->tracks);
-  }
-
-  free(song);
 }
 
 /* Inserts a new block in the block array in the given position */
@@ -132,7 +142,7 @@ void song_insert_block(struct song *song, int pos, int current)
 
     for (j = 0; j < song->playseqs[i]->length; j++)
       if (song->playseqs[i]->blocknumbers[j] >= pos)
-	song->playseqs[i]->blocknumbers[j]++;
+        song->playseqs[i]->blocknumbers[j]++;
   }
 }
 
@@ -167,8 +177,8 @@ void song_delete_block(struct song *song, int pos)
       int j;
 
       for (j = 0; j < song->playseqs[i]->length; j++)
-	if (song->playseqs[i]->blocknumbers[j] >= pos && song->playseqs[i]->blocknumbers[j] > 0)
-	  song->playseqs[i]->blocknumbers[j]--;
+        if (song->playseqs[i]->blocknumbers[j] >= pos && song->playseqs[i]->blocknumbers[j] > 0)
+          song->playseqs[i]->blocknumbers[j]--;
     }
   }
 }
@@ -233,12 +243,11 @@ void song_delete_playseq(struct song *song, int pos)
     /* Update section lists */
     for (i = 0; i < song->numsections; i++) {
       if (song->sections[i] >= pos && song->sections[i] > 0)
-	song->sections[i]--;
+        song->sections[i]--;
     }
   }
 }
 
-
 /* Inserts a new section in the section array in the given position */
 void song_insert_section(struct song *song, int pos)
 {
@@ -337,10 +346,8 @@ void song_delete_message(struct song *song, int pos)
   /* Don't delete inexisting messages */
   if (song->nummessages > 0) {
     /* Free the message in question */
-    if (song->messages[pos]->data != NULL)
-      free(song->messages[pos]->data);
-    if (song->messages[pos]->name != NULL)
-      free(song->messages[pos]->name);
+    free(song->messages[pos]->data);
+    free(song->messages[pos]->name);
     free(song->messages[pos]);
     song->nummessages--;
 
@@ -411,15 +418,14 @@ int song_check_maxtracks(struct song *song)
   } else {
     /* Tracks removed: free track datas */
     for (i = max; i < oldmax; i++) {
-      if (song->tracks[i]->name != NULL)
-        free (song->tracks[i]->name);
-      free (song->tracks[i]);
+      free(song->tracks[i]->name);
+      free(song->tracks[i]);
     }
 
     /* Reallocate tracks array */
     song->tracks = (struct track **)realloc(song->tracks, max * sizeof(struct track *));
   }
-  
+
   song->maxtracks = max;
   return 1;
 }
@@ -435,8 +441,8 @@ void song_check_instrument(struct song *song, int instrument, unsigned short def
 
       song->instruments = realloc(song->instruments, (instrument + 1) * sizeof(struct instrument *));
       for (i = song->numinstruments; i < (instrument + 1); i++) {
-	song->instruments[i] = instrument_alloc();
-	song->instruments[i]->outputs[0]->midiinterface = defaultmidiinterface;
+        song->instruments[i] = instrument_alloc();
+        song->instruments[i]->outputs[0]->midiinterface = defaultmidiinterface;
       }
 
       song->numinstruments = instrument + 1;
@@ -500,262 +506,297 @@ struct song *song_parse(xmlDocPtr doc, xmlNsPtr ns, xmlNodePtr cur)
 {
   struct song *song = NULL;
 
-  if ((!xmlStrcmp(cur->name, "song")) && (cur->ns == ns)) {
-    char *prop;
+  if ((!xmlStrcmp(cur->name, (xmlChar *)"song")) && (cur->ns == ns)) {
+    xmlChar *prop;
     /* Allocate song */
     song = (struct song *)calloc(1, sizeof(struct song));
-    song->name = xmlGetProp(cur, "name");
-    prop = xmlGetProp(cur, "tempo");
-    if (prop != NULL)
-      song->tempo = atoi(prop);
-    prop = xmlGetProp(cur, "ticksperline");
-    if (prop != NULL)
-      song->ticksperline = atoi(prop);
-    prop = xmlGetProp(cur, "mastervolume");
-    if (prop != NULL)
-      song->mastervolume = atoi(prop);
-    prop = xmlGetProp(cur, "sendsync");
-    if (prop != NULL)
-      song->sendsync = atoi(prop);
+    prop = xmlGetProp(cur, (xmlChar *)"name");
+    if (prop != NULL) {
+      song->name = strdup((char *)prop);
+      xmlFree(prop);
+    }
+
+    prop = xmlGetProp(cur, (xmlChar *)"tempo");
+    if (prop != NULL) {
+      song->tempo = atoi((char *)prop);
+      xmlFree(prop);
+    }
+
+    prop = xmlGetProp(cur, (xmlChar *)"ticksperline");
+    if (prop != NULL) {
+      song->ticksperline = atoi((char *)prop);
+      xmlFree(prop);
+    }
+
+    prop = xmlGetProp(cur, (xmlChar *)"mastervolume");
+    if (prop != NULL) {
+      song->mastervolume = atoi((char *)prop);
+      xmlFree(prop);
+    }
+
+    prop = xmlGetProp(cur, (xmlChar *)"sendsync");
+    if (prop != NULL) {
+      song->sendsync = atoi((char *)prop);
+      xmlFree(prop);
+    }
 
     cur = cur->xmlChildrenNode;
     while (cur != NULL) {
-      if ((!xmlStrcmp(cur->name, "blocks")) && (cur->ns == ns)) {
-	/* Parse and add all block elements */
-	xmlNodePtr temp = cur->xmlChildrenNode;
+      if ((!xmlStrcmp(cur->name, (xmlChar *)"blocks")) && (cur->ns == ns)) {
+        /* Parse and add all block elements */
+        xmlNodePtr temp = cur->xmlChildrenNode;
 
-	while (temp != NULL) {
-	  if (!(xmlIsBlankNode(temp))) {
-	    int number = -1;
-	    struct block *block = block_parse(doc, ns, temp);
+        while (temp != NULL) {
+          if (!(xmlIsBlankNode(temp))) {
+            int number = -1;
+            struct block *block = block_parse(doc, ns, temp);
 
-	    if (block != NULL) {
-	      prop = xmlGetProp(temp, "number");
-	      if (prop != NULL)
-		number = atoi(prop);
-
-	      if (number >= song->numblocks) {
-		int i;
-
-		song->blocks = realloc(song->blocks, (number + 1) * sizeof(struct block *));
-		for (i = song->numblocks; i <= number; i++)
-		  song->blocks[i] = NULL;
-	      }
-	      if (song->blocks[number] != NULL)
-		block_free(song->blocks[number]);
-	      song->blocks[number] = block;
-	      song->numblocks = number + 1;
-	    }
-	  }
-
-	  temp = temp->next;
-	}
-      } else if ((!xmlStrcmp(cur->name, "sections")) && (cur->ns == ns)) {
-	/* Parse and add all section elements */
-	xmlNodePtr temp = cur->xmlChildrenNode;
-
-	while (temp != NULL) {
-	  if (!(xmlIsBlankNode(temp))) {
-	    int number = -1;
-
-	    /* The section number is required */
-	    if ((!xmlStrcmp(temp->name, "section")) && (temp->ns == ns)) {
-	      xmlNodePtr temp2;
-	      prop = xmlGetProp(temp, "number");
-	      if (prop != NULL)
-		number = atoi(prop);
-
-	      if (number >= song->numsections) {
-		int i;
-
-		song->sections = realloc(song->sections, (number + 1) * sizeof(unsigned int));
-		for (i = song->numsections; i <= number; i++)
-		  song->sections[i] = 0;
-	      }
-
-	      /* Get playing sequence */
-	      temp2 = temp->xmlChildrenNode;
-	      if (temp2 != NULL)
-		song->sections[number] = atoi(temp2->content);
-	      song->numsections = number + 1;
-	    } else if (temp->type != XML_COMMENT_NODE)
-	      fprintf(stderr, "XML error: expected section, got %s\n", temp->name);
-	  }
-
-	  temp = temp->next;
-	}
-      } else if ((!xmlStrcmp(cur->name, "playingsequences")) && (cur->ns == ns)) {
-	/* Parse and add all playingsequence elements */
-
-	xmlNodePtr temp = cur->xmlChildrenNode;
-	while (temp != NULL) {
-	  if (!(xmlIsBlankNode(temp))) {
-	    int number = -1;
-	    struct playseq *playseq = playseq_parse(doc, ns, temp);
-
-	    if (playseq != NULL) {
-	      prop = xmlGetProp(temp, "number");
-	      if (prop != NULL)
-		number = atoi(prop);
-
-	      if (number >= song->numplayseqs) {
-		int i;
-
-		song->playseqs = realloc(song->playseqs, (number + 1) * sizeof(struct playseq *));
-		for (i = song->numplayseqs; i <= number; i++)
-		  song->playseqs[i] = NULL;
-	      }
-	      if (song->playseqs[number] != NULL)
-		playseq_free(song->playseqs[number]);
-	      song->playseqs[number] = playseq;
-	      song->numplayseqs = number + 1;
-	    }
-	  }
-
-	  temp = temp->next;
-	}
-      } else if ((!xmlStrcmp(cur->name, "instruments")) && (cur->ns == ns)) {
-	/* Parse and add all instrument elements */
-	xmlNodePtr temp = cur->xmlChildrenNode;
-
-	while (temp != NULL) {
-	  if (!(xmlIsBlankNode(temp))) {
-	    int number = -1;
-	    struct instrument *instrument = instrument_parse(doc, ns, temp);
-
-	    if (instrument != NULL) {
-	      prop = xmlGetProp(temp, "number");
-	      if (prop != NULL)
-		number = atoi(prop);
-
-	      if (number >= song->numinstruments) {
-		int i;
-
-		song->instruments = realloc(song->instruments, (number + 1) * sizeof(struct instrument *));
-		for (i = song->numinstruments; i <= number; i++)
-		  song->instruments[i] = NULL;
-	      }
-	      if (song->instruments[number] != NULL)
-		instrument_free(song->instruments[number]);
-	      song->instruments[number] = instrument;
-	      song->numinstruments = number + 1;
-	    }
-	  }
-
-	  temp = temp->next;
-	}
-      } else if ((!xmlStrcmp(cur->name, "tracks")) && (cur->ns == ns)) {
-	/* Parse and add all track elements */
-	xmlNodePtr temp = cur->xmlChildrenNode;
-
-	while (temp != NULL) {
-	  if (!(xmlIsBlankNode(temp))) {
-	    int track = -1;
-
-	    /* The track number is required */
-	    if ((!xmlStrcmp(temp->name, "track")) && (temp->ns == ns)) {
-	      xmlNodePtr temp2;
-	      prop = xmlGetProp(temp, "number");
-	      if (prop != NULL)
-		track = atoi(prop);
-
-	      if (track >= song->maxtracks) {
-		int i;
-
-		song->tracks = (struct track **)realloc(song->tracks, (track + 1) * sizeof(struct track *));
-		for (i = song->maxtracks; i <= track; i++)
-                  song->tracks[i] = (struct track *)calloc(1, sizeof(struct track));
-	      }
-
-	      /* Get volume, mute, solo and name */
-	      prop = xmlGetProp(temp, "volume");
-	      if (prop != NULL)
-		song->tracks[track]->volume = atoi(prop);
-	      prop = xmlGetProp(temp, "mute");
-	      if (prop != NULL)
-		song->tracks[track]->mute = atoi(prop);
-	      prop = xmlGetProp(temp, "solo");
-	      if (prop != NULL)
-		song->tracks[track]->solo = atoi(prop);
-
-	      temp2 = temp->xmlChildrenNode;
-	      if (temp2 != NULL)
-		song->tracks[track]->name = strdup(temp2->content);
-	      song->maxtracks = track + 1;
-	    } else if (temp->type != XML_COMMENT_NODE)
-	      fprintf(stderr, "XML error: expected section, got %s\n", temp->name);
-	  }
-
-	  temp = temp->next;
-	}
-      } else if ((!xmlStrcmp(cur->name, "trackvolumes")) && (cur->ns == ns)) {
-	/* Backwards compatibility: parse and add all track volume elements */
-	xmlNodePtr temp = cur->xmlChildrenNode;
-
-	while (temp != NULL) {
-	  if (!(xmlIsBlankNode(temp))) {
-	    int track = -1;
-
-	    /* The track number is required */
-	    if ((!xmlStrcmp(temp->name, "trackvolume")) && (temp->ns == ns)) {
-	      xmlNodePtr temp2;
-              prop = xmlGetProp(temp, "track");
-	      if (prop != NULL)
-		track = atoi(prop);
-
-	      if (track >= song->maxtracks) {
-		int i;
-
-		song->tracks = (struct track **)realloc(song->tracks, (track + 1) * sizeof(struct track *));
-		for (i = song->maxtracks; i <= track; i++)
-                  song->tracks[i] = (struct track *)calloc(1, sizeof(struct track));
-	      }
-
-	      /* Get the volume */
-	      temp2 = temp->xmlChildrenNode;
-	      if (temp2 != NULL) {
-		song->tracks[track]->volume = (atoi(temp2->content) & 127);
-		song->tracks[track]->mute = (atoi(temp2->content) & 128) >> 7;
+            if (block != NULL) {
+              prop = xmlGetProp(temp, (xmlChar *)"number");
+              if (prop != NULL) {
+                number = atoi((char *)prop);
+                xmlFree(prop);
               }
-	      song->maxtracks = track + 1;
-	    } else if (temp->type != XML_COMMENT_NODE)
-	      fprintf(stderr, "XML error: expected section, got %s\n", temp->name);
-	  }
 
-	  temp = temp->next;
-	}
-      } else if ((!xmlStrcmp(cur->name, "messages")) && (cur->ns == ns)) {
-	/* Parse and add all Message elements */
-	xmlNodePtr temp = cur->xmlChildrenNode;
+              if (number >= song->numblocks) {
+                int i;
 
-	while (temp != NULL) {
-	  if (!(xmlIsBlankNode(temp))) {
-	    int number = -1;
-	    struct message *message = message_parse(doc, ns, temp);
+                song->blocks = realloc(song->blocks, (number + 1) * sizeof(struct block *));
+                for (i = song->numblocks; i <= number; i++)
+                  song->blocks[i] = NULL;
+              }
+              block_free(song->blocks[number]);
+              song->blocks[number] = block;
+              song->numblocks = number + 1;
+            }
+          }
 
-	    if (message != NULL) {
-	      prop = xmlGetProp(temp, "number");
-	      if (prop != NULL)
-		number = atoi(prop);
+          temp = temp->next;
+        }
+      } else if ((!xmlStrcmp(cur->name, (xmlChar *)"sections")) && (cur->ns == ns)) {
+        /* Parse and add all section elements */
+        xmlNodePtr temp = cur->xmlChildrenNode;
 
-	      if (number >= song->nummessages) {
-		int i;
+        while (temp != NULL) {
+          if (!(xmlIsBlankNode(temp))) {
+            int number = -1;
 
-		song->messages = realloc(song->messages, (number + 1) * sizeof(struct message *));
-		for (i = song->nummessages; i <= number; i++)
-		  song->messages[i] = NULL;
-	      }
-	      if (song->messages[number] != NULL) {
-		free(song->messages[number]->data);
-		free(song->messages[number]);
-	      }
-	      song->messages[number] = message;
-	      song->nummessages = number + 1;
-	    }
-	  }
+            /* The section number is required */
+            if ((!xmlStrcmp(temp->name, (xmlChar *)"section")) && (temp->ns == ns)) {
+              xmlNodePtr temp2;
+              prop = xmlGetProp(temp, (xmlChar *)"number");
+              if (prop != NULL) {
+                number = atoi((char *)prop);
+                xmlFree(prop);
+              }
 
-	  temp = temp->next;
-	}
+              if (number >= song->numsections) {
+                int i;
+
+                song->sections = realloc(song->sections, (number + 1) * sizeof(unsigned int));
+                for (i = song->numsections; i <= number; i++)
+                  song->sections[i] = 0;
+              }
+
+              /* Get playing sequence */
+              temp2 = temp->xmlChildrenNode;
+              if (temp2 != NULL)
+                song->sections[number] = atoi((char *)temp2->content);
+              song->numsections = number + 1;
+            } else if (temp->type != XML_COMMENT_NODE)
+              fprintf(stderr, "XML error: expected section, got %s\n", temp->name);
+          }
+
+          temp = temp->next;
+        }
+      } else if ((!xmlStrcmp(cur->name, (xmlChar *)"playingsequences")) && (cur->ns == ns)) {
+        /* Parse and add all playingsequence elements */
+
+        xmlNodePtr temp = cur->xmlChildrenNode;
+        while (temp != NULL) {
+          if (!(xmlIsBlankNode(temp))) {
+            int number = -1;
+            struct playseq *playseq = playseq_parse(doc, ns, temp);
+
+            if (playseq != NULL) {
+              prop = xmlGetProp(temp, (xmlChar *)"number");
+              if (prop != NULL) {
+                number = atoi((char *)prop);
+                xmlFree(prop);
+              }
+
+              if (number >= song->numplayseqs) {
+                int i;
+
+                song->playseqs = realloc(song->playseqs, (number + 1) * sizeof(struct playseq *));
+                for (i = song->numplayseqs; i <= number; i++)
+                  song->playseqs[i] = NULL;
+              }
+              playseq_free(song->playseqs[number]);
+              song->playseqs[number] = playseq;
+              song->numplayseqs = number + 1;
+            }
+          }
+
+          temp = temp->next;
+        }
+      } else if ((!xmlStrcmp(cur->name, (xmlChar *)"instruments")) && (cur->ns == ns)) {
+        /* Parse and add all instrument elements */
+        xmlNodePtr temp = cur->xmlChildrenNode;
+
+        while (temp != NULL) {
+          if (!(xmlIsBlankNode(temp))) {
+            int number = -1;
+            struct instrument *instrument = instrument_parse(doc, ns, temp);
+
+            if (instrument != NULL) {
+              prop = xmlGetProp(temp, (xmlChar *)"number");
+              if (prop != NULL) {
+                number = atoi((char *)prop);
+                xmlFree(prop);
+              }
+
+              if (number >= song->numinstruments) {
+                int i;
+
+                song->instruments = realloc(song->instruments, (number + 1) * sizeof(struct instrument *));
+                for (i = song->numinstruments; i <= number; i++)
+                  song->instruments[i] = NULL;
+              }
+              instrument_free(song->instruments[number]);
+              song->instruments[number] = instrument;
+              song->numinstruments = number + 1;
+            }
+          }
+
+          temp = temp->next;
+        }
+      } else if ((!xmlStrcmp(cur->name, (xmlChar *)"tracks")) && (cur->ns == ns)) {
+        /* Parse and add all track elements */
+        xmlNodePtr temp = cur->xmlChildrenNode;
+
+        while (temp != NULL) {
+          if (!(xmlIsBlankNode(temp))) {
+            int track = -1;
+
+            /* The track number is required */
+            if ((!xmlStrcmp(temp->name, (xmlChar *)"track")) && (temp->ns == ns)) {
+              xmlNodePtr temp2;
+              prop = xmlGetProp(temp, (xmlChar *)"number");
+              if (prop != NULL) {
+                track = atoi((char *)prop);
+                xmlFree(prop);
+              }
+
+              if (track >= song->maxtracks) {
+                int i;
+
+                song->tracks = (struct track **)realloc(song->tracks, (track + 1) * sizeof(struct track *));
+                for (i = song->maxtracks; i <= track; i++)
+                  song->tracks[i] = (struct track *)calloc(1, sizeof(struct track));
+              }
+
+              /* Get volume, mute, solo and name */
+              prop = xmlGetProp(temp, (xmlChar *)"volume");
+              if (prop != NULL) {
+                song->tracks[track]->volume = atoi((char *)prop);
+                xmlFree(prop);
+              }
+
+              prop = xmlGetProp(temp, (xmlChar *)"mute");
+              if (prop != NULL) {
+                song->tracks[track]->mute = atoi((char *)prop);
+                xmlFree(prop);
+              }
+
+              prop = xmlGetProp(temp, (xmlChar *)"solo");
+              if (prop != NULL) {
+                song->tracks[track]->solo = atoi((char *)prop);
+                xmlFree(prop);
+              }
+
+              temp2 = temp->xmlChildrenNode;
+              if (temp2 != NULL)
+                song->tracks[track]->name = strdup((char *)temp2->content);
+              song->maxtracks = track + 1;
+            } else if (temp->type != XML_COMMENT_NODE)
+              fprintf(stderr, "XML error: expected section, got %s\n", temp->name);
+          }
+
+          temp = temp->next;
+        }
+      } else if ((!xmlStrcmp(cur->name, (xmlChar *)"trackvolumes")) && (cur->ns == ns)) {
+        /* Backwards compatibility: parse and add all track volume elements */
+        xmlNodePtr temp = cur->xmlChildrenNode;
+
+        while (temp != NULL) {
+          if (!(xmlIsBlankNode(temp))) {
+            int track = -1;
+
+            /* The track number is required */
+            if ((!xmlStrcmp(temp->name, (xmlChar *)"trackvolume")) && (temp->ns == ns)) {
+              xmlNodePtr temp2;
+              prop = xmlGetProp(temp, (xmlChar *)"track");
+              if (prop != NULL) {
+                track = atoi((char *)prop);
+                xmlFree(prop);
+              }
+
+              if (track >= song->maxtracks) {
+                int i;
+
+                song->tracks = (struct track **)realloc(song->tracks, (track + 1) * sizeof(struct track *));
+                for (i = song->maxtracks; i <= track; i++)
+                  song->tracks[i] = (struct track *)calloc(1, sizeof(struct track));
+              }
+
+              /* Get the volume */
+              temp2 = temp->xmlChildrenNode;
+              if (temp2 != NULL) {
+                song->tracks[track]->volume = (atoi((char *)temp2->content) & 127);
+                song->tracks[track]->mute = (atoi((char *)temp2->content) & 128) >> 7;
+              }
+              song->maxtracks = track + 1;
+            } else if (temp->type != XML_COMMENT_NODE)
+              fprintf(stderr, "XML error: expected section, got %s\n", temp->name);
+          }
+
+          temp = temp->next;
+        }
+      } else if ((!xmlStrcmp(cur->name, (xmlChar *)"messages")) && (cur->ns == ns)) {
+        /* Parse and add all Message elements */
+        xmlNodePtr temp = cur->xmlChildrenNode;
+
+        while (temp != NULL) {
+          if (!(xmlIsBlankNode(temp))) {
+            int number = -1;
+            struct message *message = message_parse(doc, ns, temp);
+
+            if (message != NULL) {
+              prop = xmlGetProp(temp, (xmlChar *)"number");
+              if (prop != NULL) {
+                number = atoi((char *)prop);
+                xmlFree(prop);
+              }
+
+              if (number >= song->nummessages) {
+                int i;
+
+                song->messages = realloc(song->messages, (number + 1) * sizeof(struct message *));
+                for (i = song->nummessages; i <= number; i++)
+                  song->messages[i] = NULL;
+              }
+              if (song->messages[number] != NULL) {
+                free(song->messages[number]->data);
+                free(song->messages[number]);
+              }
+              song->messages[number] = message;
+              song->nummessages = number + 1;
+            }
+          }
+
+          temp = temp->next;
+        }
       }
       cur = cur->next;
     }
@@ -773,77 +814,77 @@ void song_save(struct song *song, char *filename)
   xmlDocPtr doc;
   xmlNodePtr node, subnode;
   int i;
-  char c[10];
+  xmlChar c[10];
 
-  doc = xmlNewDoc("1.0");
-  doc->xmlRootNode = xmlNewDocNode(doc, NULL, "song", NULL);
-  xmlSetProp(doc->xmlRootNode, "name", song->name);
-  snprintf(c, 10, "%d", song->tempo);
-  xmlSetProp(doc->xmlRootNode, "tempo", c);
-  snprintf(c, 10, "%d", song->ticksperline);
-  xmlSetProp(doc->xmlRootNode, "ticksperline", c);
-  snprintf(c, 10, "%d", song->mastervolume);
-  xmlSetProp(doc->xmlRootNode, "mastervolume", c);
-  snprintf(c, 10, "%d", song->sendsync);
-  xmlSetProp(doc->xmlRootNode, "sendsync", c);
-  xmlAddChild(doc->xmlRootNode, xmlNewText("\n\n"));
+  doc = xmlNewDoc((xmlChar *)"1.0");
+  doc->xmlRootNode = xmlNewDocNode(doc, NULL, (xmlChar *)"song", NULL);
+  xmlSetProp(doc->xmlRootNode, (xmlChar *)"name", (xmlChar *)song->name);
+  snprintf((char *)c, 10, "%d", song->tempo);
+  xmlSetProp(doc->xmlRootNode, (xmlChar *)"tempo", c);
+  snprintf((char *)c, 10, "%d", song->ticksperline);
+  xmlSetProp(doc->xmlRootNode, (xmlChar *)"ticksperline", c);
+  snprintf((char *)c, 10, "%d", song->mastervolume);
+  xmlSetProp(doc->xmlRootNode, (xmlChar *)"mastervolume", c);
+  snprintf((char *)c, 10, "%d", song->sendsync);
+  xmlSetProp(doc->xmlRootNode, (xmlChar *)"sendsync", c);
+  xmlAddChild(doc->xmlRootNode, xmlNewText((xmlChar *)"\n\n"));
 
-  node = xmlNewChild(doc->xmlRootNode, NULL, "blocks", NULL);
-  xmlAddChild(node, xmlNewText("\n"));
+  node = xmlNewChild(doc->xmlRootNode, NULL, (xmlChar *)"blocks", NULL);
+  xmlAddChild(node, xmlNewText((xmlChar *)"\n"));
   /* Add all blocks */
   for (i = 0; i < song->numblocks; i++)
     block_save(song->blocks[i], i, node);
-  xmlAddChild(doc->xmlRootNode, xmlNewText("\n\n"));
+  xmlAddChild(doc->xmlRootNode, xmlNewText((xmlChar *)"\n\n"));
 
-  node = xmlNewChild(doc->xmlRootNode, NULL, "sections", NULL);
-  xmlAddChild(node, xmlNewText("\n"));
+  node = xmlNewChild(doc->xmlRootNode, NULL, (xmlChar *)"sections", NULL);
+  xmlAddChild(node, xmlNewText((xmlChar *)"\n"));
   /* Add all sections */
   for (i = 0; i < song->numsections; i++) {
-    snprintf(c, 10, "%d", song->sections[i]);
-    subnode = xmlNewChild(node, NULL, "section", c);
-    snprintf(c, 10, "%d", i);
-    xmlSetProp(subnode, "number", c);
-    xmlAddChild(node, xmlNewText("\n"));
+    snprintf((char *)c, 10, "%d", song->sections[i]);
+    subnode = xmlNewChild(node, NULL, (xmlChar *)"section", c);
+    snprintf((char *)c, 10, "%d", i);
+    xmlSetProp(subnode, (xmlChar *)"number", c);
+    xmlAddChild(node, xmlNewText((xmlChar *)"\n"));
   }
-  xmlAddChild(doc->xmlRootNode, xmlNewText("\n\n"));
+  xmlAddChild(doc->xmlRootNode, xmlNewText((xmlChar *)"\n\n"));
 
-  node = xmlNewChild(doc->xmlRootNode, NULL, "playingsequences", NULL);
-  xmlAddChild(node, xmlNewText("\n"));
+  node = xmlNewChild(doc->xmlRootNode, NULL, (xmlChar *)"playingsequences", NULL);
+  xmlAddChild(node, xmlNewText((xmlChar *)"\n"));
   /* Add all playing sequences */
   for (i = 0; i < song->numplayseqs; i++)
     playseq_save(song->playseqs[i], i, node);
-  xmlAddChild(doc->xmlRootNode, xmlNewText("\n\n"));
+  xmlAddChild(doc->xmlRootNode, xmlNewText((xmlChar *)"\n\n"));
 
-  node = xmlNewChild(doc->xmlRootNode, NULL, "instruments", NULL);
-  xmlAddChild(node, xmlNewText("\n"));
+  node = xmlNewChild(doc->xmlRootNode, NULL, (xmlChar *)"instruments", NULL);
+  xmlAddChild(node, xmlNewText((xmlChar *)"\n"));
   /* Add all instruments */
   for (i = 0; i < song->numinstruments; i++)
     instrument_save(song->instruments[i], i, node);
-  xmlAddChild(doc->xmlRootNode, xmlNewText("\n\n"));
+  xmlAddChild(doc->xmlRootNode, xmlNewText((xmlChar *)"\n\n"));
 
-  node = xmlNewChild(doc->xmlRootNode, NULL, "tracks", NULL);
-  xmlAddChild(node, xmlNewText("\n"));
+  node = xmlNewChild(doc->xmlRootNode, NULL, (xmlChar *)"tracks", NULL);
+  xmlAddChild(node, xmlNewText((xmlChar *)"\n"));
   /* Add all tracks */
   for (i = 0; i < song->maxtracks; i++) {
-    subnode = xmlNewChild(node, NULL, "track", song->tracks[i]->name);
-    snprintf(c, 10, "%d", i);
-    xmlSetProp(subnode, "number", c);
-    snprintf(c, 10, "%d", song->tracks[i]->volume);
-    xmlSetProp(subnode, "volume", c);
-    snprintf(c, 10, "%d", song->tracks[i]->mute);
-    xmlSetProp(subnode, "mute", c);
-    snprintf(c, 10, "%d", song->tracks[i]->solo);
-    xmlSetProp(subnode, "solo", c);
-    xmlAddChild(node, xmlNewText("\n"));
+    subnode = xmlNewChild(node, NULL, (xmlChar *)"track", (xmlChar *)song->tracks[i]->name);
+    snprintf((char *)c, 10, "%d", i);
+    xmlSetProp(subnode, (xmlChar *)"number", c);
+    snprintf((char *)c, 10, "%d", song->tracks[i]->volume);
+    xmlSetProp(subnode, (xmlChar *)"volume", c);
+    snprintf((char *)c, 10, "%d", song->tracks[i]->mute);
+    xmlSetProp(subnode, (xmlChar *)"mute", c);
+    snprintf((char *)c, 10, "%d", song->tracks[i]->solo);
+    xmlSetProp(subnode, (xmlChar *)"solo", c);
+    xmlAddChild(node, xmlNewText((xmlChar *)"\n"));
   }
-  xmlAddChild(doc->xmlRootNode, xmlNewText("\n\n"));
+  xmlAddChild(doc->xmlRootNode, xmlNewText((xmlChar *)"\n\n"));
 
-  node = xmlNewChild(doc->xmlRootNode, NULL, "messages", NULL);
-  xmlAddChild(node, xmlNewText("\n"));
+  node = xmlNewChild(doc->xmlRootNode, NULL, (xmlChar *)"messages", NULL);
+  xmlAddChild(node, xmlNewText((xmlChar *)"\n"));
   /* Add all Message messages */
   for (i = 0; i < song->nummessages; i++)
     message_save(song->messages[i], i, node);
-  xmlAddChild(doc->xmlRootNode, xmlNewText("\n\n"));
+  xmlAddChild(doc->xmlRootNode, xmlNewText((xmlChar *)"\n\n"));
 
   /* Use stdout if no filename is specified */
   if (filename == NULL)
@@ -853,24 +894,12 @@ void song_save(struct song *song, char *filename)
   xmlFreeDoc(doc);
 }
 
-
 void song_print(struct song *song, char *path)
 {
   nullcheck_void(song, song_print);
-  
-  char *notestrings[] = {
-    "---",
-    "C-1", "C#1", "D-1", "D#1", "E-1", "F-1", "F#1", "G-1", "G#1", "A-1", "A#1", "B-1",
-    "C-2", "C#2", "D-2", "D#2", "E-2", "F-2", "F#2", "G-2", "G#2", "A-2", "A#2", "B-2",
-    "C-3", "C#3", "D-3", "D#3", "E-3", "F-3", "F#3", "G-3", "G#3", "A-3", "A#3", "B-3",
-    "C-4", "C#4", "D-4", "D#4", "E-4", "F-4", "F#4", "G-4", "G#4", "A-4", "A#4", "B-4",
-    "C-5", "C#5", "D-5", "D#5", "E-5", "F-5", "F#5", "G-5", "G#5", "A-5", "A#5", "B-5",
-    "C-6", "C#6", "D-6", "D#6", "E-6", "F-6", "F#6", "G-6", "G#6", "A-6", "A#6", "B-6",
-    "C-7", "C#7", "D-7", "D#7", "E-7", "F-7", "F#7", "G-7", "G#7", "A-7", "A#7", "B-7",
-    "C-8", "C#8", "D-8", "D#8", "E-8", "F-8", "F#8", "G-8", "G#8", "A-8", "A#8", "B-8",
-    "C-9", "C#9", "D-9", "D#9", "E-9", "F-9", "F#9", "G-9", "G#9", "A-9", "A#9", "B-9",
-    "C-A", "C#A", "D-A", "D#A", "E-A", "F-A", "F#A", "G-A", "G#A", "A-A", "A#A", "B-A"
-  };
+
+  char *notestrings[] = { "---", "C-1", "C#1", "D-1", "D#1", "E-1", "F-1", "F#1", "G-1", "G#1", "A-1", "A#1", "B-1", "C-2", "C#2", "D-2", "D#2", "E-2", "F-2", "F#2", "G-2", "G#2", "A-2", "A#2", "B-2", "C-3", "C#3", "D-3", "D#3", "E-3", "F-3", "F#3", "G-3", "G#3", "A-3", "A#3", "B-3", "C-4", "C#4", "D-4", "D#4", "E-4", "F-4", "F#4", "G-4", "G#4", "A-4", "A#4", "B-4", "C-5", "C#5", "D-5", "D#5", "E-5", "F-5", "F#5", "G-5", "G#5", "A-5", "A#5", "B-5", "C-6", "C#6", "D-6", "D#6", "E-6", "F-6", "F#6", "G-6", "G#6", "A-6", "A#6", "B-6", "C-7", "C#7", "D-7", "D#7", "E-7", "F-7", "F#7", "G-7", "G#7", "A-7", "A#7", "B-7", "C-8", "C#8", "D-8", "D#8", "E-8", "F-8", "F#8", "G-8", "G#8", "A-8", "A#8", "B-8", "C-9", "C#9", "D-9", "D#9", "E-9", "F-9", "F#9", "G-9", "G#9", "A-9", "A#9", "B-9", "C-A",
+      "C#A", "D-A", "D#A", "E-A", "F-A", "F#A", "G-A", "G#A", "A-A", "A#A", "B-A" };
   char *instrumentstrings = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ";
   FILE *file;
   int i, j, k;
@@ -891,7 +920,7 @@ void song_print(struct song *song, char *path)
     fprintf(file, "Master Volume = %d, Track Volumes:", song->mastervolume);
     for (i = 0; i < song->maxtracks; i++) {
       if (!(i % 8))
-	fprintf(file, "\n");
+        fprintf(file, "\n");
       fprintf(file, "%2d: %3d  ", i + 1, song->tracks[i]->volume);
     }
 
@@ -904,9 +933,9 @@ void song_print(struct song *song, char *path)
 
       fprintf(file, "Playing Sequence List #%d (length = %d): %s", i + 1, playseq->length, playseq->name);
       for (j = 0; j < playseq->length; j++) {
-	if (!(j % 8))
-	  fprintf(file, "\n");
-	fprintf(file, "%3d ", playseq->blocknumbers[j] + 1);
+        if (!(j % 8))
+          fprintf(file, "\n");
+        fprintf(file, "%3d ", playseq->blocknumbers[j] + 1);
       }
       fprintf(file, "\n\n");
     }
@@ -929,16 +958,16 @@ void song_print(struct song *song, char *path)
       fprintf(file, "=================================\n");
 
       for (j = 0; j < block->length; j++) {
-	fprintf(file, "%.3d ", j);
-	for (k = 0; k < block->tracks; k++) {
-	  datastring[0] = instrumentstrings[block->notes[(block->tracks * j + k) * 2 + 1] & 0x3f];
-	  datastring[1] = instrumentstrings[(block->commands[(block->tracks * j + k) * 2] & 0xf0) >> 4];
-	  datastring[2] = instrumentstrings[block->commands[(block->tracks * j + k) * 2] & 0x0f];
-	  datastring[3] = instrumentstrings[(block->commands[(block->tracks * j + k) * 2 + 1] & 0xf0) >> 4];
-	  datastring[4] = instrumentstrings[block->commands[(block->tracks * j + k) * 2 + 1] & 0x0f];
-	  fprintf(file, "%s %s ", notestrings[block->notes[(block->tracks * j + k) * 2] & 0x7f], datastring);
-	}
-	fprintf(file, "\n");
+        fprintf(file, "%.3d ", j);
+        for (k = 0; k < block->tracks; k++) {
+          datastring[0] = instrumentstrings[block->notes[(block->tracks * j + k) * 2 + 1] & 0x3f];
+          datastring[1] = instrumentstrings[(block->commands[(block->tracks * j + k) * 2] & 0xf0) >> 4];
+          datastring[2] = instrumentstrings[block->commands[(block->tracks * j + k) * 2] & 0x0f];
+          datastring[3] = instrumentstrings[(block->commands[(block->tracks * j + k) * 2 + 1] & 0xf0) >> 4];
+          datastring[4] = instrumentstrings[block->commands[(block->tracks * j + k) * 2 + 1] & 0x0f];
+          fprintf(file, "%s %s ", notestrings[block->notes[(block->tracks * j + k) * 2] & 0x7f], datastring);
+        }
+        fprintf(file, "\n");
       }
 
       fprintf(file, "\n");
