@@ -1,6 +1,6 @@
-/* callbacks.c
+/* trackerwidget.c
  *
- * Copyright 2002-2006 Vesa Halttunen
+ * Copyright 2002-2011 Vesa Halttunen
  *
  * This file is part of Tutka. The code in this file is based on 
  * The Real SoundTracker - GTK+ Tracker widget
@@ -183,32 +183,6 @@ void tracker_set_pattern(Tracker * t, struct block *pattern)
   }
 }
 
-void tracker_set_backing_store(Tracker * t, int on)
-{
-  GtkWidget *widget;
-
-  g_return_if_fail(t != NULL);
-
-  if (on == t->enable_backing_store)
-    return;
-
-  t->enable_backing_store = on;
-
-  widget = GTK_WIDGET(t);
-  if (GTK_WIDGET_REALIZED(widget)) {
-    if (on) {
-      t->pixmap = gdk_pixmap_new(widget->window, widget->allocation.width, widget->allocation.height, -1);
-      CLEAR(t->pixmap, 0, 0, widget->allocation.width, widget->allocation.height);
-    } else {
-      gdk_pixmap_unref(t->pixmap);
-      t->pixmap = NULL;
-    }
-
-    gdk_gc_set_exposures(t->bg_gc, !on);
-    gtk_widget_queue_draw(GTK_WIDGET(t));
-  }
-}
-
 void tracker_set_xpanning(Tracker * t, int left_channel)
 {
   g_return_if_fail(t != NULL);
@@ -590,7 +564,7 @@ static void tracker_draw_clever(GtkWidget * widget, GdkRectangle * area)
   t = TRACKER(widget);
   g_return_if_fail(t->curpattern != NULL);
 
-  win = t->enable_backing_store ? (GdkWindow *) t->pixmap : widget->window;
+  win = t->pixmap;
 
   dist = t->patpos - t->oldpos;
   if (dist != 0) {
@@ -631,8 +605,7 @@ static void tracker_draw_clever(GtkWidget * widget, GdkRectangle * area)
   print_channel_headers(widget, win);
   print_bars(widget, win);
 
-  if (t->enable_backing_store)
-    gdk_draw_pixmap(widget->window, t->bg_gc, t->pixmap, area->x, area->y, area->x, area->y, area->width, area->height);
+  gdk_draw_pixmap(widget->window, t->bg_gc, t->pixmap, area->x, area->y, area->x, area->y, area->width, area->height);
 }
 
 static void tracker_draw_stupid(GtkWidget * widget, GdkRectangle * area)
@@ -699,12 +672,10 @@ static void init_display(Tracker * t, int width, int height)
     g_signal_emit(GTK_OBJECT(t), tracker_signals[XPANNING_SIGNAL], 0, t->leftchan, t->num_channels, t->disp_numchans);
   }
 
-  if (t->enable_backing_store) {
-    if (t->pixmap)
-      gdk_pixmap_unref(t->pixmap);
-    t->pixmap = gdk_pixmap_new(widget->window, widget->allocation.width, widget->allocation.height, -1);
-    CLEAR(t->pixmap, 0, 0, widget->allocation.width, widget->allocation.height);
-  }
+  if (t->pixmap)
+    gdk_pixmap_unref(t->pixmap);
+  t->pixmap = gdk_pixmap_new(widget->window, widget->allocation.width, widget->allocation.height, -1);
+  CLEAR(t->pixmap, 0, 0, widget->allocation.width, widget->allocation.height);
 }
 
 static void tracker_size_allocate(GtkWidget * widget, GtkAllocation * allocation)
@@ -737,6 +708,7 @@ void tracker_reset(Tracker * t)
     t->cursor_ch = 0;
     t->cursor_item = 0;
     t->leftchan = 0;
+    t->curpattern = NULL;
     init_display(t, widget->allocation.width, widget->allocation.height);
     adjust_xpanning(t);
     gtk_widget_queue_draw(GTK_WIDGET(t));
@@ -883,9 +855,6 @@ static void tracker_realize(GtkWidget * widget)
   gdk_gc_set_foreground(t->bg_cursor_gc, &t->colors[TRACKERCOL_BG_CURSOR]);
   gdk_gc_set_foreground(t->notes_gc, &t->colors[TRACKERCOL_NOTES]);
 
-  if (!t->enable_backing_store)
-    gdk_gc_set_exposures(t->bg_gc, 1);	/* man XCopyArea, grep exposures */
-
   init_display(t, attributes.width, attributes.height);
 
   gdk_window_set_user_data(widget->window, widget);
@@ -1106,7 +1075,6 @@ static void tracker_init(Tracker * t)
   t->fontdesc = pango_font_description_from_string("Monospace 9");
   t->oldpos = -1;
   t->curpattern = NULL;
-  t->enable_backing_store = 0;
   t->pixmap = NULL;
   t->patpos = 0;
   t->cursor_ch = 0;
